// Humax.cpp : Humax PVR9200T Media Controller
// (c) 2006 Andy Chappell
// Developed using MS Visual Studio 2005 in C


//////////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                             /////
//////////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
--------- ---- -----------------------------------------------------------------------
03-Jun-06 1.00 Initial release.
05-Jun-06 1.01 Fixed Settings options B and C and allow root dirspecs to be specified.
               Fixed problems using '/' and '\' character in Humax filenames.
05-Jun-06 1.02 Fixed incorrect free space report on list and tree options.

*****/

//////////////////////////////////////////////////////////////////////////////////////
///// INCLUDES                                                                   /////
//////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <windows.h>
#include <conio.h>
#include <sys/stat.h>
#include "usb.h"
#include "io.h"

//////////////////////////////////////////////////////////////////////////////////////
///// OTHER DEFINES                                                              /////
//////////////////////////////////////////////////////////////////////////////////////

#define CONFIG_FILE				"humax.cfg"
#define DUAL_READ_SLEEP_MS		800

//////////////////////////////////////////////////////////////////////////////////////
///// FUNCTION PROTOTYPES                                                        /////
//////////////////////////////////////////////////////////////////////////////////////

bool writeConfig();
bool readConfig();
bool writeConfigData(FILE * cfgFile, char * data, unsigned int dataLen);
bool readConfigData(FILE * cfgFile, char * data, unsigned int dataLen);

void doTitle();
void doMenu(usb_dev_handle * dh);
bool doTree(usb_dev_handle * dh);
void doSettings(usb_dev_handle * dh);
void doSettingsChangeHostDir(char * title, char * info, Entry * dir);
void doSettingsChangeDeviceDir(usb_dev_handle * dh, char * title, char * info, Entry * root, Entry * dir);
void doSettingsChangeBlockSize(char * title, char * info, int def, int min, int max, int * blockSize);
bool doTransfer(usb_dev_handle * dh);

int displayEntryTree(usb_dev_handle * dh, Entry * dir, bool displayDirs, bool displayFiles, bool displaySummary, bool displayIndexes, int level, __int64 * stats);

bool buildHostEntryTree(Entry * dir, bool includeDirs, bool includeFiles, char * filetypes);
bool buildHostEntryList(Entry * dir, bool includeDirs, bool includeFiles, char * filetypes);
int getFileType(char * filename, char * filetype);
Entry * findIndexedEntry(Entry * dir, int index);
int getInput(char * prompt, char * input, int maxLen);
bool strCompare(char * s1, char * s2);
bool processNumericList(char * input, int inputLen, bool * list, int listSize);

DWORD getProcessPriority();
bool setProcessPriority(int processPriority);

//////////////////////////////////////////////////////////////////////////////////////
///// GLOBAL VARIABLES                                                           /////
//////////////////////////////////////////////////////////////////////////////////////

char cwd[MAX_PATH];					// Current host working directory (where Humax.exe is located)
char baseHostDirMP3[MAX_PATH];		// Default host directory for MP3 file transfers
char baseHostDirImage[MAX_PATH];	// Default host directory for Video file transfers
char baseHostDirVideo[MAX_PATH];	// Default host directory for Image file transfers
char baseDeviceDirMP3[MAX_PATH];	// Default Humax directory for MP3 file transfers
char baseDeviceDirImage[MAX_PATH];	// Default Humax directory for Image file transfers
char baseDeviceDirVideo[MAX_PATH];	// Default Humax directory for Video file transfers
Entry curHostDirMP3;				// Current host directory for MP3 file transfers
Entry curHostDirImage;				// Current host directory for Image file transfers
Entry curHostDirVideo;				// Current host directory for Video file transfers
Entry curDeviceDirMP3;				// Current Humax directory for MP3 file transfers
Entry curDeviceDirImage;			// Current Humax directory for Image file transfers
Entry curDeviceDirVideo;			// Current Humax directory for Video file transfers
DWORD curProcessPriority;			// Stores the MS Windows current console process priority (MS Windows specific)

//////////////////////////////////////////////////////////////////////////////////////
///// MAIN PROGRAM ENTRY POINT                                                   /////
//////////////////////////////////////////////////////////////////////////////////////

// Console entry point
// Inputs
// - argc : number of supplied command-line arguments (supplied by console)
// - argv : array of string arguments (supplied by console)
// Returns
// - 0    : always
int main(int argc, char * argv[])
{
	// Display program title
	doTitle();
	printf("PLEASE READ THE RELEASE NOTES BEFORE USING THIS SOFTWARE\n\n");

	// Do initialisation including reading the config file
	doInit(argv);

	// Set desired process priority (MS Windows specific)
	curProcessPriority = getProcessPriority();
	bool setPriorityOK = setProcessPriority(processPriority);
	if(!setPriorityOK)
		printf("WARNING: could not set the process priority!\n\n");

	// Connect to the device
	usb_dev_handle * dh = humaxConnect();
	if(dh == NULL)
	{
		printf("ERROR: connecting to device!\n\n");
		return 0;
	}
	printf("Humax PVR9200T connected\n\n");

	// Display basic device info
	doInfo(dh, false);

	// Display main menu
	doMenu(dh);

	// Disconnect from Humax device
	humaxDisconnect(dh);
	printf("Humax PVR9200T disconnected\n");

	// Reset the process priority to previous (MS Windows specific)
	if(setPriorityOK)
		setProcessPriority(curProcessPriority);

	// Exit
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////
///// CONFIG FILE FUNCTIONS                                                      /////
//////////////////////////////////////////////////////////////////////////////////////

// Write settings to config file
// Returns
// - true  : if write config file succeeded
// - false : if write config file failed
bool writeConfig()
{
	// Open config file for writing
	FILE * cfgFile;
	if(fopen_s(&cfgFile, CONFIG_FILE, "wb") != 0)
		return false;

	// Write config data
	char buf[7];
	buf[0] = (unsigned char)downloadBlockSize;
	buf[1] = (unsigned char)uploadPrimaryBlockSize;
	buf[2] = (unsigned char)uploadRecoveryBlockSize;
	buf[3] = (uploadErrorRecovery ? 1 : 0);
	buf[4] = (usbDataTraceConsole ? 1 : 0);
	buf[5] = (usbDataTraceLogFile ? 1 : 0);
	buf[6] = (unsigned char)processPriority;
	bool ok =
		writeConfigData(cfgFile, baseHostDirMP3, MAX_PATH) &&
		writeConfigData(cfgFile, baseHostDirImage, MAX_PATH) &&
		writeConfigData(cfgFile, baseHostDirVideo, MAX_PATH) &&
		writeConfigData(cfgFile, baseDeviceDirMP3, MAX_PATH) &&
		writeConfigData(cfgFile, baseDeviceDirImage, MAX_PATH) &&
		writeConfigData(cfgFile, baseDeviceDirVideo, MAX_PATH) &&
		writeConfigData(cfgFile, buf, 7);

	// Close config file
	fclose(cfgFile);

	// Return status
	return ok;
}

// Read settings from config file
// Returns
// - true  : if read config file succeeded
// - false : if read config file failed
bool readConfig()
{
	// Open config file for reading
	FILE * cfgFile;
	if(fopen_s(&cfgFile, CONFIG_FILE, "rb") != 0)
		return false;

	// Initialise Entries
	initEntry(&curHostDirMP3, NULL, 0, 0);
	initEntry(&curHostDirImage, NULL, 0, 0);
	initEntry(&curHostDirVideo, NULL, 0, 0);
	initEntry(&curDeviceDirMP3, NULL, 0, 0);
	initEntry(&curDeviceDirImage, NULL, 0, 0);
	initEntry(&curDeviceDirVideo, NULL, 0, 0);

	// Read config data
	char buf[7];
	bool ok =
		readConfigData(cfgFile, baseHostDirMP3, MAX_PATH) &&
		readConfigData(cfgFile, baseHostDirImage, MAX_PATH) &&
		readConfigData(cfgFile, baseHostDirVideo, MAX_PATH) &&
		readConfigData(cfgFile, baseDeviceDirMP3, MAX_PATH) &&
		readConfigData(cfgFile, baseDeviceDirImage, MAX_PATH) &&
		readConfigData(cfgFile, baseDeviceDirVideo, MAX_PATH) &&
		readConfigData(cfgFile, buf, 7);

	if(ok)
	{
		strcpy_s(curHostDirMP3.path, baseHostDirMP3);
		strcpy_s(curHostDirImage.path, baseHostDirImage);
		strcpy_s(curHostDirVideo.path, baseHostDirVideo);
		strcpy_s(curDeviceDirMP3.path, baseDeviceDirMP3);
		strcpy_s(curDeviceDirImage.path, baseDeviceDirImage);
		strcpy_s(curDeviceDirVideo.path, baseDeviceDirVideo);
		downloadBlockSize = (unsigned char)buf[0];
		uploadPrimaryBlockSize = (unsigned char)buf[1];
		uploadRecoveryBlockSize = (unsigned char)buf[2];
		uploadErrorRecovery = (buf[3] != 0);
		usbDataTraceConsole = (buf[4] != 0);
		usbDataTraceLogFile = (buf[5] != 0);
		processPriority = (unsigned char)buf[6];
	}

	// Close config file
	fclose(cfgFile);

	// Return status
	return ok;
}

// Write data item to config file
// Inputs
// - cfgFile : config file handle
// - data    : char data to write to config file
// - dataLen : length of data to write to config file
// Returns
// - true    : write succeeded
// - false   : write failed
bool writeConfigData(FILE * cfgFile, char * data, unsigned int dataLen)
{
	unsigned int cbRet = fwrite(data, 1, dataLen, cfgFile);
	return (cbRet == dataLen);
}

// Read data item from config file
// Inputs
// - cfgFile : config file handle
// - data    : char data buffer to read data into
// - dataLen : length of data to read
// Returns
// - true    : read succeeded
// - false   : read failed
bool readConfigData(FILE * cfgFile, char * data, unsigned int dataLen)
{
	unsigned int cbRet = fread(data, 1, dataLen, cfgFile);
	return (cbRet == dataLen);
}
/* ADS
//////////////////////////////////////////////////////////////////////////////////////
///// INITIALISATION FUNCTIONS                                                   /////
//////////////////////////////////////////////////////////////////////////////////////
bool doInit(char * argv[]){}
Entry * createEntry(char * path, int nameLen, __int64 size){}
void initEntry(Entry * e, char * path, int nameLen, __int64 size){}
Block * createBlock(__int64 begByteNo, __int64 endByteNo, Block * curBlock, Block * nextBlock){}
void deleteEntryTree(Entry * dir){}
void deleteBlockList(Block * blocks){}
*/
//////////////////////////////////////////////////////////////////////////////////////
///// TITLE AND MENU FUNCTIONS                                                   /////
//////////////////////////////////////////////////////////////////////////////////////

// Display program title
void doTitle()
{
	printf("===============================================================================\n");
	printf("%s %s (%s)\n", PROGRAM_NAME, PROGRAM_VERSION, VERSION_DATE);
	printf("%s\n", COPYRIGHT);
	printf("===============================================================================\n\n");
}

// Display main menu
// Inputs
// - dh : USB device handle
void doMenu(usb_dev_handle * dh)
{
	// List of valid options
	char * options = "12CMRDXLTIFS?E\x1b";
	char * legend[] = { "1disk", "2disk", "Chdir", "Mkdir", "Rename", "Delete", "Xfer", "List", "Tree", "Info", "Format", "Settings", "Help", "Exit" };

	// Do main menu loop
	int choice = 0;
	while(choice != 'E')
	{
		// Display menu options
		printf("-------------------------------------------------------------------------------\n");
		printf("<1>disk  <2>disk  <C>hdir  <M>kdir  <R>ename  <D>elete\n");
		printf("<X>fer   <L>ist   <T>ree   <I>nfo   <F>ormat  <S>ettings\n");
		printf("-------------------------------------------------------------------------------\n");
		printf("Device Path: %s\n", getCurDeviceDir()->path);
		printf("-------------------------------------------------------------------------------\n");
		printf("Select option (E or ESCAPE to exit, ? for help): ");

		// Get user choice
		while(choice == 0)
		{
			choice = _getch();
			if(choice == 27)
				choice = 'E';
			if(choice >= 'a' && choice <= 'z')
				choice = choice & 0xdf;
			if(strFind(options, choice) == -1)
				choice = 0;
		}

		// Display chosen option
		printf("%s\n", legend[strFind(options, choice)]);
		printf("-------------------------------------------------------------------------------\n\n");

		// Process option
		if(choice != 'E')
		{
			switch(choice)
			{
				case '1':
					if(!doChangeDisk(dh, '1'))
						printf("ERROR: '1disk' option failed!\n\n");
					break;
				case '2':
					if(!doChangeDisk(dh, '2'))
						printf("ERROR: '2disk' option failed!\n\n");
					break;
				case 'C':
					if(!doChdir(dh))
						printf("ERROR: 'chdir' option failed!\n\n");
					break;
				case 'M':
					if(!doMkdir(dh))
						printf("ERROR: 'mkdir' option failed!\n\n");
					break;
				case 'R':
					if(!doRename(dh))
						printf("ERROR: 'rename' option failed!\n\n");
					break;
				case 'D':
					if(!doDelete(dh))
						printf("ERROR: 'delete' option failed!\n\n");
					break;
				case 'X':
					if(!doTransfer(dh))
						printf("ERROR: 'xfer' option failed!\n\n");
					break;
				case 'L':
					if(!doList(dh))
						printf("ERROR: 'list' option failed!\n\n");
					break;
				case 'T':
					if(!doTree(dh))
						printf("ERROR: 'tree' option failed!\n\n");
					break;
				case 'I':
					if(!doInfo(dh, true))
						printf("ERROR: 'info' option failed!\n\n");
					break;
				case 'F':
					if(!doFormat(dh))
						printf("ERROR: 'format' option failed!\n\n");
					break;
				case 'S':
					doSettings(dh);
					break;
				case '?':
					doHelp();
					break;
			}
			choice = 0;
		}

		//Delete entry trees
		deleteEntryTree(&deviceCurDir[0]);
		deleteEntryTree(&deviceCurDir[1]);
		deleteEntryTree(&curHostDirMP3);
		deleteEntryTree(&curHostDirImage);
		deleteEntryTree(&curHostDirVideo);
		deleteEntryTree(&curDeviceDirMP3);
		deleteEntryTree(&curDeviceDirImage);
		deleteEntryTree(&curDeviceDirVideo);
	}
}
/* ADS
bool doChangeDisk(usb_dev_handle * dh, char deviceDiskNo){}

bool doList(usb_dev_handle * dh){}
*/
// Do the tree option
// Inputs
// - dh    : USB device handle
// Returns
// - true  : operation succeeded
// - false : operation failed
bool doTree(usb_dev_handle * dh)
{
	// Get current directory and build directory tree
	Entry * dir = getCurDeviceDir();
	__int64 stats[6];
	if(buildDeviceEntryTree(dh, dir, 0, stats) == -1)
	{
		printf("ERROR: failed to build directory tree for '%s'!\n\n", dir->path);
		return false;
	}

	// Display tree
	displayEntryTree(dh, dir, true, true, true, false, 0, stats);

	// Return success
	return true;
}
/* ADS
bool doChdir(usb_dev_handle * dh){}
bool doRename(usb_dev_handle * dh){}
bool doDelete(usb_dev_handle * dh){}
bool doMkdir(usb_dev_handle * dh){}
bool doInfo(usb_dev_handle * dh, bool fullInfo){}
bool doFormat(usb_dev_handle * dh){}
*/
// Do the Settings option
// Inputs
// - dh    : USB device handle
void doSettings(usb_dev_handle * dh)
{
	// Display sub-menu
	char * options = "ABCDEFGHIJKLM\x0d";
	char input[MAX_PATH];
	Entry root;
	int inputLen = 0;
	int val = 0;
	int choice = 0;
	while(choice != 13)
	{
		// Display menu
		printf("  CHANGE FILE TRANSFER SETTINGS\n\n");

		printf("  [A] Default Host MP3 Directory           : %s\n", curHostDirMP3.path);
		printf("  [B] Default Host Image Directory         : %s\n", curHostDirImage.path);
		printf("  [C] Default Host Video Directory         : %s\n\n", curHostDirVideo.path);

		printf("  [D] Default Humax MP3 Directory          : %s\n", curDeviceDirMP3.path);
		printf("  [E] Default Humax Image Directory        : %s\n", curDeviceDirImage.path);
		printf("  [F] Default Humax Video Directory        : %s\n\n", curDeviceDirVideo.path);

		double downloadMaxRate = (12000000.0 / (8192.0 + (downloadBlockSize * 2048.0))) * (downloadBlockSize * 256.0);
		printf("  [G] Host-Humax Xfer Block Size           : %d (%.2f KB)\n", downloadBlockSize, (downloadBlockSize / 4.0));
		printf("      Host-Humax Xfer Max Theoretical Rate : %.2f KB/Sec\n\n", (downloadMaxRate / 1024.0));

		double uploadMaxRate = (12000000.0 / (8192.0 + (uploadPrimaryBlockSize * 2048.0))) * (uploadPrimaryBlockSize * 256.0);
		printf("  [H] Humax-Host Xfer Primary Block Size   : %d (%.2f KB)\n", uploadPrimaryBlockSize, (uploadPrimaryBlockSize / 4.0));
		printf("  [I] Humax-Host Xfer Recovery Block Size  : %d (%.2f KB)\n", uploadRecoveryBlockSize, (uploadRecoveryBlockSize / 4.0));
		printf("  [J] Humax-Host Xfer Error-Recovery       : %s\n", (uploadErrorRecovery ? "ENABLED" : "DISABLED"));
		printf("      Humax-Host Xfer Max Theoretical Rate : %.2f KB/Sec\n\n", (uploadMaxRate / 1024.0));

		printf("  [K] USB Data Trace -> Console            : %s\n", (usbDataTraceConsole ? "ENABLED" : "DISBALED"));
		printf("  [L] USB Data Trace -> Log File           : %s\n\n", (usbDataTraceLogFile ? "ENABLED" : "DISBALED"));

		printf("  [M] Transfer Process Priority            : %d (", processPriority);
		switch(processPriority)
		{
			case 0:
				printf("NORMAL");
				break;
			case 1:
				printf("ABOVE NORMAL");
				break;
			case 2:
				printf("HIGH");
				break;
			case 3:
				printf("REALTIME");
				break;
		}
		printf(")\n\n");

		// Get input from user
		printf("  Select choice (ENTER to end): ");
		choice = 0;
		while(choice == 0)
		{
			choice = _getch();
			if(choice >= 'a' && choice <= 'z')
				choice = choice & 0xdf;
			if(strFind(options, choice) == -1)
				choice = 0;
		}
		printf("%c\n\n", (char)choice);

		// Process user input
		switch(choice)
		{
			case 'A':
				doSettingsChangeHostDir(
					"Change Default Host MP3 Directory",
					"Sets the default Host directory where MP3 files are to be uploaded and\n  downloaded.",
					&curHostDirMP3);
				strcpy_s(baseHostDirMP3, curHostDirMP3.path);
				break;
			case 'B':
				doSettingsChangeHostDir(
					"Change Default Host Image Directory",
					"Sets the default Host directory where Image files are to be uploaded and\n  downloaded.",
					&curHostDirImage);
				strcpy_s(baseHostDirImage, curHostDirImage.path);
				break;
			case 'C':
				doSettingsChangeHostDir(
					"Change Default Host Video Directory",
					"Sets the default Host directory where Video files are to be uploaded and\n  downloaded.",
					&curHostDirVideo);
				strcpy_s(baseHostDirVideo, curHostDirVideo.path);
				break;
			case 'D':
				initEntry(&root, "1:/", 0, 0);
				doSettingsChangeDeviceDir(
					dh,
					"Change Default Humax MP3 Directory",
					"Sets the default Humax directory where MP3 files are to be uploaded and\n  downloaded.",
					&root,
					&curDeviceDirMP3);
				deleteEntryTree(&root);
				strcpy_s(baseDeviceDirMP3, curDeviceDirMP3.path);
				break;
			case 'E':
				initEntry(&root, "1:/", 0, 0);
				doSettingsChangeDeviceDir(
					dh,
					"Change Default Humax Image Directory",
					"Sets the default Humax directory where Image files are to be uploaded and\n  downloaded.",
					&root,
					&curDeviceDirImage);
				deleteEntryTree(&root);
				strcpy_s(baseDeviceDirImage, curDeviceDirImage.path);
				break;
			case 'F':
				initEntry(&root, "2:/", 0, 0);
				doSettingsChangeDeviceDir(
					dh,
					"Change Default Humax Video Directory",
					"Sets the default Humax directory where Video files are to be uploaded and\n  downloaded.",
					&root,
					&curDeviceDirVideo);
				deleteEntryTree(&root);
				strcpy_s(baseDeviceDirVideo, curDeviceDirVideo.path);
				break;
			case 'G':
				doSettingsChangeBlockSize(
					"Change Host-Humax Xfer Block Size",
					"Sets the download data block transfer size. Larger sizes produce faster\n  downloads.",
					160,
					1,
					252,
					&downloadBlockSize);
				break;
			case 'H':
				doSettingsChangeBlockSize(
					"Change Humax-Host Xfer Primary Block Size",
					"Sets the upload data block transfer size. This block size is used for initial\n  transfer attempts and subsequent seeking through already transferred parts\n  of a file. Larger sizes are faster but may also be more prone to errors.",
					240,
					1,
					252,
					&uploadPrimaryBlockSize);
				break;
			case 'I':
				doSettingsChangeBlockSize(
					"Change Humax-Host Xfer Recovery Block Size",
					"Sets the upload data block transfer size for resends of the corrupted parts\n  of files. Smaller block sizes are slower but can usually be more reliable\n  than larger sizes. Investigation has shown that the first 4 blocks of any\n  data block is usually transmitted without error so a block size of 4 is the\n  optimum setting.",
					4,
					1,
					252,
					&uploadRecoveryBlockSize);
				break;
			case 'J':
				uploadErrorRecovery = !uploadErrorRecovery;
				printf("  Humax-Host Xfer Error-Recovery %s\n\n", (uploadErrorRecovery ? "ENABLED" : "DISABLED"));
				printf("  -----------------------------------------------------------------------------\n\n");
				break;
			case 'K':
				usbDataTraceConsole = !usbDataTraceConsole;
				printf("  USB Data Trace -> Console %s\n\n", (usbDataTraceConsole ? "ENABLED" : "DISABLED"));
				printf("  -----------------------------------------------------------------------------\n\n");
				break;
			case 'L':
				usbDataTraceLogFile = !usbDataTraceLogFile;
				printf("  USB Data Trace -> Log File %s\n\n", (usbDataTraceLogFile ? "ENABLED" : "DISABLED"));
				printf("  -----------------------------------------------------------------------------\n\n");
				break;
			case 'M':
				printf("  -----------------------------------------------------------------------------\n\n");
				printf("  Change Transfer Process Priority\n\n");
				printf("  Sets the process priority for file transfers. The highest piority settings\n");
				printf("  can dramatically reduce the number of errors occurring during transfers.\n\n");
				printf("  Default Value : 3 (REALTIME)\n");
				printf("  Current Value : %d (", processPriority);
				switch(processPriority)
				{
					case 0:
						printf("NORMAL");
						break;
					case 1:
						printf("ABOVE NORMAL");
						break;
					case 2:
						printf("HIGH");
						break;
					case 3:
						printf("REALTIME");
						break;
				}
				printf(")\n");
				printf("   Valid Values : 0 = NORMAL\n");
				printf("                  1 = ABOVE NORMAL\n");
				printf("                  2 = HIGH\n");
				printf("                  3 = REALTIME\n");
				inputLen = getInput("  Set New Value : ", input, 256);
				printf("\n");
				if(inputLen > 0)
				{
					val = strtol(input, NULL, 10);
					if(val >= 0 && val <= 3)
					{
						if(!setProcessPriority(val))
							printf("WARNING: could not set the process priority!\n\n");
						else
							processPriority = val;
					}
					else
						printf("\n  ERROR: invalid value '%s'!\n\n", input);
				}
				printf("  -----------------------------------------------------------------------------\n\n");
				break;
		}

		// Update config file
		if(choice != 13 && !writeConfig())
			printf("ERROR: updating config file '%s'!\n\n", CONFIG_FILE);
	}
}
/* ADS
void doSettingsChangeBlockSize(char * title, char * info, int def, int min, int max, int * blockSize){}
*/
// Do the change Device directory setting
// Inputs
// - dh        : USB device handle
// - title     : Settings option title string
// - info      : Settings option summary info string
// - root      : current root directory setting
// - dir       : current option setting
// Returns
// - dir       : new option setting written back
void doSettingsChangeDeviceDir(usb_dev_handle * dh, char * title, char * info, Entry * root, Entry * dir)
{
	// Build directory tree
	char input[MAX_PATH];
	Entry * found = NULL;
	__int64 stats[6];
	int inputLen = 0;
	int val = 0;
	int dirCount = 0;
	bool valid = false;
	printf("  -----------------------------------------------------------------------------\n\n");
	printf("  %s\n\n", title);
	printf("  %s\n\n", info);
	if(buildDeviceEntryTree(dh, root, 0, stats) == -1)
	{
		printf("ERROR: failed to build device directory tree for '%s'!\n\n", root->path);
		return;
	}

	// Display entry tree
	dirCount = displayEntryTree(dh, root, true, false, false, true, 0, stats) + 1;

	// Get user input
	inputLen = getInput("  Enter directory number: ", input, 256);
	printf("\n");
	valid = true;
	if(inputLen > 0)
	{
		// User entered something so change the setting
		val = strtol(input, NULL, 10);
		if(val < 1 || val > dirCount)
			valid = false;
		else
		{
			found = findIndexedEntry(root, val);
			if(found == NULL)
				valid = false;
			else
				strcpy_s(dir->path, found->path);
		}
	}
	if(!valid)
		printf("ERROR: invalid entry '%s'!\n\n", input);
	printf("  -----------------------------------------------------------------------------\n\n");
}

// Do the change Host directory setting
// Inputs
// - title     : Settings option title string
// - info      : Settings option summary info string
// - dir       : current option setting
// Returns
// - dir       : new option setting written back
void doSettingsChangeHostDir(char * title, char * info, Entry * dir)
{
	// Display current value
	int inputLen = 0;
	char input[256];
	struct _stat64 inf;
	printf("  -----------------------------------------------------------------------------\n\n");
	printf("  %s\n\n", title);
	printf("  %s\n\n", info);
	printf("  Default Value : %s\n", cwd);
	printf("  Current Value : %s\n", dir->path);

	// Get user input
	inputLen = getInput("  Set New Value : ", input, MAX_PATH);
	printf("\n");
	if(inputLen > 0)
	{
		// User entered something so change the setting
		if(inputLen == 3 && ((input[0] >= 'a' && input[0] <= 'z') || (input[0] >= 'A' && input[0] <= 'Z')) && input[1] == ':' && input[2] == '\\')
			strcpy_s(dir->path, input);
		else
		{
			// Remove trailing '\' char if present
			if(input[inputLen - 1] == '\\')
				input[inputLen - 1] = 0;

			// If filespec does not exist then display error msg
			if(_stat64(input, &inf) != 0)
				printf("ERROR: directory '%s' is invalid!\n\n", input);
			else
			{
				// Valid path specified so store it
				strcpy_s(dir->path, input);
				strcat_s(dir->path, "\\");
			}
		}
	}
	printf("  -----------------------------------------------------------------------------\n\n");
}

// Do the file transfer option
// Inputs
// - dh    : USB device handle
// Returns
// - true  : operation succeeded
// - false : operation failed
bool doTransfer(usb_dev_handle * dh)
{
	// Display sub-menu
	char * options = "ABCDE\x0d";
	int fileCount = 0;
	int inputLen = 0;
	int choice = 0;
	while(choice != 13)
	{
		// Display transfer menu
		printf("  FILE TRANSFER MENU\n\n");
		printf("  [A] Video  Humax -> Host\n");
		printf("  [B] MP3    Host -> Humax\n");
		printf("  [C] MP3    Humax -> Host\n");
		printf("  [D] Image  Host -> Humax\n");
		printf("  [E] Image  Humax -> Host\n\n");
		printf("  Select choice (ENTER to end): ");

		// Get user input
		while(choice == 0)
		{
			choice = _getch();
			if(choice >= 'a' && choice <= 'z')
				choice = choice & 0xdf;
			if(strFind(options, choice) == -1)
				choice = 0;
		}
		printf("%c\n\n", (char)choice);

		// Process user selection
		switch(choice)
		{
			case 'A':
				doTransfersDeviceToHost(dh, "Video", baseDeviceDirVideo, &curDeviceDirVideo, baseHostDirVideo, &curHostDirVideo);
				break;
			case 'B':
				doTransfersHostToDevice(dh, "MP3", baseHostDirMP3, &curHostDirMP3, baseDeviceDirMP3, &curDeviceDirMP3, "*.mp3");
				break;
			case 'C':
				doTransfersDeviceToHost(dh, "MP3", baseDeviceDirMP3, &curDeviceDirMP3, baseHostDirMP3, &curHostDirMP3);
				break;
			case 'D':
				doTransfersHostToDevice(dh, "Image", baseHostDirImage, &curHostDirImage, baseDeviceDirImage, &curDeviceDirImage, "*.jpg");
				break;
			case 'E':
				doTransfersDeviceToHost(dh, "Image", baseDeviceDirImage, &curDeviceDirImage, baseHostDirImage, &curHostDirImage);
				break;
		}
		if(choice != 13)
		{
			printf("  -----------------------------------------------------------------------------\n\n");
			choice = 0;
		}
	}

	// Return success
	return true;
}

// Do file transfers from host to device
// Inputs
// - dh            : USB device handle
// - legend        : "Image", "MP3" or "Video"
// - baseSourceDir : Base host source directory
// - sourceDir     : Current host source directory
// - baseTargetDir : Base device target directory
// - targetDir     : Current device target directory
// - filetypes     : "*.mp3" or "*.jpg"
// Returns
// - true          : operation succeeded
// - false         : operation failed
bool doTransfersHostToDevice(usb_dev_handle * dh, char * legend, char * baseSourceDir, Entry * sourceDir, char * baseTargetDir, Entry * targetDir, char * filetypes)
{
	// Display host file list and get user input
	char name[MAX_PATH];
	char input[256];
	int fileCount = 0;
	int inputLen = 0;
	int dirCount = 0;
	__int64 stats[6];
	long val = 0;
	Entry source;
	Entry target;
	initEntry(&source, NULL, 0, 0);
	initEntry(&target, NULL, 0, 0);
	bool done = false;
	bool filter = true;
	while(!done)
	{
		// Clear existing entry trees
		deleteEntryTree(&source);
		deleteEntryTree(&target);
		initEntry(&source, sourceDir->path, 0, 0);
		initEntry(&target, targetDir->path, 0, 0);

		// Display transfer source and target dirs and file filter
		printf("  -----------------------------------------------------------------------------\n\n");
		printf("  Transfer %s files from Host to Humax\n\n", legend);
		printf("  [   A]  Host Source  %s\n", source.path);
		printf("  [   B] Humax Target  %s\n", target.path);
		printf("  [   C]  File Filter  %s\n\n", filter ? filetypes : "*.*");

		// Get source (host) directory contents
		if(!buildHostEntryList(&source, false, true, (filter ? filetypes : NULL)))
		{
			printf("ERROR: retrieving host directory contents for '%s'!\n\n", source.path);
			deleteEntryTree(&source);
			deleteEntryTree(&target);
			return false;
		}

		// Get target (device) directory contents
		if(setDeviceDirectory(dh, &target) == -1)
		{
			printf("ERROR: retrieving device directory contents for '%s'!\n\n", target.path);
			deleteEntryTree(&source);
			deleteEntryTree(&target);
			return false;
		}

		// List Host directory contents for user to select from
		fileCount = displayEntryList(dh, &source, false, true, false, false, true, stats);
		if(fileCount == 0)
			printf("  There are no %s files in the current Host directory to download!\n\n", legend);

		// Get file list input from user
		done = true;
		inputLen = getInput("  Specify files to transfer (? for help): ", input, 256);
		printf("\n");
		if(inputLen == 0)
		{
			// Nothing entered so just end
			deleteEntryTree(&source);
			deleteEntryTree(&target);
			return true;
		}

		// Check if special input (A, B, C or ?)
		if(input[0] == '?')
		{
			// Help requested so display it
			printf("  - Enter A to change the source directory.\n");
			printf("  - Enter B to change the target directory.\n");
			printf("  - Enter C to show %s%s files.\n", (filter ? "" : "only "), (filter ? "all" : legend));
			printf("  - Enter * to transfer all files.\n");
			printf("  - Enter index numbers of files to transfer (eg. 2,4,6-10)\n\n");
			done = false;
		}
		else if(input[0] == 'a' || input[0] == 'A')
		{
			// Change source directory requested
			printf("  -----------------------------------------------------------------------------\n\n");
			printf("  Change source directory\n\n");
			deleteEntryTree(&source);
			initEntry(&source, baseSourceDir, 0, 0);
			buildHostEntryTree(&source, true, false, NULL);
			dirCount = displayEntryTree(dh, &source, true, false, false, true, 0, stats);
			inputLen = getInput("  Specify directory: ", input, 256);
			printf("\n");
			if(inputLen > 0)
			{
				val = strtol(input, NULL, 10);
				Entry * found = findIndexedEntry(&source, val);
				if(found != NULL)
					strcpy_s(sourceDir->path, found->path);
			}
			deleteEntryTree(sourceDir);
			deleteEntryTree(&source);
			done = false;
		}
		else if(input[0] == 'b' || input[0] == 'B')
		{
			// Change target directory requested
			printf("  -----------------------------------------------------------------------------\n\n");
			printf("  Change target directory\n\n");
			deleteEntryTree(&target);
			initEntry(&target, baseTargetDir, 0, 0);
			buildDeviceEntryTree(dh, &target, 0, stats);
			dirCount = displayEntryTree(dh, &target, true, false, false, true, 0, stats);
			inputLen = getInput("  Specify directory: ", input, 256);
			printf("\n");
			if(inputLen > 0)
			{
				val = strtol(input, NULL, 10);
				Entry * found = findIndexedEntry(&target, val);
				if(found != NULL)
					strcpy_s(targetDir->path, found->path);
			}
			deleteEntryTree(targetDir);
			deleteEntryTree(&target);
			done = false;
		}
		else if(input[0] == 'c' || input[0] == 'C')
		{
			// Toggle file filter requested so toggle it (just a little bit)
			filter = !filter;
			printf("  File Filter %s\n\n", filter ? filetypes : "*.*");
			done = false;
		}
	}

	// Determine which files requested from user input
	bool * xferFile = new bool[fileCount];
	char * status = new char[fileCount];
	if(processNumericList(input, inputLen, xferFile, fileCount))
	{
		// Check if any files need overwriting
		Entry * e = source.firstChild;
		bool prompted = false;
		bool overwriteAll = false;
		bool end = false;
		while(!end && e != NULL)
		{
			// Is this a candidate file
			if(e->index != -1 && xferFile[e->index - 1])
			{
				// Get selected host filename and create the path of the target file on the device
				// truncating the filename to a max of 59 characters
				getFileName(e->path, e->nameLen, name);
				getFilePath(targetDir->path, 0, name, false, target.path);
				target.nameLen = getValidDeviceFilePath(target.path, e->nameLen);

				// Check the contents of the first 1KB of data of the host file AND
				// the file on the device (by doing a short transfer over USB).
				// Return statuses are:
				// - FILE_NOT_FOUND : host file does not exist on the Humax
				// - FILE_EQUAL     : host file exists on the Humax and has the same file contents (well first 1KB of data anyway!)
				// - FILE_DIFFERENT : host file exists on the Humax but has different file contents
				status[e->index - 1] = checkFile(dh, e, &target);

				// Process the file status
				if(status[e->index - 1] == FILE_DIFFERENT)
				{
					// The host file exists on the Humax but has different contents (first 1KB) so prompt user for overwrite
					if(!overwriteAll)
					{
						prompted = true;
						printf("  - Overwrite '%s' (Yes, No, All, End)? ", name);
						int choice = _getch();
						switch(choice)
						{
							case 'Y':
							case 'y':
								printf("YES\n");
								break;
							case 'A':
							case 'a':
								printf("ALL\n");
								overwriteAll = true;
								break;
							case 'E':
							case 'e':
								printf("END\n");
								end = true;
								break;
							case 'N':
							case 'n':
							default:
								printf("NO\n");
								xferFile[e->index - 1] = false;
								break;
						}
					}
				}
				else if(status[e->index - 1] == FILE_EQUAL)
				{
					// The source and target files are the same file so just display msg and exclude from the transfer
					printf("  - Already transferred '%s'\n", name);
					xferFile[e->index - 1] = false;
					prompted = true;
				}
			}

			// Do next entry
			e = e->sibling;
		}
		if(prompted)
			printf("\n");

		// Display list of files to transfer
		if(!end)
		{
			// Loop to display a list of files scheduled to be transferred
			e = source.firstChild;
			bool listed = false;
			while(e != NULL)
			{
				if(e->index != -1 && xferFile[e->index - 1])
				{
					if(!listed)
						printf("  The following files will be transferred to the device:\n\n");
					getFileName(e->path, e->nameLen, name);
					printf("  - '%s'\n", name);
					listed = true;
				}
				e = e->sibling;
			}
			if(listed)
			{
				printf("\n");
				printf("  -----------------------------------------------------------------------------\n\n");
			}

			// Finally loop for each file to transfer
			e = source.firstChild;
			bool end = false;
			while(!end && e != NULL)
			{
				if(e->index != -1 && xferFile[e->index - 1])
				{
					getFileName(e->path, e->nameLen, name);
					getFilePath(targetDir->path, 0, name, false, target.path);
					target.nameLen = getValidDeviceFilePath(target.path, e->nameLen);
					getFileInfo(dh, &target);
					end = !xferFileHostToDevice(dh, e, &target);
				}
				e = e->sibling;
			}
		}
	}
	else
		printf("ERROR: invalid entry '%s'!\n\n", input);
	free(xferFile);
	free(status);

	// Return success
	return true;
}

// Do file transfers from device to host
// Inputs
// - dh            : USB device handle
// - legend        : "Image", "MP3" or "Video"
// - baseSourceDir : Base device source directory
// - sourceDir     : Current device source directory
// - baseTargetDir : Base host target directory
// - targetDir     : Current host target directory
// Returns
// - true          : operation succeeded
// - false         : operation failed
bool doTransfersDeviceToHost(usb_dev_handle * dh, char * legend, char * baseSourceDir, Entry * sourceDir, char * baseTargetDir, Entry * targetDir)
{
	// Display device file list and get user input
	char name[MAX_PATH];
	char input[256];
	int fileCount = 0;
	int inputLen = 0;
	int dirCount = 0;
	__int64 stats[6];
	long val = 0;
	Entry source;
	Entry target;
	initEntry(&source, NULL, 0, 0);
	initEntry(&target, NULL, 0, 0);
	bool done = false;
	while(!done)
	{
		// Clear existing entry lists
		deleteEntryTree(&source);
		deleteEntryTree(&target);
		initEntry(&source, sourceDir->path, 0, 0);
		initEntry(&target, targetDir->path, 0, 0);

		// Display transfer source and target dirs
		printf("  -----------------------------------------------------------------------------\n\n");
		printf("  Transfer %s files from Humax to Host\n\n", legend);
		printf("  [   A] Humax Source  %s\n", source.path);
		printf("  [   B]  Host Target  %s\n\n", target.path);

		// Get source device directory contents
		if(setDeviceDirectory(dh, &source) == -1)
		{
			printf("ERROR: retrieving device directory contents for!\n\n", source.path);
			deleteEntryTree(&source);
			deleteEntryTree(&target);
			return false;
		}

		// Get target host directory contents
		if(!buildHostEntryList(&target, false, true, NULL))
		{
			printf("ERROR: retrieving host directory contents for '%s'!\n\n", target.path);
			deleteEntryTree(&source);
			deleteEntryTree(&target);
			return false;
		}

		// List Humax files for user to choose from
		fileCount = displayEntryList(dh, &source, false, true, false, false, true, stats);
		if(fileCount == 0)
			printf("  There are no %s files in the current Humax directory to upload!\n\n", legend);

		// Get input from user
		done = true;
		inputLen = getInput("  Specify files to transfer (? for help): ", input, 256);
		printf("\n");
		if(inputLen == 0)
		{
			// Nothing entered so just end
			deleteEntryTree(&source);
			deleteEntryTree(&target);
			return true;
		}

		// Check if special input
		if(input[0] == '?')
		{
			// Help requested so display it
			printf("  - Enter A to change the source directory.\n");
			printf("  - Enter B to change the target directory.\n");
			printf("  - Enter * to transfer all files.\n");
			printf("  - Enter index numbers of files to transfer (eg. 2,4,6-10)\n\n");
			done = false;
		}
		else if(input[0] == 'a' || input[0] == 'A')
		{
			// Change source directory requested
			printf("  -----------------------------------------------------------------------------\n\n");
			printf("  Change source directory\n\n");
			deleteEntryTree(&source);
			initEntry(&source, baseSourceDir, 0, 0);
			buildDeviceEntryTree(dh, &source, 0, stats);
			dirCount = displayEntryTree(dh, &source, true, false, false, true, 0, stats);
			inputLen = getInput("  Specify directory: ", input, 256);
			printf("\n");
			if(inputLen > 0)
			{
				val = strtol(input, NULL, 10);
				Entry * found = findIndexedEntry(&source, val);
				if(found != NULL)
					strcpy_s(sourceDir->path, found->path);
			}
			deleteEntryTree(sourceDir);
			deleteEntryTree(&source);
			done = false;
		}
		else if(input[0] == 'b' || input[0] == 'B')
		{
			// Change target directory requested
			printf("  -----------------------------------------------------------------------------\n\n");
			printf("  Change target directory\n\n");
			deleteEntryTree(&target);
			initEntry(&target, baseTargetDir, 0, 0);
			buildHostEntryTree(&target, true, false, NULL);
			dirCount = displayEntryTree(dh, &target, true, false, false, true, 0, stats);
			inputLen = getInput("  Specify directory: ", input, 256);
			printf("\n");
			if(inputLen > 0)
			{
				val = strtol(input, NULL, 10);
				Entry * found = findIndexedEntry(&target, val);
				if(found != NULL)
					strcpy_s(targetDir->path, found->path);
			}
			deleteEntryTree(targetDir);
			deleteEntryTree(&target);
			done = false;
		}
	}

	// Extract numbers from list
	bool * xferFile = new bool[fileCount];
	char * status = new char[fileCount];
	if(processNumericList(input, inputLen, xferFile, fileCount))
	{
		// Check if any files need overwriting
		Entry * e = source.firstChild;
		bool prompted = false;
		bool overwriteAll = false;
		bool end = false;
		while(!end && e != NULL)
		{
			// Check if candidate file
			if(e->index != -1 && xferFile[e->index - 1])
			{
				// Get selected device filename and create the path of the target file on the host
				// ensuring no invalid characters are present
				getFileName(e->path, e->nameLen, name);
				getFilePath(targetDir->path, 0, name, false, target.path);
				target.nameLen = getValidHostFilePath(target.path, e->nameLen);

				// Check the contents of the first 1KB of data of the host file AND
				// the file on the device (by doing a short transfer over USB).
				// Return statuses are:
				// - FILE_NOT_FOUND : Humax file does not exist on the host
				// - FILE_EQUAL     : Humax file exists on the host and has the same file contents (well first 1KB of data anyway!)
				// - FILE_DIFFERENT : Humax file exists on the host but has different file contents
				status[e->index - 1] = checkFile(dh, e, &target);

				// Process the file status
				if(status[e->index - 1] == FILE_DIFFERENT)
				{
					// Source file exists on the target but the file contents are different so prompt user for overwrite
					if(!overwriteAll)
					{
						prompted = true;
						printf("  - Overwrite '%s' (Yes, No, All, End)? ", name);
						int choice = _getch();
						switch(choice)
						{
							case 'Y':
							case 'y':
								printf("YES\n");
								break;
							case 'A':
							case 'a':
								printf("ALL\n");
								overwriteAll = true;
								break;
							case 'E':
							case 'e':
								printf("END\n");
								end = true;
								break;
							case 'N':
							case 'n':
							default:
								printf("NO\n");
								xferFile[e->index - 1] = false;
								break;
						}
					}
				}
			}
			e = e->sibling;
		}
		if(prompted)
			printf("\n");

		// Display list of files scheduled to be transferred
		if(!end)
		{
			// Display list of non-chase-transfer files to transfer
			bool listed = false;
			e = source.firstChild;
			while(e != NULL)
			{
				if(e->index != -1 && xferFile[e->index - 1] && !e->isRecording)
				{
					if(!listed)
						printf("  The following files will be transferred to the Host PC:\n\n");
					getFileName(e->path, e->nameLen, name);
					printf("  - '%s'\n", name);
					listed = true;
				}
				e = e->sibling;
			}

			// Display list of chase-transfer files to transferred after all non-chase-transfer files have been done
			e = source.firstChild;
			while(e != NULL)
			{
				if(e->index != -1 && xferFile[e->index - 1] && e->isRecording)
				{
					if(!listed)
						printf("  The following files will be transferred to the Host PC:\n\n");
					getFileName(e->path, e->nameLen, name);
					printf("  - '%s' [REC] [CHASE-XFER]\n", name);
					listed = true;
				}
				e = e->sibling;
			}
			if(listed)
			{
				printf("\n");
				printf("  -----------------------------------------------------------------------------\n\n");
			}

			// First transfer non-chase-transfer files. Loop for each file to transfer
			e = source.firstChild;
			bool end = false;
			while(!end && e != NULL)
			{
				// Check if candidate file
				if(e->index != -1 && xferFile[e->index - 1] && !e->isRecording)
				{
					// If target file exists and contents are different, then delete the target
					if(status[e->index - 1] == FILE_DIFFERENT)
					{
						target.size = 0;
						remove(target.path);
					}

					// Do the transfer
					getFileName(e->path, e->nameLen, name);
					getFilePath(targetDir->path, 0, name, false, target.path);
					target.nameLen = getValidHostFilePath(target.path, e->nameLen);
					getFileInfo(dh, &target);
					end = !xferFileDeviceToHost(dh, e, &target);
					xferFile[e->index - 1] = false;
				}

				// Do next entry
				e = e->sibling;
			}

			// Now scoop up any remaining chase-transfer files that need to be transferred
			e = source.firstChild;
			while(!end && e != NULL)
			{
				// Check if candidate file
				if(e->index != -1 && xferFile[e->index - 1])
				{
					// If target file exists and contents are different, then delete the target
					if(status[e->index - 1] == FILE_DIFFERENT)
					{
						target.size = 0;
						remove(target.path);
					}

					// Do the transfer
					getFileName(e->path, e->nameLen, name);
					getFilePath(targetDir->path, 0, name, false, target.path);
					target.nameLen = getValidHostFilePath(target.path, e->nameLen);
					getFileInfo(dh, &target);
					end = !xferFileDeviceToHost(dh, e, &target);
				}

				// Do next entry
				e = e->sibling;
			}
		}
	}
	else
		printf("ERROR: invalid entry '%s'!\n\n", input);
	free(xferFile);
	free(status);

	// Return success
	return true;
}

// Do the display help option
void doHelp()
{
	printf("  Chdir    : change directory on the device\n");
	printf("  Xfer     : transfer files to/from the device\n");
	printf("  Format   : format disk partitions on the device\n");
}
/* ADS
//////////////////////////////////////////////////////////////////////////////////////
///// SINGLE FILE USB TRANSFER FUNCTIONS                                         /////
//////////////////////////////////////////////////////////////////////////////////////
bool xferFileDeviceToHost(usb_dev_handle * dh, Entry * sourceFile, Entry * targetFile){}
bool xferFileHostToDevice(usb_dev_handle * dh, Entry * sourceFile, Entry * targetFile){}

//////////////////////////////////////////////////////////////////////////////////////
///// HIGH_LEVEL USB DEVICE FUNCTIONS                                            /////
//////////////////////////////////////////////////////////////////////////////////////
int setDeviceDirectory(usb_dev_handle * dh, Entry * dir){}
int buildDeviceEntryTree(usb_dev_handle * dh, Entry * dir, int level, __int64 * stats){}
char checkFile(usb_dev_handle * dh, Entry * sourceFile, Entry * targetDir){}
bool getFileInfo(usb_dev_handle * dh, Entry * file){}
int displayEntryList(usb_dev_handle * dh, Entry * dir, bool displayDirs, bool displayFiles, bool displayLinks, bool displaySummary, bool displayIndexes, __int64 * stats){}
*/
// Displays a tree of files and directories in a given directory returning the total number of entries displayed
// Inputs
// - dh             : USB device handle
// - dir            : top-level directory to build tree for
// - displayDirs    : true if directories are to be displayed
// - displayFiles   : true if files are to be displayed
// - displaySummary : true if header/footer summary info to be displayed
// - displayIndexes : true if selection index numbers are to be display
// - level          : always set to 0
// - stats          : array of 6 integers, value ignored on entry
// Returns
// - dir            : hierarchical linked-list file/directory tree
// - stats          : [0] tot entries [1] tot entries size [2] tot dirs [3] tot dirs size [4] tot files [5] tot files size
// - int            : total number of entries displayed (same as stats[0])
int displayEntryTree(usb_dev_handle * dh, Entry * dir, bool displayDirs, bool displayFiles, bool displaySummary, bool displayIndexes, int level, __int64 * stats)
{
	// Check if this is the top-level of the recursion tree
	__int64 diskSpace[2];
	char name[MAX_PATH];
	if(level == 0)
	{
		// Top-level so initialise stats
		for(int a = 0; a < 6; a++)
			stats[a] = 0;

		// Check if summary display requested
		if(displaySummary)
		{
			// Get free disk space
			if(!humaxGetDiskSpace(dh, diskSpace))
				printf("ERROR: getting disk space info for disk %c!\n", dir->path[0]);

			// Display header
			printf("  Tree of %s in '%s'\n\n",
				(displayDirs && !displayFiles ? "directories" :
					(!displayDirs && displayFiles ? "files" : "files and directories")),
				dir->path);
			printf("  %s%sName\n", (displayIndexes ? " Index " : ""), (displayDirs && !displayFiles ? "" : "        Size  "));
			printf("  %s%s-------------------------------------------------------%s%s\n", (displayIndexes ? "------ " : ""), (displayDirs && !displayFiles ? "" : "------------  "), (displayIndexes ? "" : "-------"), (displayDirs && !displayFiles ? "--------------" : ""));
		}

		// Display top-level directory filepath
		dir->index = 1;
		printf("  ");
		if(displayIndexes)
			printf("[%4d] ", (int)dir->index);
		printf("%s%s\n", (displayDirs && !displayFiles ? "" : "              "), dir->path);
		level++;
	}

	// Loop for all entries in the current directory level
	Entry * e = dir->firstChild;
	while(e != NULL)
	{
		// Check if file or directory
		bool isDir = isDirectory(e);
		if(displayDirs && isDir)
		{
			// Got a directory and directories were requested so display it
			printf("  ");
			stats[ENTRIES_COUNT]++;
			stats[ENTRIES_SIZE] = stats[ENTRIES_SIZE] + e->size;
			e->index = (int)stats[ENTRIES_COUNT] + 1;
			if(displayIndexes)
				printf("[%4d] ", (int)e->index);
			getFileName(e->path, e->nameLen, name);
			stats[DIRS_COUNT]++;
			stats[DIRS_SIZE] = stats[DIRS_SIZE] + e->size;
			printf("%s", (displayDirs && !displayFiles ? "" : "       <DIR>  "));
			for(int a = 0; a < level; a++)
				printf("  ");
			printf("%s%c\n", name, (dir->path[0] == '1' || dir->path[0] == '2' ? '/' : '\\'));
			displayEntryTree(dh, e, displayDirs, displayFiles, displaySummary, displayIndexes, level + 1, stats);
		}
		else if(displayFiles && !isDir)
		{
			// Got a file and files were requested so display it
			printf("  ");
			stats[ENTRIES_COUNT]++;
			stats[ENTRIES_SIZE] = stats[ENTRIES_SIZE] + e->size;
			e->index = (int)stats[ENTRIES_COUNT] + 1;
			if(displayIndexes)
				printf("[%4d] ", (int)e->index);
			getFileName(e->path, e->nameLen, name);
			stats[FILES_COUNT]++;
			stats[FILES_SIZE] = stats[FILES_SIZE] + e->size;
			printf("%9.2f MB  ", (double)e->size / 1048576.0);
			for(int a = 0; a < level; a++)
				printf("  ");
			printf("%s%s\n", name, (e->isRecording ? RECORDING_TAG : ""));
		}

		// Do next entry
		e = e->sibling;
	}

	// Is this the top-level
	if(level == 1)
	{
		// Display summary requested?
		if(displaySummary)
		{
			// Yes, display tree summary
			printf("  %s%s-------------------------------------------------------%s%s\n", (displayIndexes ? "------ " : ""), (displayDirs && !displayFiles ? "" : "------------  "), (displayIndexes ? "" : "-------"), (displayDirs && !displayFiles ? "--------------" : ""));
			if(displayDirs && !displayFiles)
				printf("  %sTotal %d dir%s, %d file%s (%.2f MB free)\n\n", (displayIndexes ? "       " : ""), (int)stats[DIRS_COUNT], (stats[DIRS_COUNT] == 1 ? "" : "s"), (int)stats[FILES_COUNT], (stats[FILES_COUNT] == 1 ? "" : "s"), (double)diskSpace[0] / 1048576.0);
			else
				printf("  %s%9.2f MB  Total %d dir%s, %d file%s (%.2f MB free)\n\n", (displayIndexes ? "       " : ""), (double)stats[FILES_SIZE] / 1048576.0, (int)stats[DIRS_COUNT], (stats[DIRS_COUNT] == 1 ? "" : "s"), (int)stats[FILES_COUNT], (stats[FILES_COUNT] == 1 ? "" : "s"), (double)diskSpace[0] / 1048576.0);
		}
		else
			printf("\n");
	}

	// Return total number of entries displayed
	return (int)stats[ENTRIES_COUNT];
}

//////////////////////////////////////////////////////////////////////////////////////
///// GENERAL UTILITY FUNCTIONS                                                  /////
//////////////////////////////////////////////////////////////////////////////////////

// Builds a host directory entry list
// Inputs
// - dir          : host directory path for which to get entries
// - includeDirs  : true if files are to be included
// - includeFiles : true if directories are to be included
// - filetypes    : "*.*", "*.jpg" or "*.mp3"
// Returns
// - dir          : contains a linked-list of child entries
// - true         : operation succeeded
// - false        : operation failed
bool buildHostEntryList(Entry * dir, bool includeDirs, bool includeFiles, char * filetypes)
{
	// Create host search path
	char path[MAX_PATH];
	char type[MAX_PATH];
	char name[MAX_PATH];
	char name2[MAX_PATH];
	getFilePath(dir->path, 0, "*.*", false, path);

	// Find first host file
	Entry * newEntry;
	struct __finddata64_t c_file;
	intptr_t hFile;
	hFile = _findfirst64(path, &c_file);
	if(hFile == -1)
		return false;

	// Loop for each entry in the host directory
	int ret = 0;
	while(ret == 0)
	{
		// Check if required entry
		bool isDir = ((c_file.attrib & _A_SUBDIR) != 0);
		getFileType(c_file.name, type);
		bool isValidType = (filetypes == NULL || (strlen(type) != 0 && strstr(filetypes, type) != NULL));
		if(((includeDirs && isDir) || (includeFiles && !isDir)) && strcmp(c_file.name, ".") != 0 && strcmp(c_file.name, "..") != 0 && isValidType)
		{
			// Create new entry
			getFilePath(dir->path, 0, c_file.name, false, path);
			if(isDir)
				strcat_s(path, "\\");
			newEntry = createEntry(path, (isDir ? 0 : strlen(c_file.name)), c_file.size);

			// Insertion sort the entry into the current list, directories first then files, sorted in ascending
			// ASCII alphabetical order
			Entry * prevEntry = NULL;
			Entry * curEntry = dir->firstChild;
			if(curEntry == NULL)
			{
				// First entry so ceate as a child of the current parent directory
				dir->firstChild = newEntry;
			}
			else
			{
				// Get name of new entry
				getFileName(newEntry->path, newEntry->nameLen, name);
				strLowerCase(name);

				// Now find the correct place to insert the new entry
				bool done = false;
				while(!done && curEntry != NULL)
				{
					// Get entry name
					getFileName(curEntry->path, curEntry->nameLen, name2);
					strLowerCase(name2);
					bool isDir2 = isDirectory(curEntry);

					// Have we found the correct place, directories first, files last, sorted ASCII alphabetical, ascending
					done = (isDir && !isDir2) || (((isDir && isDir2) || (!isDir && !isDir2)) && strcmp(name, name2) < 0);
					if(!done)
					{
						// Not found correct location so check next entry
						prevEntry = curEntry;
						curEntry = curEntry->sibling;
					}
				}

				// Link entry into existing list
				Entry * temp = NULL;
				if(prevEntry == NULL)
				{
					// New entry is first in the list
					temp = dir->firstChild;
					dir->firstChild = newEntry;
				}
				else
				{
					// New entry is not first in list
					temp = prevEntry->sibling;
					prevEntry->sibling = newEntry;
				}
				newEntry->sibling = temp;
			}
		}

		// Get next host directory entry
		ret = _findnext64(hFile, &c_file);
	}

	// End file search
	_findclose(hFile);

	// Return success
	return true;
}
/* ADS
void buildDeviceEntryList(char * data, bool * sizeChanged, int noEntries, Entry * dir){}
*/
// Builds a complete host entry tree in memory from the current directory downwords
// Inputs
// - dir          : directory to build the host file/directory tree for
// - includeDirs  : true if directories are to be included
// - includeFiles : true if files are to be included
// - filetypes    : "*.", "*.jpg" or "*.mp3"
// Returns
// - dir          : hierarchical linked-list tree of child and sibling entries containing the tree structure
// - true         : operation succeeded
// - false        : operation failed
bool buildHostEntryTree(Entry * dir, bool includeDirs, bool includeFiles, char * filetypes)
{
	// Get current host directory list
	if(!buildHostEntryList(dir, includeDirs, includeFiles, filetypes))
		return false;

	// Loop for each entry in the current directory
	Entry * e = dir->firstChild;
	while(e != NULL)
	{
		// If current entry is a directory then recurse to next level
		if(isDirectory(e) && !buildHostEntryTree(e, includeDirs, includeFiles, filetypes))
			return false;

		// Do next entry
		e = e->sibling;
	}

	// Return success
	return true;
}
/* ADS
bool isXferComplete(__int64 fileSize, Block *blocks){}

__int64 xferGetRecoveryByteCount(Block * blocks){}

__int64 xferGetGoodByteCount(Block * blocks){}

Entry * getCurDeviceDir(){}

bool isDirectory(Entry * e){}

bool isRootDirectory(Entry * e){}

int getFilePath(char *path, int nameLen, char *newName, bool forceParent, char *filepath){}

int getValidDeviceFilePath(char * filepath, int nameLen){}

int getValidHostFilePath(char * filepath, int nameLen){}

int getFileName(char * path, int nameLen, char * filename){}
*/
// Extracts just the filetype from a file or directory and returns it in lower-case
// Inputs
// - filename : filename only
// - filetype : buffer to receive filetype, ignored on entry
// Returns
// - filetype : filetype in lower-case
// - int      : length of filetype string
int getFileType(char * filename, char * filetype)
{
	int filenameLen = strlen(filename);
	if(filenameLen == 0)
	{
		filetype[0] = 0;
		return 0;
	}
	int p = filenameLen - 1;
	while(p > 0 && filename[p] != '.')
		p--;
	if(p == 0)
		filetype[0] = 0;
	else
	{
		memcpy(filetype, filename + p + 1, filenameLen - p);
		filetype[filenameLen - p] = 0;
	}
	return strLowerCase(filetype);
}

// Locates a file or directory entry with the given index and returns it
// Inputs
// - dir   : top-level directory to begin index search
// - index : index number to search for
// Returns
// - Entry : the Entry if index found or NULL if index not found
Entry * findIndexedEntry(Entry * dir, int index)
{
	// If found at top-level then return it
	if(dir->index == index)
		return dir;

	// Loop for all entries in the directory level
	Entry * e = dir->firstChild;
	while(e != NULL)
	{
		// If found the index then return the entry
		if(e->index == index)
			return e;

		// Check if entry is a directory
		if(isDirectory(e))
		{
			// Recurse down to next level
			Entry * found = findIndexedEntry(e, index);
			if(found != NULL)
				return found;
		}

		// Do next entry
		e = e->sibling;
	}

	// Index entry not found so return NULL
	return NULL;
}

// Get text input from the user, strip off leading and trailing spaces, and return the resulting input length
// Inputs
// - prompt : prompt to display to user
// - input  : input text buffer (set on exit)
// - maxLen : maximum length of input text buffer
// Returns
// - input  : input text buffer returned on exit
// - int    : length of input entered by the user
int getInput(char * prompt, char * input, int maxLen)
{
	// Prompt user
	printf("%s", prompt);

	// Get text input
	memset(input, maxLen, 0);
	fgets(input, maxLen, stdin);

	// Strip leading and trailing whitespace
	int p = 0;
	while(p < maxLen - 1)
	{
		if(input[p] < ' ')
			break;
		p++;
	}
	int beg = 0;
	while(beg < p && input[beg] == ' ')
		beg++;
	int end = p - 1;
	while(end > 0 && input[end] == ' ')
		end--;
	end++;
	if(end > beg)
		memcpy(input, input + beg, end - beg);
	input[end - beg] = 0;

	// Return resulting input length
	return strlen(input);
}
/* ADS
int strFind(char * s, int c){}
*/
// Compare two strings (ignoring alphabetic case) returning true if equal
// Inputs
// - s1    : input string 1 to compare
// - s2    : input string 2 to compare with s1
// Returns
// - true  : strings are the same (ignoring case)
// - false : string are different
bool strCompare(char * s1, char * s2)
{
	unsigned int len = strlen(s1);
	if(strlen(s2) < len)
		len = strlen(s2);
	for(unsigned int a = 0; a < len; a++)
		if(toLowerCase(s1[a]) != toLowerCase(s2[a]))
			return false;
	return true;
}
/* ADS
int strLowerCase(char * s){}

char toLowerCase(char c){}

int incCBWTag(){}
*/
// Takes a numeric list (including ranges) as input by the user and returns an array of
// booleans with true/false values
// Inputs
// - input    : text input
// - inputLen : length of input text
// - list     : boolean array of 'listSize' entries (input values ignored)
// - listSize : size of list
// Returns
// - list     : array of booleans indicating true if selected, false if not
// - true     : if input is valid
// - false    : if input is invalid
bool processNumericList(char * input, int inputLen, bool * list, int listSize)
{
	// Initialise boolean array
	bool valid = true;
	for(int a = 0; a < listSize; a++)
		list[a] = false;
	bool all = false;
	int end = 0;

	// If '*' specified as first character then set all entries to true
	if(inputLen == 1 && input[0] == '*')
	{
		all = true;
		for(int p = 0; p < listSize; p++)
			list[p] = true;
	}
	else
	{
		// Process text list separated by commas or spaces
		while(valid && end < inputLen)
		{
			// Get next token
			int beg = end;
			while(beg < inputLen && (input[beg] == ' ' || input[beg] == ','))
				beg++;
			end = beg;
			while(end < inputLen && input[end] != ' ' && input[end] != ',')
				end++;

			// Extract number
			int numa = 0;
			int numb = 0;
			int p = beg;
			while(p < end && input[p] >= '0' && input[p] <= '9')
				numa = numa * 10 + (input[p++] - '0');

			// Check if valid range value specified
			if(p < end)
			{
				if(input[p] != '-')
					valid = false;
				else
				{
					p++;
					while(p < end && input[p] >= '0' && input[p] <= '9')
						numb = numb * 10 + (input[p++] - '0');
				}
			}
			else
				numb = numa;

			// Check if valid token
			valid = (p < end ? false : (numa <= numb && numa >= 1 && numa <= listSize && numb >= 1 && numb <= listSize));

			// If finally got valid token set respective entries true in the boolean array
			if(valid)
				for(int p = numa; p <= numb; p++)
					list[p - 1] = true;
		}
	}

	// Return validity status
	return valid;
}

//////////////////////////////////////////////////////////////////////////////////////
///// USB TRACING FUNCTIONS                                                      /////
//////////////////////////////////////////////////////////////////////////////////////
/* ADS
void usbTrace(bool usbDataTraceConsole, bool usbDataTraceLogFile, bool status, char * func, int id, char * msg, char * data, int dataLen){}

void writeTraceInfo(FILE * logFile, bool usbDataTraceConsole, bool usbDataTraceLogFile, char * buf){}
*/
//////////////////////////////////////////////////////////////////////////////////////
///// MS WINDOWS SPECIFIC PROCESS PRIORITY FUNCTIONS                             /////
//////////////////////////////////////////////////////////////////////////////////////

// Gets the current MS Windows process priority (MS Windows specific)
// Returns
// - DWORD : contains the MS Windows specific ID of the current process priority
DWORD getProcessPriority()
{
	return GetPriorityClass(GetCurrentProcess());
}

// Sets the MS Windows process priority returning true if operation succeeded (MS Windows specific)
// Inputs
// - processPriority : [0] Normal [1] Above Normal [2] High [3] Realtime
// Returns
// - true            : operation succeeded
// - false           : operation failed
bool setProcessPriority(int processPriority)
{
	// Set process priority
	long priority = NORMAL_PRIORITY_CLASS;
	switch(processPriority)
	{
		case 0:
			priority = NORMAL_PRIORITY_CLASS;
			break;
		case 1:
			priority = ABOVE_NORMAL_PRIORITY_CLASS;
			break;
		case 2:
			priority = HIGH_PRIORITY_CLASS;
			break;
		case 3:
			priority = REALTIME_PRIORITY_CLASS;
			break;
	}

	// Return true if successfully set
	return (SetPriorityClass(GetCurrentProcess(), priority) != 0);
}
/* ADS
//////////////////////////////////////////////////////////////////////////////////////
///// HUMAX USB BULK TRANSFER COMMAND INTERFACE FUNCTIONS                        /////
//////////////////////////////////////////////////////////////////////////////////////
int humaxGetNoDirEntries(usb_dev_handle * dh){}
bool humaxGetDirEntries(usb_dev_handle * dh, int noEntries, bool doDualRead, char * data, bool * sizeChanged){}
bool humaxGetDiskSpace(usb_dev_handle * dh, __int64 * diskSpace){}
bool humaxXferBeginDevToHost(usb_dev_handle * dh, int dCBWTag, char * filePath){}
bool humaxXferBeginHostToDev(usb_dev_handle * dh, int dCBWTag, char * filePath, __int64 fileSize){}
bool humaxXferEnd(usb_dev_handle * dh, int dCBWTag, char * filePath){}
int humaxXferNextDevToHost(usb_dev_handle * dh, int dCBWTag, char * data, int noBytes){}
bool humaxXferNextHostToDev(usb_dev_handle * dh, int dCBWTag, char * data, int noBytes){}
bool humaxRenameFile(usb_dev_handle * dh, char * oldFilePath, char * newFilePath){}
bool humaxCreateDir(usb_dev_handle * dh, char * dirPath){}
bool humaxDeleteFile(usb_dev_handle * dh, char * filePath){}
bool humaxFormatDiskOne(usb_dev_handle * dh){}
bool humaxSetDirectory(usb_dev_handle * dh, char * dirPath){}
int humaxGetNoPartitions(usb_dev_handle * dh){}
bool humaxSetDiskNo(usb_dev_handle * dh, char diskNo){}
*/
