////////////////////////////////////////////////////////////////////////////////
/// \class DeviceInfoDialog DeviceInfoDialog.h
///
/// \version 1.00
/// \date 04-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// See COPYING file that comes with this distribution.
///
/// \brief Humax device information dialog Class.
///
/// A Dialog class used to display information downloaded from the Humax.
/// Including Humax hardware information and Disk/File Information.
///
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#ifndef DEVICEINFODIALOG_H
#define DEVICEINFODIALOG_H

#include "ui_DeviceInfoDialog.h"

#include <QDialog>

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class DeviceInfoDialog : public QDialog
{    
	Q_OBJECT
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Device Information Dialog Class.
	 */
	DeviceInfoDialog(QWidget *parent = 0);

	/*! \brief Set the Manufacturer text in the dialog.
	 *
	 * Sets the manufacturer field displayed on the information dialog with
	 * the text provided.
	 * \param manufacturer The text to be displayed.
	 */
	inline void setManufacturer(const QString &manufacturer)
		{ ui.mpManufacturerEdit->setText(manufacturer); }

	/*! \brief Set the Product Name text in the dialog.
	 *
	 * Sets the Product Name field displayed on the information dialog with
	 * the text provided.
	 * \param productName The text to be displayed.
	 */
	inline void setProductName(const QString &productName)
		{ ui.mpProductNameEdit->setText(productName); }

	/*! \brief Set the Serial Number text in the dialog.
	 *
	 * Sets the Serial Number field displayed on the information dialog with
	 * the text provided.
	 * \param serialNo The text to be displayed.
	 */
	inline void setSerialNo(const QString &serialNo)
		{ ui.mpSerialNoEdit->setText(serialNo); }

	/*! \brief Set the Hardware Version text in the dialog.
	 *
	 * Sets the Hardware Version field displayed on the information dialog with
	 * the text provided.
	 * \param hardwareVersion The text to be displayed.
	 */
	inline void setHardwareVerson(const QString &hardwareVersion)
		{ ui.mpHardwareVersonEdit->setText(hardwareVersion); }

	/*! \brief Set the USB Version text in the dialog.
	 *
	 * Sets the USB Version field displayed on the information dialog with
	 * the text provided.
	 * \param usbVersion The text to be displayed.
	 */
	inline void setUsbVersion(const QString &usbVersion)
		{ ui.mpUsbVersionEdit->setText(usbVersion); }

	/*! \brief Set the USB Endpoints text in the dialog.
	 *
	 * Sets the USB Endpoints field displayed on the information dialog with
	 * the text provided.
	 * \param usbEndpoints The text to be displayed.
	 */
	inline void setUsbEndpoints(const QString &usbEndpoints)
		{ ui.mpUsbEndpointsEdit->setText(usbEndpoints); }

	/*! \brief Set the disk information visible/invisible.
	 *
	 * Shows or Hides the area of the dialog displaying the disk information.
	 * Default is the area is shown.
	 * \param show Boolean flag to show/hide the disk information.
	 */
	void showDiskInfo(const bool &show);

	/*! \brief Update the Disk One Space information.
	 *
	 * Updates the disk one space information, including the total, free, and
	 * used space as well as the Disk Bar.
	 * \param free	The amount of free space on the disk.
	 * \param total	The total amount of space on the disk.
	 */
	void setDiskOneSpace(const qint64 &free, const qint64 &total);

	/*! \brief Updates the Disk One Directory Information.
	 *
	 * Updates the information showing the total size of the directories on
	 * disk one and the number of directories.
	 * \param size	The total size (in bytes) to be displayed.
	 * \param count	The number of directories on the disk.
	 */
	void setDiskOneDirectories(const qint64 &size, const qint64 &count);

	/*! \brief Updates the Disk One File Information.
	 *
	 * Updates the information showing the total size of the files on
	 * disk one and the number of files.
	 * \param size	The total size (in bytes) to be displayed.
	 * \param count	The number of files on the disk.
	 */
	void setDiskOneFiles(const qint64 &size, const qint64 &count);

	/*! \brief Updates the Disk One Entry (Files and Directories) Information.
	 *
	 * Updates the information showing the total size of the items (Files and
	 * Directories) on disk one and the number of items.
	 * \param size	The total size (in bytes) to be displayed.
	 * \param count	The number of items on the disk.
	 */
	void setDiskOneEntries(const qint64 &size, const qint64 &count);

	/*! \brief Update the Disk Two Space information.
	 *
	 * Updates the disk two space information, including the total, free, and
	 * used space as well as the Disk Bar.
	 * \param free	The amount of free space on the disk.
	 * \param total	The total amount of space on the disk.
	 */
	void setDiskTwoSpace(const qint64 &free, const qint64 &total);

	/*! \brief Updates the Disk Two Directory Information.
	 *
	 * Updates the information showing the total size of the directories on
	 * disk two and the number of directories.
	 * \param size	The total size (in bytes) to be displayed.
	 * \param count	The number of directories on the disk.
	 */
	void setDiskTwoDirectories(const qint64 &size, const qint64 &count);

	/*! \brief Updates the Disk Two File Information.
	 *
	 * Updates the information showing the total size of the files on
	 * disk two and the number of files.
	 * \param size	The total size (in bytes) to be displayed.
	 * \param count	The number of files on the disk.
	 */
	void setDiskTwoFiles(const qint64 &size, const qint64 &count);

	/*! \brief Updates the Disk Two Entry (Files and Directories) Information.
	 *
	 * Updates the information showing the total size of the items (Files and
	 * Directories) on disk one and the number of items.
	 * \param size	The total size (in bytes) to be displayed.
	 * \param count	The number of items on the disk.
	 */
	void setDiskTwoEntries(const qint64 &size, const qint64 &count);

	/*! \brief Updates the current recodring information.
	 *
	 * Updates the information showing which video files are currently
	 * being recorded.
	 * \param recordingStatus Currently recording files.
	 */
	inline void setRecordingStatus(const QString &recordingStatus)
		{ ui.mpRecordingStatusEdit->setText(recordingStatus); }

private:
	/*! \brief Device Information Dialog widget information.
	 *
	 * Contains all of the information/widgets needed to created the dialog.
	 * This information is stored in DeviceInfoDialog.ui and is auto-generated by Qt.
	 */
	Ui::DeviceInfoDialog ui;
};

#endif // DEVICEINFODIALOG_H
////////////////////////////////////////////////////////////////////////////////
