////////////////////////////////////////////////////////////////////////////////
/// \class FileReplaceDialog FileReplaceDialog.h
///
/// \version 1.00
/// \date 12-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// See COPYING file that comes with this distribution.
///
/// \brief Replace File(s) Dialog Class.
///
/// Replace file(s) dialog to request whether the application should overwrite
/// files when needed.
///
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
12-07-06 1.00 Initial release.

*****/
#ifndef FILEREPLACEDIALOG_H
#define FILEREPLACEDIALOG_H

#include "ui_FileReplaceDialog.h"

#include <QDialog>

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class FileReplaceDialog : public QDialog
{    
	Q_OBJECT
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Replace File(s) Dialog Class.
	 */
	FileReplaceDialog(QWidget *parent = 0);

	/*! \brief Set the displayed filename.
	 *
	 * Sets the filename to be displayed in the file replace dialog.
	 * \param filename The name of the file.
	 */
	void SetFilename(const QString &filename);

	/*! \brief Set the target file information.
	 *
	 * Sets the target file size to be displayed on the dialog.
	 * \param size The size (in bytes) of the target file.
	 */
	void SetTargetInfo(qint64 size);

	void SetTargetInfo(QString size);

	/*! \brief Set the source file information.
	 *
	 * Sets the source file size to be displayed on the dialog.
	 * \param size The size (in bytes) of the source file.
	 */
	void SetSourceInfo(qint64 size);

	void SetSourceInfo(QString size);

	/*! \brief Set display to single file mode.
	 *
	 * If only one (single) file is being copied then there is no need for a
	 * 'Yes To All' button or a Cancel button. Setting the dialog to single 
	 * file mode hides these buttons to give an Overwrite? Yes No dialog.
	 * \param single boolean flag, true if single file mode.
	 */
	void SetSingleFileReplace(const bool &single);

	/*! \brief Indicated which button was pressed to close the dialog.
	 *
	 * When a button is pressed on the dialog the return flag mReturnState is
	 * set to the enumerator associated with that button. The dialog is then
	 * closed and exec() returns the enumerator assigned to the mReturnState
	 * flag.
	 * \sa mReturnState and exec().
	 */
	enum ReturnState
	{
		CANCEL,		/*!< Cancel Button pressed. */
		YES,		/*!< Yes Button pressed. */
		YESTOALL,	/*!< Yes To All Button pressed. */
		NO			/*!< No Button pressed. */
	};

public slots:
	/*! \brief Called when the user presses the Yes button.
	 *
	 * This function is called when the Yes button is pressed, it sets
	 * the mReturnState flag to YES then closes the dialog.
	 * \sa ReturnState and mReturnState.
	 */
	void on_mpYesButton_clicked();

	/*! \brief Called when the user presses the Yes To All button.
	 *
	 * This function is called when the Yes To All button is pressed, it sets
	 * the mReturnState flag to YESTOALL then closes the dialog.
	 * \sa ReturnState and mReturnState.
	 */
	void on_mpYesToAllButton_clicked();

	/*! \brief Called when the user presses the No button.
	 *
	 * This function is called when the No button is pressed, it sets
	 * the mReturnState flag to NO then closes the dialog.
	 * \sa ReturnState and mReturnState.
	 */
	void on_mpNoButton_clicked();

	/*! \brief Called when the user presses the Cancel button.
	 *
	 * This function is called when the Cancel button is pressed, it sets
	 * the mReturnState flag to CANCEL then closes the dialog.
	 * \sa ReturnState and mReturnState.
	 */
	void on_mpCancelButton_clicked();

	/*! \brief Opens the dialog modally and returns the result.
	 *
	 * The is a standard Qt QDialog function which has been overridden.
	 * The function opens the dialog modally and waits for it to be closed
	 * by calling the overridden QDialog::exec(). When this returns, this
	 * function substitutes the return value with the return state set by the
	 * closing button.
	 * \return The return state depending on which button is pressed.
	 * \sa ReturnState and mReturnState.
	 */
	int exec();

private:
	/*! \brief Replace File Dialog widget information.
	 *
	 * Contains all of the information/widgets needed to created the dialog.
	 * This information is stored in FileReplaceDialog.ui and is auto-generated by Qt.
	 */
	Ui::FileReplaceDialog ui;

	/*! \brief The return value of the modal dialog.
	 *
	 * The return state value of the dialog when opened modally via exec().
	 * \sa ReturnState and exec().
	 */
	ReturnState mReturnState;
};

#endif // FILEREPLACEDIALOG_H
////////////////////////////////////////////////////////////////////////////////
