////////////////////////////////////////////////////////////////////////////////
/// \class HumaxController HumaxController.h
///
/// \version 0.99a
/// \date 04-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// See COPYING file that comes with this distribution.
///
/// \brief \b ** 
///
/// --
///
////////////////////////////////////////////////////////////////////////////////
#ifndef HUMAXCONTROLLER_H
#define HUMAXCONTROLLER_H

#include "usb.h"
#include "HumaxInterface.h"
#include <QStringList>
#include <QThread>
#include <QTime>
#include <QVariant>

////////////////////////////////////////////////////////////////////////////////
///// OTHER DEFINES                                                        /////
////////////////////////////////////////////////////////////////////////////////

#define HUMAX_MAX_DIR_ENTRIES	896				/*!< */
#define RECORDING_TAG			" [REC]"		/*!< */
#define HUMAX_MAX_FILENAME_LEN	59				/*!< */
#define MISSING_TAG_ELINKER		"USBS"			/*!< */
#define MISSING_TAG_THIS		"###MISSING###"	/*!< */

////////////////////////////////////////////////////////////////////////////////
///// FOR FILE STATS                                                       /////
////////////////////////////////////////////////////////////////////////////////

#define ENTRIES_COUNT			0	/*!< */
#define ENTRIES_SIZE			1	/*!< */
#define DIRS_COUNT				2	/*!< */
#define DIRS_SIZE				3	/*!< */
#define FILES_COUNT				4	/*!< */
#define FILES_SIZE				5	/*!< */

////////////////////////////////////////////////////////////////////////////////
///// FOR FILE CHECK                                                       /////
////////////////////////////////////////////////////////////////////////////////

#define FILE_NOT_FOUND			0	/*!< */
#define FILE_EQUAL				1	/*!< */
#define FILE_DIFFERENT			2	/*!< */

////////////////////////////////////////////////////////////////////////////////
///// FOR TRANSFER STATS                                                   /////
////////////////////////////////////////////////////////////////////////////////

#define GOOD_BYTE_COUNT			0	/*!< */
#define RECOV_BYTE_COUNT		1	/*!< */
#define BAD_BYTE_COUNT			2	/*!< */
#define TOT_BYTE_COUNT			3	/*!< */
#define ELAPSED_SECS			4	/*!< */
#define RATE_BYTES				5	/*!< */

////////////////////////////////////////////////////////////////////////////////
///// STRUCTURE DEFINITIONS                                                /////
////////////////////////////////////////////////////////////////////////////////

/*! \brief \b **
 *
 * --
 */
struct Entry
{
	char path[MAX_PATH];	/*!< Drive/directory/filename, for example:
							 * - Host file: "c:\dir\filename.typ"
							 * - Host dirs: "c:\dir1\dir2\"  (dirs always end with trailing '\')
							 * - Device file: "1:/dir/filename.typ"
							 * - Device dirs: "1:/dir1/dir2/"  (dirs always end with trailing '/'). */
	int nameLen;			/*!< Filename length. */
	qint64 size;			/*!< Size of file or directory in bytes. */
	bool isRecording;		/*!< True if file and is a Humax file currently recording video content. */
	int index;				/*!< Index number of file (used in file/directory searches). */
	Entry * sibling;		/*!< Next sibling file or directory. */
	Entry * firstChild;		/*!< First child file or sub-directory. */
};

/*! \brief \b **
 *
 * --
 */
struct Block
{
	qint64 begByteNo;		/*!< First byte of good data block for file transfers. */
	qint64 endByteNo;		/*!< Last byte of good data block for file transfers. */
	Block * nextBlock;		/*!< Next data block. */
};

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class HumaxController : public QThread, public HumaxInterface
{    
	Q_OBJECT
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Humax Controller Class.
	 */
	HumaxController(QObject * parent = 0);

//	virtual ~HumaxController();

protected:
	virtual void run();

	QList<Entry *> mUploadFileList;
	QString mUploadDestDir;

	QString mDownloadBaseDir;
	QStringList mDownloadFileList;
	QString mDownloadDestDir;

	bool mPaused;
	int mOverwriteReturnValue;

signals:
	void transferStarted(int id, QString fileName);

	void uploadProgress(int percent, QString source, double good,
		double recovered, double bad, double total, double rate,
			int elapsedSecs, int remainSecs);

	void downloadProgress(int percent, double transferred,
		double remaining, double rate, int elapsedSecs, int remainSecs);

	void transferComplete(int id);

	void transferOverwrite(QString name, QString targetSize, QString sourceSize, bool);

	void transferError(QString error);

public slots:
	void transferCancelled();

	void transferOverwriteReturn(int retVal);

public:

	bool Connect();
	bool IsConnected();
	bool Disconnect();

	//bool doInit(char * argv[]);
	bool Init();

	// bool doChangeDisk(usb_dev_handle * dh, char diskNo);
	// See MainWindow::on_mpDeviceDriveCombo_currentIndexChanged()
	void ChangeDisk(char diskNo);

	// bool doList(usb_dev_handle * dh);
	// See MainWindow::UpdateDeviceList()

	// bool doChdir(usb_dev_handle * dh);
	// See MainWindow::on_mpDeviceDriveView_doubleClicked()

	// bool doRename(usb_dev_handle * dh);
	// See MainWindow::on_mpActionDeviceRename_triggered()

	// bool doDelete(usb_dev_handle * dh);
	// See MainWindow::on_mpActionDeviceDelete_triggered()

	// bool doMkdir(usb_dev_handle * dh);
	// See MainWindow::on_mpActionDeviceNewFolder_triggered()

	// bool doInfo(usb_dev_handle * dh, bool fullInfo)
	// See MainWindow::OnFileConnect()
	// and MainWindow::on_mpActionInformation_triggered()

	// bool doFormat(usb_dev_handle * dh);
	// See MainWindow::OnFileFormat()
	bool Format();

	// bool doTransfersDeviceToHost(usb_dev_handle * dh, char * legend, char * baseSourceDir, Entry * sourceDir, char * baseTargetDir, Entry * targetDir);
	bool doTransfersDeviceToHost(QList<Entry *> fileList, const QString &destDir);
	// bool doTransfersHostToDevice(usb_dev_handle * dh, char * legend, char * baseSourceDir, Entry * sourceDir, char * baseTargetDir, Entry * targetDir, char * filetypes);
	bool doTransfersHostToDevice(const QString &basePath, const QStringList &fileList, const QString &destDir);

	// void doHelp();
	// See MainWindow::on_mpActionInformation_triggered()
	// and MainWindow::on_mpActionAbout_triggered()

//protected:
	Entry *createEntry(const char *path, int nameLen, qint64 size);
	void initEntry(Entry *e, const char *path, int nameLen, qint64 size);
	void deleteEntryTree(Entry *dir);
	Block *createBlock(qint64 begByteNo, qint64 endByteNo, Block *curBlock, Block *nextBlock);
	void deleteBlockList(Block *blocks);

	bool xferFileDeviceToHost(Entry *sourceFile, Entry *targetFile);
	bool xferFileHostToDevice(Entry *sourceFile, Entry *targetFile);

	int setDeviceDirectory(Entry *dir);
	int buildDeviceEntryTree(Entry *dir, int level, qint64 *stats);
	int checkFile(Entry *sourceFile, Entry *targetDir);
	bool getFileInfo(Entry *file);
	int displayEntryList(Entry *dir, bool displayDirs, bool displayFiles, bool displayLinks, bool displaySummary, bool displayIndexes, qint64 *stats);

	void buildDeviceEntryList(char *data, bool *sizeChanged, int noEntries, Entry *dir);
	QString getFileName(const QString &filePath);
	bool isXferComplete(qint64 fileSize, Block *goodBlocks);
	qint64 xferGetRecoveryByteCount(Block *goodBlocks);
	qint64 xferGetGoodByteCount(Block *goodBlocks);
	Entry *getCurDeviceDir();
	bool isDirectory(Entry *e);
	bool isRootDirectory(Entry *e);
	int getFilePath(char *path, int nameLen, char *newName,
						bool forceParent, char *filepath);
	
	/*! \brief Returns a valid device filepath by truncating the filename.
	 *
	 * Returns a valid device filepath by truncating the filename to a maximum of
	 * 59 characters. If the filetype after the '.' character in the filename is
	 * 4 or less characters then preserves that filetype.
	 * \param filepath	Input filepath, Output modified valid filepath.
	 * \param nameLen	Length of filename in filepath.
	 * \return Length of returned new filename length part of.
	 */
	int getValidDeviceFilePath(char *filepath, int nameLen);

	/*! \brief Returns a valid host filepath.
	 *
	 * Returns a valid host filepath by replacing any invalid characters in
	 * the filename with the underscore '_' character.
	 * \param filepath	Input filepath, Output modified valid filepath.
	 * \param nameLen	Length of filename in filepath.
	 * \return Length of filename part only.
	 */
	int getValidHostFilePath(char *filepath, int nameLen);

//private:
	/*! \brief Current device disk number.
	 *
	 * Current device disk number ('1' for Images/MP3 disk, '2' for Video disk).
	 */
	char deviceCurDiskNo;

	/*! \brief Current Humax directory.
	 *
	 * Current Humax directory for disks '1' and '2'.
	 */
	Entry deviceCurDir[2];

	/*! \brief Host to Humax transfer block size.
	 *
	 * Host to Humax transfer block size (min 1, max 252)
	 * Actual size of block is 256 x this value.
	 */
	int downloadBlockSize;

	/*! \brief Humax to Host Primary/Seek transfer block size.
	 *
	 * Humax to Host Primary/Seek transfer block size (min 1, max 252)
	 * Actual size of block is 256 x this value.
	 */
	int uploadPrimaryBlockSize;

	/*! \brief Humax to Host Recovery transfer block size.
	 *
	 * Humax to Host Recovery transfer block size (min 1, max 252)
	 * Actual size of block is 256 x this value.
	 */
	int uploadRecoveryBlockSize;

	/*! \brief Stores the current process priority.
	 *
	 * Stores the current process priority as specified by the user.
	 */
	int processPriority;

	/*! \brief Transfer cancelled flag.
	 *
	 * Indicated to the transfer thread that the user has requested the
	 * transfer is stopped.
	 */
	bool mCancelled;
};

#endif // HUMAXCONTROLLER_H
////////////////////////////////////////////////////////////////////////////////
