////////////////////////////////////////////////////////////////////////////////
/// \class HumaxInterface HumaxInterface.h
///
/// \version 1.00
/// \date 04-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// See COPYING file that comes with this distribution.
///
/// \brief Class provided to interface with the USB interface.
///
/// This class is provided to give humax specific functionality to the GUI by
/// supplying commands to and retrieving data from the USB interface.
///
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#ifndef HUMAXINTERFACE_H
#define HUMAXINTERFACE_H

#include <QtGlobal>
#include "UsbInterface.h"

#ifndef WIN32
// 260?
#define MAX_PATH 32656
typedef qint64 __int64;
#endif

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class HumaxInterface : public UsbInterface
{    
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the interface Class.
	 */
	HumaxInterface();

	/*! \brief Destructor.
	 *
	 * Clean up any memory usage and destroy the class.
	 */
	virtual ~HumaxInterface();

	/*! \brief Connects to the device.
	 *
	 * Connects to the device via the USB interface using the LIBUSB(-WIN32)
	 * drivers.
	 * \return	If successfully connected returns the USB device handle used
	 *			by all other functions in this program. if failed, just
	 *			returns NULL.
	 */
	usb_dev_handle *humaxConnect();

	/*! \brief Disconnects from the device.
	 * 
	 * Disconnects from the device via the USB interface.
	 * \return Boolean, true - success, false - failed.
	 */
	bool humaxDisconnect();

	/*! \brief Gets the number of directory entries
	 *
	 * Gets the number of directory entries in the current device directory.
	 * HMC Command: [J].
	 * \return	The total number of directory entries in the current device
	 *			directory (min 0, max 1024)
	 */
	int humaxGetNoDirEntries();

	/*! \brief Reads the directory entries.
	 *
	 * Reads the directory entries from the device current directory.
	 * HMC Command: [K].
	 * \param noEntries		Total number of directory entries to read
	 * \param doDualRead	True if a read, pause, read operation to determine
	 *						the 'isRecording' status of Humax video files.
	 * \param data			Output data buffer into which to read the raw
	 *						USB data.
	 * \param sizeChanged	Output boolean array of 'noEntries' in size set
	 *						to true for Humax video files currently being
	 *						recorded.
	 * \return				Boolean, operation succeeded, operation failed.
	 */
	bool humaxGetDirEntries(int noEntries, bool doDualRead, char *data,
								bool *sizeChanged);

	/*! \brief Get the free and total disk space.
	 *
	 * Get the free and total disk space from the device for the current
	 * disk partition.
	 * HMC Command: [L].
	 * \param diskSpace	Output array of 2 integers to receive the 'free'
	 *					and 'total' disk space values (set on exit).
	 *					[0] 'free' disk space [1] 'total' disk space.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxGetDiskSpace(qint64 *diskSpace);

	/*! \brief Initiates a USB "M1" transfer.
	 *
	 * Initiates a USB "M1" device to host file transfer.
	 * HMC Command: [M1].
	 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
	 *					used throughout each transfer process.
	 * \param filePath	Humax filepath string of file to be transferred
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxXferBeginDevToHost(int dCBWTag, char *filePath);

	/*! \brief Initiates a USB "M2" transfer.
	 *
	 * Initiates a USB "M2" host to device file transfer.
	 * HMC Command: [M2].
	 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
	 *					used throughout each transfer process.
	 * \param filePath	Humax filepath string of the new file to be
	 *					transferred and created on the Humax.
	 * \param fileSize	Size of the new target file to be created on
	 *					the Humax.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxXferBeginHostToDev(int dCBWTag, char *filePath, qint64 fileSize);

	/*! \brief Ends a file transfer.
	 *
	 * Ends an "M1" or "M2" USB file transfer.
	 * HMC Command: [N].
	 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
	 					used throughout each transfer process.
	 * \param filePath	Humax filepath string of file transfer to end.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxXferEnd(int dCBWTag, char *filePath);

	/*! \brief Read a block of data for "M1" file transfers.
	 *
	 * Read a block of data for "M1" file transfers from device to host.
	 * HMC Command: [O].
	 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
	 					used throughout each transfer process.
	 * \param data		Output the data buffer into which receive file data
	 *					from the Humax via USB.
	 * \param noBytes	Output the size of the data block to read from the
	 *					Humax.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	int humaxXferNextDevToHost(int dCBWTag, char *data, int noBytes);

	/*! \brief Read a block of data for "M2" transfers.
	 *
	 * Read a block of data for "M2" file transfers from host to device.
	 * HMC Command: [P].
	 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
	 					used throughout each transfer process.
	 * \param data		Contains the data to be transferred to the Humax.
	 * \param noBytes	Contains the size of the data in the data buffer
	 *					to be transferred to the Humax.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxXferNextHostToDev(int dCBWTag, char *data, int noBytes);

	/*! \brief Rename a file on the device.
	 *
	 * Rename a file on the device.
	 * HMC Command: [R].
	 * \param oldFilePath	The Humax filepath of the file or directory to
	 *						be renamed.
	 * \param newFilePath	The Humax filepath of the new name for the file
	 *						or directory.
	 * \return				Boolean, operation succeeded, operation failed.
	 */
	bool humaxRenameFile(char *oldFilePath, char *newFilePath);

	/*! \brief Create a new directory on the device.
	 *
	 * Create a new directory on the device.
	 * HMC Command: [S].
	 * \param dirPath	The Humax filepath of the new directory to be created
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxCreateDir(char *dirPath);

	/*! \brief Delete a file on the device
	 *
	 * Delete a file on the device
	 * HMC Command: [T].
	 * \param filePath	The Humax filepath of the file or directory to
	 *					be deleted.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxDeleteFile(char *filePath);

	/*! \brief Formats the MP3/Image partition on the device.
	 *
	 * Formats the MP3/Image partition on the device and creates the initial
	 * MP3' and 'Image' dirs in the root directory. Note that no known
	 * interface currently exists to do the same for the Video disk partition
	 * no 2 though!
	 * HMC Command: [U].
	 * \return	Boolean, operation succeeded, operation failed.
	 */
	bool humaxFormatDiskOne();

	/*! \brief Sets the current directory on the device
	 *
	 * Sets the current directory on the device
	 * HMC Command: [V].
	 * \param dirPath	The Humax filepath of the new directory to be
	 *					changed to.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxSetDirectory(char *dirPath);

	/*! \brief Retrieves the total number of accessible disk partitions on
	 *			the device.
	 *
	 * Retrieves the total number of accessible disk partitions on the device
	 * (should always return 2) but as we already know that this software just
	 * assumes that value and therefore doesn't use this function! Function
	 * included for the sake of completeness anyway.
	 * HMC Command: [W].
	 * \return The number of partitions present (always 2 for the PVR9200T).
	 */
	int humaxGetNoPartitions();

	/*! \brief Sets the current disk partition on the device
	 *
	 * Sets the current disk partition on the device
	 * HMC Command: [X].
	 * \param diskNo	ASCII '1' for the Image/MP3 disk and ASCII '2' for
	 *					the Video disk.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
	bool humaxSetDiskNo(char diskNo);

	// My Additional functions:

	/*! \brief Get the manufacturer text from the device.
	 *
	 * Get the manufacturer information text from the humax device.
	 * \return The manufacturer string.
	 */
	QString humaxManufacturer();

	/*! \brief Get the product text from the device.
	 *
	 * Get the product information text from the humax device.
	 * \return The product string.
	 */
	QString humaxProduct();

	/*! \brief Get the hardware version text from the device.
	 *
	 * Get the hardware version information text from the humax device.
	 * \return The hardware version string.
	 */
	QString humaxHardwareVersion();

	/*! \brief Get the serial number text from the device.
	 *
	 * Get the serial number information text from the humax device.
	 * \return The serial number string.
	 */
	QString humaxSerialNumber();

protected:
	/*! \brief Increment the USB Bulk Transfer command block.
	 *
	 * Increment the USB Bulk Transfer command block TagID value and
	 * return the new value.
	 * \return The new USB Command Block TagID value used in USB operations.
	 * \sa dCBWTag.
	 */
	int incCBWTag();

private:
	/*! \brief USB command block TagID 
	 *
	 * USB command block TagID, a unique value for each command sent to
	 * the Humax.
	 * \sa incCBWTag().
	 */
	int dCBWTag;
};

#endif // HUMAXINTERFACE_H
////////////////////////////////////////////////////////////////////////////////
