////////////////////////////////////////////////////////////////////////////////
/// \class HumaxItem HumaxItem.h
///
/// \version 1.00
/// \date 04-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
///
/// \brief A Class to store item data for the HumaxModel class. 
///
/// This class stores data for an individual item in the HumaxModel class.
///
/// This file is based on part of the example classes of the Qt Toolkit.
///
/// This file may be used under the terms of the GNU General Public
/// License version 2.0 as published by the Free Software Foundation
/// and appearing in the file LICENSE.GPL included in the packaging of
/// this file.  Please review the following information to ensure GNU
/// General Public Licensing requirements will be met:
/// http://www.trolltech.com/products/qt/opensource.html
///
/// If you are unsure which license is appropriate for your use, please
/// review the following information:
/// http://www.trolltech.com/products/qt/licensing.html or contact the
/// sales department at sales@trolltech.com.
///
/// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
/// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#ifndef HUMAXITEM_H
#define HUMAXITEM_H

#include <QList>
#include <QVariant>

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class HumaxItem
{
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Item Class.
	 * \param data		Row data for this item.
	 * \param parent	The parent of this item.
	 */
	HumaxItem(const QList<QVariant> &data, HumaxItem *parent = 0);

	/*! \brief Destructor.
	 *
	 * Clean up any memory usage and destroy the class.
	 */
	virtual ~HumaxItem();

	/*! \brief Append a child item to this item.
	 *
	 * Add a HumaxItem to the end of the list of child items.
	 * \param child The child item to be added.
	 * \sa mChildItems.
	 */
	void appendChild(HumaxItem *child);

	/*! \brief Get the child item at the given row.
	 *
	 * Get the child HumaxItem at the supplied row index.
	 * \param row The index of the child item.
	 * \return The child item at the index or 0 if no item found.
	 * \sa mChildItems.
	 */
	HumaxItem *child(int row);

	/*! \brief Get the number of child items.
	 *
	 * Returns the number of children attached to this model item.
	 * \return The number of children.
	 * \sa mChildItems.
	 */
	int childCount() const;

	/*! \brief Get the number of columns.
	 *
	 * Returns the number of data columns stored in the model item.
	 * \return The number of data columns.
	 */
	int columnCount() const;

	/*! \brief Get the data for a given column.
	 *
	 * Returns the data stored in the column requested.
	 * \param column The data index of the item.
	 * \return The data stored in the column.
	 * \sa mItemData.
	 */
	QVariant data(int column) const;

	/*! \brief Get the row of this item.
	 *
	 * Returns the row index of this item.
	 * \return The row index.
	 */
	int row() const;

	/*! \brief Get the parent item.
	 *
	 * Returns the parent model item of this item.
	 * \return The parent item.
	 * \sa mpParentItem.
	 */
	HumaxItem *parent();

	/*! \brief Is this item a directory item.
	 *
	 * Returns whether this item is a directory item.
	 * \return Boolean, true if a directory item.
	 */
	inline bool isDirectory() const { return mDir; };

	/*! \brief Set this item to be a directory item.
	 *
	 * Set whether this item is a directory item.
	 * \param dir	Directory status.
	 */
	inline void setDirectory(const bool &dir) { mDir = dir; };

	/*! \brief Is this item associated with a recording file.
	 *
	 * Returns whether this item is associated with a recording file.
	 * \return Boolean, true if recording.
	 */
	inline bool isRecording() const { return mRecording; };

	/*! \brief  Set the recording association.
	 *
	 * Set whether this item is associated with a file currently
	 * being recorded.
	 * \param rec	Recording status.
	 */
	inline void setRecording(const bool &rec) { mRecording = rec; };
/*
	void sortChildren();
*/
private:
	/*! \brief List of child items.
	 *
	 * Stores a list of all the child model items associated with this item.
	 */
	QList<HumaxItem*> mChildItems;

	/*! \brief List of column data for the item.
	 *
	 * Stores a list of the column data for this model item.
	 */
	QList<QVariant> mItemData;

	/*! \brief The parent item.
	 *
	 * Stroes the parent model item.
	 */
	HumaxItem *mpParentItem;

	/*! \brief Directory flag.
	 *
	 * The directory flag of the model item, if true the item is a directory.
	 */
	bool mDir;

	/*! \brief Recording flag.
	 *
	 * The recording flag of the model item, if true the associated file on
	 * the device is currently being recorded.
	 */
	bool mRecording;
};

#endif // HUMAXITEM_H
