////////////////////////////////////////////////////////////////////////////////
/// \class HumaxModel HumaxModel.h
///
/// \version 1.00
/// \date 04-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
///
/// \brief Model to control the data (HumaxItem 's) in the Device View.
///
/// Data Model to control the data (HumaxItem 's) displayed in the
/// Device View. See QAbstractItemModel for more details.
///
/// This file is based on part of the example classes of the Qt Toolkit.
///
/// This file may be used under the terms of the GNU General Public
/// License version 2.0 as published by the Free Software Foundation
/// and appearing in the file LICENSE.GPL included in the packaging of
/// this file.  Please review the following information to ensure GNU
/// General Public Licensing requirements will be met:
/// http://www.trolltech.com/products/qt/opensource.html
///
/// If you are unsure which license is appropriate for your use, please
/// review the following information:
/// http://www.trolltech.com/products/qt/licensing.html or contact the
/// sales department at sales@trolltech.com.
///
/// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
/// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#ifndef HUMAXMODEL_H
#define HUMAXMODEL_H

#include <QAbstractItemModel>
#include <QIcon>
#include <QModelIndex>
#include <QTreeView>
#include <QVariant>

class HumaxController;
class HumaxItem;
struct Entry;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class HumaxModel : public QAbstractItemModel
{
	Q_OBJECT
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Model Class.
	 */
	HumaxModel();

	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Model Class.
	 * \param controller	Humax Controller class to help decode the
	 *						entry list.
	 * \param list			The list of entries in the current directory
	 *						on the device.
	 * \param parent		The parent class of the Model.
	 */
	HumaxModel(HumaxController *controller,
				Entry *list, QObject *parent = 0);

	/*! \brief Destructor.
	 *
	 * Clean up any memory usage and destroy the class.
	 */
	virtual ~HumaxModel();

	/*! \brief Update the model with new data.
	 *
	 * Populates the model with the list of entries supplied, this will
	 * then be shown in the device view.
	 * \param controller	Humax Controller class to help decode the
	 *						entry list.
	 * \param list			The list of entries in the current directory
	 *						on the device.
	 */
	void SetEntryList(HumaxController *controller, Entry *list);

	/*! \brief Get the data for a given model \a index and \a role.
	 *
	 * Get the data for a given model \a index and \a role for
	 * display in the device view.
	 * \param index	The index of the HumaxItem data to be populated.
	 * \param role	The role for the type of data displayed.
	 * \return The data to be displayed for the \a index and \a role.
	 * \sa QAbstractItemModel::data().
	 */
	QVariant data(const QModelIndex &index, int role) const;

	/*! \brief Get the display state for a given model \a index.
	 *
	 * Get the display state for a given model \a index for display 
	 *in the device view.
	 * \param index	The index of the HumaxItem data to be populated.
	 * \return Display flags of how the item is shown.
	 * \sa QAbstractItemModel::flags().
	 */
	Qt::ItemFlags flags(const QModelIndex &index) const;

	/*! \brief Supplies the header data for a row/column section.
	 *
	 * Supplies the data to be displayed in the view's header row or
	 * column.
	 * \param section		The section index to be populated.
	 * \param orientation	The header orientation being populated.
	 * \param role			The role for the type of data displayed.
	 * \return The data to be displayed in the header.
	 * \sa QAbstractItemModel::headerData().
	 */
	QVariant headerData(int section, Qt::Orientation orientation,
						int role = Qt::DisplayRole) const;

	/*! \brief Gets the model index item for a row and column.
	 *
	 * Gets the model index item for the row and column position supplied
	 * based on the parent index.
	 * \param row		The row of the index requested.
	 * \param column	The column of the index requested.
	 * \param parent	The parent item to base the request.
	 * \return The model index item at the position request.
	 * \sa QAbstractItemModel::index().
	 */
	QModelIndex index(int row, int column,
						const QModelIndex &parent = QModelIndex()) const;

	/*! \brief Gets the parent model index for the item supplied.
	 *
	 * Gets the parent model index for the model item supplied.
	 * \param index	The index of the HumaxItem data to be populated.
	 * \return The parent item or a null item if no parent available.
	 * \sa QAbstractItemModel::parent().
	 */
	QModelIndex parent(const QModelIndex &index) const;

	/*! \brief Get the number of rows in the model.
	 *
	 * Gets the number of rows in the model based on the parent item. If
	 * the parent item is NULL then the root item is used.
	 * \param parent The item to use for the row count.
	 * \return The number of rows in the item.
	 * \sa QAbstractItemModel::rowCount().
	 */
	int rowCount(const QModelIndex &parent = QModelIndex()) const;

	/*! \brief Get the number of columns in the model.
	 *
	 * Gets the number of columns in the model based on the parent item. If
	 * the parent item is NULL then the root item is used.
	 * \param parent The item to use for the column count.
	 * \return The number of columns in the item.
	 * \sa QAbstractItemModel::columnCount().
	 */
	int columnCount(const QModelIndex &parent = QModelIndex()) const;

	/*! \brief Get the filename for a model item.
	 *
	 * Returns the name of the item stored in the model under the
	 * index given.
	 * \param index	The index of the HumaxItem data to be retrieved.
	 * \return The filename of the item.
	 */
	QString fileName(const QModelIndex &index) const;

	/*! \brief Get the file size for a model item.
	 *
	 * Returns the size of the item stored in the model under the
	 * index given.
	 * \param index	The index of the HumaxItem data to be retrieved.
	 * \return The size of theitem on the device.
	 */
	qint64 fileSize(const QModelIndex &index) const;

	/*! \brief Get the directory state of an item.
	 *
	 * Returns true if the model item index represents a directory;
	 * otherwise returns false.
	 * \param index	The index of the HumaxItem data to be retrieved.
	 * \return Boolean, true if the model item is a directory item.
	 */
	bool isDir(const QModelIndex &index) const;

	/*! \brief Get the recording state of an item.
	 *
	 * Returns true if the model item index represents a recording item;
	 * otherwise returns false.
	 * \param index	The index of the HumaxItem data to be retrieved.
	 * \return Boolean, true if the model item is in a recording state.
	 */
	bool isRecording(const QModelIndex &index) const;

	/*! \brief Get the path of the model root item.
	 *
	 * Get the device path of the model root item.
	 * \return The Device path string.
	 * \sa setModelPath() and mModelPath.
	 */
	inline QString getModelPath() const { return mModelPath; };

	/*! \brief Set the path of the model root item.
	 *
	 * Set the device path of the model root item.
	 * \param modelPath The Device path string.
	 * \sa getModelPath() and mModelPath.
	 */
	inline void setModelPath(const QString &modelPath)
		{ mModelPath = modelPath; };

private:
	/*! \brief Converts the Entry list supplied into a list of HumaxItems.
	 *
	 *  Converts the Entry list supplied when the data changes into a list
	 * of HumaxItems to be displayed in the Device View.
	 * \param controller	Humax Controller class to help decode the
	 *						entry list.
	 * \param list			The list of entries in the current directory
	 *						on the device.
	 * \param parent		The top level item the list is to be added.
	 */
	void setupModelData(HumaxController *controller,
						Entry *list, HumaxItem *parent);

	/*! \brief The root model item.
	 *
	 * The root HumaxItem of the model.
	 */
	HumaxItem *mpRootItem;

	/*! \brief The device path of the model.
	 *
	 * The device path of the model being displayed.
	 */
	QString mModelPath;

	/*! \brief Store an icon for displaying recording files.
	 *
	 * Stores a local icon for displaying with recording files.
	 */
	QIcon mRecordingIcon;
};

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class QTreeViewEx : public QTreeView
{
	Q_OBJECT
public:
	inline QTreeViewEx(QWidget *parent = 0)
		: QTreeView(parent) {};
protected slots:
	virtual void selectionChanged(const QItemSelection & selected,
		const QItemSelection & deselected)
	{
		QTreeView::selectionChanged(selected, deselected);
		emit itemSelectionChanged();
	};

signals:
	void itemSelectionChanged();
};

#endif // HUMAXMODEL_H
////////////////////////////////////////////////////////////////////////////////
