////////////////////////////////////////////////////////////////////////////////
/// \class QHostProxyModel QHostProxyModel.h
///
/// \version 1.00
/// \date 11-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// See COPYING file that comes with this distribution.
///
/// \brief Class to enable filtering of the Host Directory View.
///
/// A Proxy Model class to enable the filtering of the host directory
/// model/view implementation. Used to filter file types while still leaving
/// the directories visible. Usual filters are '.jpg', '.mp3', and '.ts'.
///
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
11-07-06 1.00 Initial release.

*****/
#ifndef QHOSTPROXYMODEL_H
#define QHOSTPROXYMODEL_H

#include <QSortFilterProxyModel>
#include <QStringList>

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class QHostProxyModel : public QSortFilterProxyModel
{
	Q_OBJECT
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Host Filter Proxy Model Class.
	 */
	QHostProxyModel(QObject *parent = 0);

	/*! \brief Destructor.
	 *
	 * Clean up any memory usage and destroy the class.
	 */
	inline virtual ~QHostProxyModel() {};

	/*! \brief Set the list of file extensions to be filtered.
	 *
	 * Inputs the list of file extensions to be filtered by the proxy.
	 * Eg '.jpg'.
	 * \param filter The filter list.
	 * \sa getFilterList() and mFilterList.
	 */
	inline void setFilterList(const QStringList &filter)
		{ mFilterList = filter; };

	/*! \brief Get the list of file extensions being filtered.
	 *
	 * Returns the list of file extensions being used as the filters for the proxy.
	 * Eg '.jpg'.
	 * \return The filter list.
	 * \sa setFilterList() and mFilterList.
	 */
	inline QStringList getFilterList() const
		{ return mFilterList; };

protected:
	/*! \brief Decides whether a row should be filtered based on the list.
	 *
	 * Filters the given row based on the list supplied to the proxy.
	 * Automatically called from the model/view when updating the display.
	 * \param source_row	The row of the model that the filter is to
	 *						be applied.
	 * \param source_parent	The parent model to get the filter data.
	 * \return Boolean, TRUE displays the row, FALSE filters it out.
	 * \sa setFilterList() and mFilterList.
	 */
	virtual bool filterAcceptsRow(int source_row,
					const QModelIndex &source_parent) const;

private:
	/*! \brief Stores the list of filters.
	 *
	 * Stores a list of file extensions to be filtered by the proxy for
	 * the directory view.
	 */
	QStringList mFilterList;
};

#endif // QHOSTPROXYMODEL_H
////////////////////////////////////////////////////////////////////////////////
