////////////////////////////////////////////////////////////////////////////////
/// \class TransferDialog TransferDialog.h
///
/// \version 1.00
/// \date 05-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// See COPYING file that comes with this distribution.
///
/// \brief Media Controller Data Transfer Dialog Class.
///
/// Data Transfer dialog to display information about the current
/// upload/download transfer to/from the humax pvr9200t device.
///
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
05-07-06 1.00 Initial release.

*****/
#ifndef TRANSFERDIALOG_H
#define TRANSFERDIALOG_H

#include "ui_TransferDialog.h"

#include <QDialog>

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class TransferDialog : public QDialog
{    
	Q_OBJECT
public:
	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the Transfer Dialog Class.
	 */
	TransferDialog(QWidget *parent = 0);

	/*! \brief Dialog transfer mode enumerator.
	 *
	 * The dialog can be shown in one of two modes, upload transfer
	 * and download transfer, each displays different widgets for transfer
	 * information.
	 * \sa setTransferMode().
	 */
	enum TransferMode { DOWNLOAD, UPLOAD };

	/*! \brief Dialog transfer mode set accessor.
	 *
	 * Set up the dialogs display mode for either upload or download.
	 * Default is download.
	 * \param mode The display mode to be shown.
	 * \sa TransferMode.
	 */
	void setTransferMode(TransferMode mode);

	/*! \brief Informs the transfer dialog of the number of files in the transfer.
	 *
	 * Informs the transfer dialog of the number of files in the current
	 * transfer so that the dialog can update the total progress bar.
	 * \param count The number of files to be transferred.
	 * \sa mFileCount
	 */
	inline void SetFileCount(const int &count) { mFileCount = count; };

signals:
	/*! \brief Signals that a cancel request has been made.
	 *
	 * This Qt Signal indicates to whatever is caonnected that the user has
	 * requested that the current transfer operation is to be cancelled.
	 */
	void transferCancelled();

	/*! \brief
	 */
	void transferOverwriteReturn(int retVal);

public slots:
	/*! \brief Called by the controller when a file transfer is started.
	 *
	 * Called by the controller transfer thread to indicate when it has
	 * started downloading a file.
	 * \param id		The index of the file download.
	 * \param fileName	The filename of the download.
	 * \sa transferComplete() and mFileCount.
	 */
	void transferStarted(int id, QString fileName);

	/*! \brief Displays upload progess on the dialog.
	 *
	 * Updates the upload progress information in the
	 * dialog and the progress bar.
	 * \param percent		The percentage the transfer is complete.
	 * \param source		The type of transfer, EG transfer, chase, recovery.
	 * \param good			The number of good bytes transferred.
	 * \param recovered		The number of bytes recovered.
	 * \param bad			The number of bad bytes transferred.
	 * \param total			The total number of bytes transferred.
	 * \param rate			The transfer rate of the data.
	 * \param elapsedSecs	The number of seconds that have elapsed since the
	 *						start of the transfer.
	 * \param remainSecs	The number of seconds (approx) remaining to the
	 *						end of the transfer.
	 */
	void uploadProgress(int percent, QString source, double good,
		double recovered, double bad, double total, double rate,
			int elapsedSecs, int remainSecs);

	/*! \brief Displays download progess on the dialog.
	 *
	 * Updates the download progress information in the
	 * dialog and the progress bar.
	 * \param percent		The percentage the transfer is complete.
	 * \param transferred	The number of bytes transferred.
	 * \param remaining		The number of bytes yet to be transferred.
	 * \param rate			The transfer rate of the data.
	 * \param elapsedSecs	The number of seconds that have elapsed since the
	 *						start of the transfer.
	 * \param remainSecs	The number of seconds (approx) remaining to the
	 *						end of the transfer.
	 */
	void downloadProgress(int percent, double transferred,
		double remaining, double rate, int elapsedSecs, int remainSecs);

	/*! \brief Indicates the transfer of a file is complete.
	 *
	 * At the end of each file transfer this slot function is called to
	 * notify the transfer dialog of the completion in order to update the
	 * progress bars.
	 * \param id	The index of the file that has finished transferring.
	 */
	void transferComplete(int id);

	/*! \brief
	 */
	void transferOverwrite(QString name, QString targetSize, QString sourceSize, bool single);

	/*! \brief Called if the user presses the cancel button.
	 *
	 * This function is called when the user presses the cancel button in
	 * order to abort the transfer. The function will notify the transfer
	 * process thread to cancel and then wait for it to finish.
	 */
	void on_cancelButton_clicked();

protected:
	/*! \brief Called when the Close [x] buttons is pressed.
	 *
	 * This event is raise when the windows close [x] button is pressed.
	 * Note that the close button should be removed from the window.
	 * \param event	The close event information.
	 */
	void closeEvent(QCloseEvent *event);

private:
	/*! \brief Transfer Dialog widget information.
	 *
	 * Contains all of the information/widgets needed to created the dialog.
	 * This information is stored in TransferDialog.ui and is auto-generated
	 * by Qt.
	 */
	Ui::TransferDialog ui;

	/*! \brief The current transfer display mode.
	 *
	 * The current display mode of the dialog. Determines whether the upload
	 * or download data is displayed on the dialog.
	 * \sa setTransferMode().
	 */
	TransferMode mMode;

	/*! \brief The number of files in the transfer.
	 *
	 * Stores the number of files being transferred while the dialog is
	 * displayed. Used for updating the progress bar.
	 * \sa SetFileCount().
	 */
	int mFileCount;

	/*! \brief The current file being transferred.
	 *
	 * Stores the file name of the current file being transferred.
	 * \sa transferStarted().
	 */
	QString mCurrentFile;

	/*! \brief Megabyte text string constant.
	 *
	 * Static constant to store the MB string for updating the display.
	 */
	static const QString mcMB;

	/*! \brief Kilobytes per second text string constant.
	 *
	 * Static constant to store the KBpS string for updating the display.
	 */
	static const QString mcKBPS;
};

#endif // TRANSFERDIALOG_H
////////////////////////////////////////////////////////////////////////////////
