////////////////////////////////////////////////////////////////////////////////
/// \class UsbInterface UsbInterface.h
///
/// \version 1.00
/// \date 04-07-2006
/// \author Andrew Smith <andrew.smith@freenet.co.uk>
///
/// \par Copyright:
/// See COPYING file that comes with this distribution.
///
/// \brief USB Interface Class.
///
/// Low Level USB interface class used to communicate with libusb.
///
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#ifndef USBINTERFACE_H
#define USBINTERFACE_H

#include "usb.h"
#include <stdio.h>

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
class UsbInterface
{    
public:

	/*! \brief Constructor.
	 *
	 * Construct and initialise the data in the USB Interface Class.
	 */
	UsbInterface();

	/*! \brief Destructor.
	 *
	 * Clean up any memory usage and destroy the class.
	 */
	virtual ~UsbInterface();

	/*! \brief Read the device status block.
	 *
	 * Reads a Bulk Transfer "CSW" status block from the device via USB.
	 * \param functionName	String name of calling function.
	 * \param checkpointID	Checkpoint ID within the calling function.
	 * \param msg			General display message.
	 * \param buf			Data buffer into which to read the USB data from
	 *						the device.
	 * \return true - operation succeeded; false - operation failed.
	 */
	bool readBulkCSW(char *functionName, int checkpointID,
						char *msg, char *buf);

	/*! \brief Reads a device data block.
	 *
	 * Reads a Bulk Transfer data block from the device via USB and returns
	 * the number of bytes read.
	 * \param functionName	String name of calling function.
	 * \param checkpointID	Checkpoint ID within calling function.
	 * \param msg			General display message.
	 * \param buf			Output data buffer into which to read the USB
	 *						data from the device.
	 * \param cb			Expected length of received USB data.
	 * \return The actual number of bytes received.
	 */
	int readBulkData(char *functionName, int checkpointID,
						char *msg, char *buf, int cb);

	/*! \brief Write a device command block.
	 *
	 * Writes a Bulk Transfer "CBW" command block to the device via USB.
	 * \param functionName				Name of calling function.
	 * \param checkpointID				Checkpoint ID withing calling function.
	 * \param msg						General display message.
	 * \param dCBWTag					Unique USB transaction TagID value,
	 *									different for each USB operation.
	 * \param dCBWDataTransferLength	Expected data size transfer length.
	 * \param bmCBWFlags				USB directional flags (0x00 = Host to
	 *									Humax, 0x80 = Humax to Host).
	 * \param CBWCB						USB CBW (Command Block) data buffer
	 *									(excluding the CBW header, just the
	 *									data part).
	 * \param bCBWCBLength				Length of Command Block data (excluding
	 *									the CBW header).
	 * \return true - operation succeeded; false - operation failed.
	 */
	bool writeBulkCBW(char *functionName, int checkpointID,
						char *msg, int dCBWTag, int dCBWDataTransferLength,
						int bmCBWFlags, char *CBWCB, int bCBWCBLength);

	/*! \brief Write a device data block.
	 *
	 * Writes a Bulk Transfer data block to the device via USB.
	 * \param functionName	String name of calling function.
	 * \param checkpointID	Checkpoint ID for location within the calling 
	 *						function.
	 * \param msg			General display message.
	 * \param buf			Data buffer containing data to be sent over USB
	 *						to the device.
	 * \param len			Length of data to send
	 * \return true - operation succeeded; false - operation failed.
	 */
	bool writeBulkData(char *functionName, int checkpointID,
						char *msg, char *buf, int len);

	/*! \brief Set Accessor for error recovery state.
	 *
	 * Set Accessor to update the current state of the USB upload error recovery.
	 * \param recovery	Set error recovery on or off (true or false).
	 * \return None (void).
	 * \sa uploadErrorRecovery
	 */
	inline void ErrorRecovery(const bool &recovery)
		{ uploadErrorRecovery = recovery; }

	/*! \brief Get Accessor for error recovery state.
	 *
	 * Get Accessor to supply the current state of the USB upload error recovery.
	 * \return Error recovery state.
	 * \sa uploadErrorRecovery
	 */
	inline bool ErrorRecovery() const
		{ return uploadErrorRecovery; }

	/*! \brief Set Accessor for console trace.
	 *
	 * Set Accessor to update the current state of the USB data console trace.
	 * \param trace	Set trace on or off (true or false).
	 * \return None (void).
	 * \sa usbDataTraceConsole
	 */
	inline void TraceConsole(const bool &trace)
		{ usbDataTraceConsole = trace; }

	/*! \brief Get Accessor for console trace.
	 *
	 * Get Accessor to supply the current state of the USB data console trace.
	 * \return Console trace state.
	 * \sa usbDataTraceConsole
	 */
	inline bool TraceConsole() const
		{ return usbDataTraceConsole; }

	/*! \brief Set Accessor for log file trace.
	 *
	 * Set Accessor to update the current state of the USB data log file trace.
	 * \param trace	Set trace on or off (true or false).
	 * \return None (void).
	 * \sa usbDataTraceLogFile
	 */
	inline void TraceLogFile(const bool &trace)
		{ usbDataTraceLogFile = trace; }

	/*! \brief Get Accessor for log file trace.
	 *
	 * Get Accessor to supply the current state of the USB data log file trace.
	 * \return Log file trace state.
	 * \sa usbDataTraceLogFile
	 */
	inline bool TraceLogFile() const
		{ return usbDataTraceLogFile; }

protected:

	/*! \brief Display USB trace.
	 *
	 * Display a USB transaction data trace
	 * \param usbDataTraceConsole	True if output to console required.
	 * \param usbDataTraceLogFile	True if output to log file required.
	 * \param status				True if operation succeeded, false if failed.
	 * \param func					String name of function.
	 * \param id					Checkpoint ID within function.
	 * \param msg					Informational message string to be displayed.
	 * \param data					USB data to be displayed.
	 * \param dataLen				Length of USB data to be displayed.
	 * \return None (void).
	 */
	void usbTrace(bool usbDataTraceConsole, bool usbDataTraceLogFile,
					bool status, char *func, int id, char *msg,
					char *data, int dataLen);

protected:

	/*! \brief USB Device Handle.
	 *
	 * Stores the USB device connection handle (when the device is connected
	 * or NULL). Used in all access calls to the device. NOTE: SHOULD be closed
	 * once application is complete or may cause problems on the device and
	 * reconnecting.
	 */
	usb_dev_handle *mpUsbHandle;

//	struct usb_device *mpHumaxDevice; // Handle to the USB device

private:

	/*! \brief Output USB trace.
	 *
	 * Display USB transaction trace info to console and/or log file
	 * \param logFile				File handle to log file.
	 * \param usbDataTraceConsole	True if output to console requested.
	 * \param usbDataTraceLogFile	True if output to log file requested.
	 * \param buf					Buffer containing data to be
	 * 								displayed / written.
	 * \return None (void).
	 */
	void writeTraceInfo(FILE *logFile, bool usbDataTraceConsole,
						bool usbDataTraceLogFile, char *buf);

	/*! \brief Error-detection / Correction flag.
	 *
	 * Enables error-detection and correction using multiple transfers if true.
	 * \sa ErrorRecovery()
	 */
	bool uploadErrorRecovery;

	/*! \brief USB console trace flag.
	 *
	 * Enables USB data tracing to console window if true.
	 * \sa TraceConsole()
	 */
	bool usbDataTraceConsole;

	/*! \brief USB log file trace flag.
	 *
	 * Enables USB data tracing to log file if true.
	 * \sa TraceLogFile()
	 */
	bool usbDataTraceLogFile;
};

#endif // USBINTERFACE_H
////////////////////////////////////////////////////////////////////////////////
