////////////////////////////////////////////////////////////////////////////////
// \class DeviceInfoDialog DeviceInfoDialog.cpp
//
// \version 1.00
// \date 04-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief Humax device information dialog Class.
//
// A Dialog class used to display information downloaded from the Humax.
// Including Humax hardware information and Disk/File Information.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#include "DeviceInfoDialog.h"

#include "HumaxGlobals.h"

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Device Information Dialog Class.
 */
////////////////////////////////////////////////////////////////////////////////
DeviceInfoDialog::DeviceInfoDialog(QWidget *parent)
: QDialog(parent)
{
	ui.setupUi(this);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Set the disk information visible/invisible.
 *
 * Shows or Hides the area of the dialog displaying the disk information.
 * Default is the area is shown.
 * \param show Boolean flag to show/hide the disk information.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::showDiskInfo(const bool &show)
{
	if( show )
	{
		ui.mpPartitionOneBox->show();
		ui.mpPartitionTwoBox->show();
 		resize(QSize(322, 488).expandedTo(minimumSizeHint()));
	}
	else
	{
		ui.mpPartitionOneBox->hide();
		ui.mpPartitionTwoBox->hide();
 		resize(QSize(322, 208).expandedTo(minimumSizeHint()));
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Update the Disk One Space information.
 *
 * Updates the disk one space information, including the total, free, and
 * used space as well as the Disk Bar.
 * \param free	The amount of free space on the disk.
 * \param total	The total amount of space on the disk.
 */
void DeviceInfoDialog::setDiskOneSpace(const qint64 &free, const qint64 &total)
{
	ui.mpUsedSpaceOneEdit->setText(Humax::SizeAsString(total - free/*, 9, 2*/));

	ui.mpFreeSpaceOneEdit->setText(Humax::SizeAsString(free/*, 9, 2*/));

	ui.mpTotalSpaceOneEdit->setText(Humax::SizeAsString(total/*, 9, 2*/));

	ui.mpDiskOneBar->setDiskSpace(free, total);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the Disk One Directory Information.
 *
 * Updates the information showing the total size of the directories on
 * disk one and the number of directories.
 * \param size	The total size (in bytes) to be displayed.
 * \param count	The number of directories on the disk.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::setDiskOneDirectories(const qint64 &size, const qint64 &count)
{
	ui.mpDirectoriesOneEdit->
		setText(Humax::SizeAsString(size/*, 9, 2*/) + QString(" [%1]").arg(count));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the Disk One File Information.
 *
 * Updates the information showing the total size of the files on
 * disk one and the number of files.
 * \param size	The total size (in bytes) to be displayed.
 * \param count	The number of files on the disk.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::setDiskOneFiles(const qint64 &size, const qint64 &count)
{
	ui.mpFilesOneEdit->
		setText(Humax::SizeAsString(size/*, 9, 2*/) + QString(" [%1]").arg(count));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the Disk One Entry (Files and Directories) Information.
 *
 * Updates the information showing the total size of the items (Files and
 * Directories) on disk one and the number of items.
 * \param size	The total size (in bytes) to be displayed.
 * \param count	The number of items on the disk.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::setDiskOneEntries(const qint64 &size, const qint64 &count)
{
	ui.mpEntriesOneEdit->
		setText(Humax::SizeAsString(size/*, 9, 2*/) + QString(" [%1]").arg(count));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Update the Disk Two Space information.
 *
 * Updates the disk two space information, including the total, free, and
 * used space as well as the Disk Bar.
 * \param free	The amount of free space on the disk.
 * \param total	The total amount of space on the disk.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::setDiskTwoSpace(const qint64 &free, const qint64 &total)
{
	ui.mpUsedSpaceTwoEdit->setText(Humax::SizeAsString(total - free/*, 9, 2*/));

	ui.mpFreeSpaceTwoEdit->setText(Humax::SizeAsString(free/*, 9, 2*/));

	ui.mpTotalSpaceTwoEdit->setText(Humax::SizeAsString(total/*, 9, 2*/));

	ui.mpDiskTwoBar->setDiskSpace(free, total);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the Disk Two Directory Information.
 *
 * Updates the information showing the total size of the directories on
 * disk two and the number of directories.
 * \param size	The total size (in bytes) to be displayed.
 * \param count	The number of directories on the disk.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::setDiskTwoDirectories(const qint64 &size, const qint64 &count)
{
	ui.mpDirectoriesTwoEdit->
		setText(Humax::SizeAsString(size/*, 9, 2*/) + QString(" [%1]").arg(count));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the Disk Two File Information.
 *
 * Updates the information showing the total size of the files on
 * disk two and the number of files.
 * \param size	The total size (in bytes) to be displayed.
 * \param count	The number of files on the disk.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::setDiskTwoFiles(const qint64 &size, const qint64 &count)
{
	ui.mpFilesTwoEdit->
		setText(Humax::SizeAsString(size/*, 9, 2*/) + QString(" [%1]").arg(count));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the Disk Two Entry (Files and Directories) Information.
 *
 * Updates the information showing the total size of the items (Files and
 * Directories) on disk one and the number of items.
 * \param size	The total size (in bytes) to be displayed.
 * \param count	The number of items on the disk.
 */
////////////////////////////////////////////////////////////////////////////////
void DeviceInfoDialog::setDiskTwoEntries(const qint64 &size, const qint64 &count)
{
	ui.mpEntriesTwoEdit->
		setText(Humax::SizeAsString(size/*, 9, 2*/) + QString(" [%1]").arg(count));
}

////////////////////////////////////////////////////////////////////////////////
