////////////////////////////////////////////////////////////////////////////////
// \class FileReplaceDialog FileReplaceDialog.h
//
// \version 1.00
// \date 12-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief Replace File(s) Dialog Class.
//
// Replace file(s) dialog to request whether the application should overwrite
// files when needed.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
12-07-06 1.00 Initial release.

*****/
#include "FileReplaceDialog.h"

#include <QFileIconProvider>

#include "HumaxGlobals.h"

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the -- Dialog Class.
 */
////////////////////////////////////////////////////////////////////////////////
FileReplaceDialog::FileReplaceDialog(QWidget *parent)
: QDialog(parent)
, mReturnState(CANCEL)
{
	ui.setupUi(this);

	QFileIconProvider iconProvider;
	QPixmap pixmap = iconProvider.icon(QFileIconProvider::File).
		pixmap(QSize(24, 32));

	ui.mpTargetIcon->setPixmap(pixmap.scaledToHeight(32));
	ui.mpSourceIcon->setPixmap(pixmap.scaledToHeight(32));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Set the displayed filename.
 *
 * Sets the filename to be displayed in the file replace dialog.
 * \param filename The name of the file.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::SetFilename(const QString &filename)
{
	ui.mpFileLabel->
		setText(tr("This folder already contains a file named '%1'.").arg(filename));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Set the target file information.
 *
 * Sets the target file size to be displayed on the dialog.
 * \param size The size (in bytes) of the target file.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::SetTargetInfo(qint64 size)
{
	ui.mpTargetSizeLabel->setText(Humax::SizeAsString(size));
}

void FileReplaceDialog::SetTargetInfo(QString size)
{
	ui.mpTargetSizeLabel->setText(size);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Set the source file information.
 *
 * Sets the source file size to be displayed on the dialog.
 * \param size The size (in bytes) of the source file.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::SetSourceInfo(qint64 size)
{
	ui.mpSourceSizeLabel->setText(Humax::SizeAsString(size));
}

void FileReplaceDialog::SetSourceInfo(QString size)
{
	ui.mpSourceSizeLabel->setText(size);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Set display to single file mode.
 *
 * If only one (single) file is being copied then there is no need for a
 * 'Yes To All' button or a Cancel button. Setting the dialog to single 
 * file mode hides these buttons to give an Overwrite? Yes No dialog.
 * \param single boolean flag, true if single file mode.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::SetSingleFileReplace(const bool &single)
{
	if( single )
	{
		ui.mpYesToAllButton->hide();
		ui.mpCancelButton->hide();
	}
	else
	{
		ui.mpYesToAllButton->show();
		ui.mpCancelButton->show();
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called when the user presses the Yes button.
 *
 * This function is called when the Yes button is pressed, it sets
 * the mReturnState flag to YES then closes the dialog.
 * \sa ReturnState and mReturnState.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::on_mpYesButton_clicked()
{
	mReturnState = YES;
	accept();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called when the user presses the Yes To All button.
 *
 * This function is called when the Yes To All button is pressed, it sets
 * the mReturnState flag to YESTOALL then closes the dialog.
 * \sa ReturnState and mReturnState.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::on_mpYesToAllButton_clicked()
{
	mReturnState = YESTOALL;
	accept();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called when the user presses the No button.
 *
 * This function is called when the No button is pressed, it sets
 * the mReturnState flag to NO then closes the dialog.
 * \sa ReturnState and mReturnState.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::on_mpNoButton_clicked()
{
	mReturnState = NO;
	reject();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called when the user presses the Cancel button.
 *
 * This function is called when the Cancel button is pressed, it sets
 * the mReturnState flag to CANCEL then closes the dialog.
 * \sa ReturnState and mReturnState.
 */
////////////////////////////////////////////////////////////////////////////////
void FileReplaceDialog::on_mpCancelButton_clicked()
{
	mReturnState = CANCEL;
	reject();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Opens the dialog modally and returns the result.
 *
 * The is a standard Qt QDialog function which has been overridden.
 * The function opens the dialog modally and waits for it to be closed
 * by calling the overridden QDialog::exec(). When this returns, this
 * function substitutes the return value with the return state set by the
 * closing button.
 * \return The return state depending on which button is pressed.
 * \sa ReturnState and mReturnState.
 */
////////////////////////////////////////////////////////////////////////////////
int FileReplaceDialog::exec()
{
	QDialog::exec();
	return mReturnState;
}

////////////////////////////////////////////////////////////////////////////////
