////////////////////////////////////////////////////////////////////////////////
// \class HumaxInterface HumaxInterface.cpp
//
// \version 1.00
// \date 04-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief Class provided to interface with the USB interface.
//
// This class is provided to give humax specific functionality to the GUI by
// supplying commands to and retrieving data from the USB interface.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#include "HumaxInterface.h"

#include <QString>

#include <string.h>
#include <time.h>

extern struct usb_device *mpHumaxDevice; // Handle to the USB device

////////////////////////////////////////////////////////////////////////////////
///// TRACE FUNCTION                                                       /////
////////////////////////////////////////////////////////////////////////////////

#define USB_TRACE(status, func, id, msg, data, dataLen) if(TraceConsole() || TraceLogFile()) \
			usbTrace(TraceConsole(), TraceLogFile(), status, func, id, msg, data, dataLen);

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the interface Class.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxInterface::HumaxInterface()
: UsbInterface()
, dCBWTag(0)
{
	// Randomly set dCBWTag value for USB bulk data transfer
	time_t t;
	time(&t);
	dCBWTag = (int)(t & 0xffff);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Destructor.
 *
 * Clean up any memory usage and destroy the class.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxInterface::~HumaxInterface()
{
	if( mpUsbHandle != NULL )
		humaxDisconnect();
}

////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// HUMAX USB CONNECT / DISCONNECT / BULK READ / BULK WRITE FUNCTIONS    /////
///// THESE FUNCTIONS INTERFACE DIRECTLY WITH THE LIBUSB-WIN32 DRIVER      /////
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/*! \brief Connects to the device.
 *
 * Connects to the device via the USB interface using the LIBUSB(-WIN32)
 * drivers.
 * \return	If successfully connected returns the USB device handle used
 *			by all other functions in this program. if failed, just
 *			returns NULL.
 */
////////////////////////////////////////////////////////////////////////////////
usb_dev_handle * HumaxInterface::humaxConnect()
{
	// Initialise USB API
	usb_init();
	USB_TRACE(true, "humaxConnect", 1, "USB API initialised", NULL, 0);

	// Loop for a maximum number of attempts (currently 1).
	// Note: this was set at 3 but attempting to force the Humax to do something it doesn't want to
	// do can crash the poor thing! So currently set to 1 just to make it's life a bit easier.
	usb_dev_handle * dh = NULL;
	bool error = true;
	for(int a = 0; error && a < 1; a++)
	{
		// If second or third pass then reset the device and try again (currently disabled - see above)
		if(a > 0 && dh != NULL)
		{
			if(usb_reset(dh) != 0)
			{
				USB_TRACE(false, "humaxConnect", 2, "Device reset", NULL, 0);
				return false;
			}
			USB_TRACE(true, "humaxConnect", 2, "Device reset", NULL, 0);
///			Sleep(1000);
		}

		// Enumerate busses and devices
		usb_find_busses();
		usb_find_devices();

		// Find the device. Humax PVR9200T VendorID = 0x02ad, ProductID = 0x138c
		// The Ozzie (G'day mate) version of the Humax (PVR Smart) might use a different ProductID for this
		// so it's easy enough to change this if necessary. I'm not in Oz so can't try this out.
		struct usb_bus * bus;
		struct usb_device * dev;
		for(bus = usb_get_busses(); !mpHumaxDevice && bus; bus = bus->next)
			for(dev = bus->devices; !mpHumaxDevice && dev; dev = dev->next)
				if(dev->descriptor.idVendor == 0x02ad && dev->descriptor.idProduct == 0x138c)
					mpHumaxDevice = dev;
		if(!mpHumaxDevice)
		{
			USB_TRACE(false, "humaxConnect", 3, "Find Humax PVR9200T device", NULL, 0);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 3, "Find Humax PVR9200T device", NULL, 0);

		// Open the device
		dh = usb_open(mpHumaxDevice);
		if(dh == NULL)
		{
			USB_TRACE(false, "humaxConnect", 4, "Open Humax PVR9200T device", NULL, 0);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 4, "Open Humax PVR9200T device", NULL, 0);

		// Select configuration 0x01
		if(usb_set_configuration(dh, 0x01))
		{
			USB_TRACE(false, "humaxConnect", 5, "Select configuration 0x01", NULL, 0);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 5, "Select configuration 0x01", NULL, 0);

		// Claim interface 0x00
		if(usb_claim_interface(dh, 0x00))
		{
			USB_TRACE(false, "humaxConnect", 6, "Claim interface 0x01", NULL, 0);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 6, "Claim interface 0x01", NULL, 0);

		// Clear any halt conditions on the bulk transfer endpoint 0x01 (Humax -> Host)
		if(usb_clear_halt(dh, 0x01))
		{
			USB_TRACE(false, "humaxConnect", 7, "Clear halt condition on endpoint 0x01", NULL, 0);
			usb_release_interface(dh, 0x00);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 7, "Clear halt condition on endpoint 0x01", NULL, 0);

		// Clear any halt conditions on the bulk transfer endpoint 0x81 (Host -> Humax)
		if(usb_clear_halt(dh, 0x81))
		{
			USB_TRACE(false, "humaxConnect", 8, "Clear halt condition on endpoint 0x81", NULL, 0);
			usb_release_interface(dh, 0x00);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 8, "Clear halt condition on endpoint 0x81", NULL, 0);

		// Send vendor connect request 0xcf to device. Note that when this is done, the Humax 'greys-out' the
		// Image and MP3 functions on the recording menus.
		if(usb_control_msg(dh, 0x40, 0xcf, 0x00, 0x00, NULL, 0, 1000) != 0)
		{
			USB_TRACE(false, "humaxConnect", 9, "Send connect request (0xcf) to device", NULL, 0);
			usb_release_interface(dh, 0x00);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 9, "Send connect request (0xcf) to device", NULL, 0);

		// Receive vendor connect ack 0xcf from device
		char buf[2];
		if(usb_control_msg(dh, 0xc0, 0xcf, 0x00, 0x00, buf, 2, 1000) != 2 || (unsigned char)buf[0] != 0xcf || buf[1] != 0x00)
		{
			USB_TRACE(false, "humaxConnect", 10, "Receive connect ack (0xcf) from device", NULL, 0);
			usb_release_interface(dh, 0x00);
			continue;
		}
		USB_TRACE(true, "humaxConnect", 10, "Receive connect ack (0xcf) from device", NULL, 0);

		// Force-clear connect error flag to ensure connect loop only occurs once (see comments at top of loop)
		error = false;
	}

	// If error and device is open then close it
	if(error && dh)
	{
		usb_close(dh);
		dh = NULL;
		USB_TRACE(true, "humaxConnect", 11, "Device closed", NULL, 0);
	}

	// Return the device handle as success
	return dh;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Disconnects from the device.
 * 
 * Disconnects from the device via the USB interface.
 * \return Boolean, true - success, false - failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxDisconnect()
{
	// Send vendor disconnect request 0xce to the device. When this completes, the Image and MP3 options in
	// the recording menu re-appear again
	bool error = false;
	if( usb_control_msg(mpUsbHandle, 0x40, 0xce, 0x00, 0x00, NULL, 0, 1000) != 0 )
	{
		error = true;
		USB_TRACE(false, "humaxDisconnect", 1, "Send disconnect request (0xce) to device", NULL, 0);
	}
	USB_TRACE(true, "humaxDisconnect", 1, "Send disconnect request (0xce) to device", NULL, 0);

	// Receive vendor disconnect ack 0xce from device
	char buf[2];
	if( !error && usb_control_msg(mpUsbHandle, 0xc0, 0xce, 0x00, 0x00, buf, 2, 1000) != 2 || (unsigned char)buf[0] != 0xce || buf[1] != 0x00 )
	{
		error = true;
		USB_TRACE(false, "humaxDisconnect", 2, "Receive disconnect ack (0xce) from device", NULL, 0);
	}
	USB_TRACE(true, "humaxDisconnect", 2, "Receive disconnect ack (0xce) from device", NULL, 0);

	// Release interface 0x00
	if( usb_release_interface(mpUsbHandle, 0x00) )
	{
		error = true;
		USB_TRACE(false, "humaxDisconnect", 3, "Release interface 0x00", NULL, 0);
	}
	USB_TRACE(true, "humaxDisconnect", 3, "Release interface 0x00", NULL, 0);

	// Close the device
	if( usb_close(mpUsbHandle) )
	{
		error = true;
		USB_TRACE(false, "humaxDisconnect", 4, "Close device", NULL, 0);
	}
	USB_TRACE(true, "humaxDisconnect", 4, "Close device", NULL, 0);

	if( !error )
		mpUsbHandle = NULL;

	// Return disconnect error status
	return !error;
}

////////////////////////////////////////////////////////////////////////////////
///// HUMAX USB BULK TRANSFER COMMAND INTERFACE FUNCTIONS                  /////
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/*! \brief Gets the number of directory entries
 *
 * Gets the number of directory entries in the current device directory.
 * HMC Command: [J].
 * \return	The total number of directory entries in the current device
 *			directory (min 0, max 1024)
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxInterface::humaxGetNoDirEntries()
{
	int noEntries = 0;
	char buf[13];
	buf[0] = 'J';
	memset(&buf[1], 0x00, 7);
	if(!writeBulkCBW("humaxGetNoDirEntries", 1, "Write CBW", incCBWTag(), 4, 0x80, buf, 8))
		return -1;

	if(readBulkData("humaxGetNoDirEntries", 2, "Read bulk data", buf, 4) != 4)
		return -1;
	noEntries = ((unsigned char)buf[2] << 8) + (unsigned char)buf[3];

	if(!readBulkCSW("humaxGetNoDirEntries", 3, "Read CSW", buf) || buf[12] != 0)
		return -1;

	// Return no entries
	return noEntries;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Reads the directory entries.
 *
 * Reads the directory entries from the device current directory.
 * HMC Command: [K].
 * \param noEntries		Total number of directory entries to read
 * \param doDualRead	True if a read, pause, read operation to determine
 *						the 'isRecording' status of Humax video files.
 * \param data			Output data buffer into which to read the raw
 *						USB data.
 * \param sizeChanged	Output boolean array of 'noEntries' in size set
 *						to true for Humax video files currently being
 *						recorded.
 * \return				Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxGetDirEntries(int noEntries, bool doDualRead, char *data, bool *sizeChanged)
{
	// Clear size changed flags
	for(int a = 0; a < noEntries; a++)
		sizeChanged[a] = false;

	// If no entries requested then just exit
	if(noEntries == 0)
		return true;

	// Do primary read of directory content data
	char buf[13];
	int dataSize = 0x48 * noEntries;
	buf[0] = 'K';
	memset(&buf[1], 0x00, 7);
	if(!writeBulkCBW("humaxGetDirEntries", 1, "Write CBW", incCBWTag(), dataSize, 0x80, buf, 8))
		return false;

	if(readBulkData("humaxGetDirEntries", 2, "Read bulk data", data, dataSize) != dataSize)
		return false;

	if(!readBulkCSW("humaxGetDirEntries", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// If requested a dual read of the directory contents, read again after a short pause to determine
	// if any of the file sizes have changed and update the 'sizeChanged' flag accordingly
	if(doDualRead)
	{
		// Pause 800 milliseconds
///		Sleep(DUAL_READ_SLEEP_MS);

		// Do second read
		char * data2 = new char[dataSize];
		buf[0] = 'K';
		memset(&buf[1], 0x00, 7);
		if(!writeBulkCBW("humaxGetDirEntries", 4, "Write CBW", incCBWTag(), dataSize, 0x80, buf, 8))
		{
			free(data2);
			return false;
		}

		if(readBulkData("humaxGetDirEntries", 5, "Write bulk data", data2, dataSize) != dataSize)
		{
			free(data2);
			return false;
		}

		if(!readBulkCSW("humaxGetDirEntries", 6, "Read CSW", buf) || buf[12] != 0)
		{
			free(data2);
			return false;
		}

		// Scan each entry to determine if file size has changed and update 'sizeChanged' flag for the entry accordingly
		for(int a = 0; a < noEntries; a++)
		{
			__int64 size = (((__int64)((unsigned char)data[a * 0x48 + 0x40]) * 1000000000) + (unsigned char)data[a * 0x48 + 0x44] + ((unsigned char)data[a * 0x48 + 0x45] * 256) + ((unsigned char)data[a * 0x48 + 0x46] * 65536) + ((unsigned char)data[a * 0x48 + 0x47] * 16777216));
			__int64 size2 = (((__int64)((unsigned char)data2[a * 0x48 + 0x40]) * 1000000000) + (unsigned char)data2[a * 0x48 + 0x44] + ((unsigned char)data2[a * 0x48 + 0x45] * 256) + ((unsigned char)data2[a * 0x48 + 0x46] * 65536) + ((unsigned char)data2[a * 0x48 + 0x47] * 16777216));
			sizeChanged[a] = (size != size2);
		}

		// Copy data from the second read to the main buffer because it has the latest info
		memcpy(data, data2, dataSize);
		free(data2);
	}

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the free and total disk space.
 *
 * Get the free and total disk space from the device for the current
 * disk partition.
 * HMC Command: [L].
 * \param diskSpace	Output array of 2 integers to receive the 'free'
 *					and 'total' disk space values (set on exit).
 *					[0] 'free' disk space [1] 'total' disk space.
 * \return				Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxGetDiskSpace(qint64 *diskSpace)
{
	char buf[13];
	char buf2[0x10];
	buf[0] = 'L';
	memset(&buf[1], 0x00, 7);
	if(!writeBulkCBW("humaxGetDiskSpace", 1, "Write CBW", incCBWTag(), 0x10, 0x80, buf, 8))
		return false;

	if(readBulkData("humaxGetDiskSpace", 2, "Write bulk data", buf2, 0x10) != 0x10)
		return false;

	if(!readBulkCSW("humaxGetDiskSpace", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return the free and total disk space in terms of number of bytes
	diskSpace[0] = (((__int64)((unsigned char)buf2[0]) * 1000000000) + (unsigned char)buf2[4] + ((unsigned char)buf2[5] * 256) + ((unsigned char)buf2[6] * 65536) + ((unsigned char)buf2[7] * 16777216));
	diskSpace[1] = (((__int64)((unsigned char)buf2[8]) * 1000000000) + (unsigned char)buf2[12] + ((unsigned char)buf2[13] * 256) + ((unsigned char)buf2[14] * 65536) + ((unsigned char)buf2[15] * 16777216));

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Initiates a USB "M1" transfer.
 *
 * Initiates a USB "M1" device to host file transfer.
 * HMC Command: [M1].
 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
 *					used throughout each transfer process.
 * \param filePath	Humax filepath string of file to be transferred
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxXferBeginDevToHost(int dCBWTag, char * filePath)
{
	char buf[13];
	buf[0] = 'M';
	buf[1] = 0x01;
	memset(&buf[2], 0x00, 6);
	int filePathLen = strlen(filePath) - 1;
	if(!writeBulkCBW("humaxXferBeginDevToHost", 1, "Write CBW", dCBWTag, filePathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxXferBeginDevToHost", 2, "Write bulk data", filePath + 2, filePathLen))
		return false;

	if(!readBulkCSW("humaxXferBeginDevToHost", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Initiates a USB "M2" transfer.
 *
 * Initiates a USB "M2" host to device file transfer.
 * HMC Command: [M2].
 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
 *					used throughout each transfer process.
 * \param filePath	Humax filepath string of the new file to be
 *					transferred and created on the Humax.
 * \param fileSize	Size of the new target file to be created on
 *					the Humax.
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxXferBeginHostToDev(int dCBWTag, char * filePath, qint64 fileSize)
{
	char buf[13];
	buf[0] = 'M';
	buf[1] = 0x02;
	buf[2] = 0;
	buf[3] = (unsigned char)(fileSize / 16777216);
	buf[4] = (unsigned char)(fileSize / 65536 % 256);
	buf[5] = 0;
	buf[6] = (unsigned char)(fileSize % 256);
	buf[7] = (unsigned char)(fileSize / 256 % 256);
	int filePathLen = strlen(filePath) - 1;
	if(!writeBulkCBW("humaxXferBeginHostToDev", 1, "Write CBW", dCBWTag, filePathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxXferBeginHostToDev", 2, "Write bulk data", filePath + 2, filePathLen))
		return false;

	if(!readBulkCSW("humaxXferBeginHostToDev", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Ends a file transfer.
 *
 * Ends an "M1" or "M2" USB file transfer.
 * HMC Command: [N].
 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
					used throughout each transfer process.
 * \param filePath	Humax filepath string of file transfer to end.
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxXferEnd(int dCBWTag, char * filePath)
{
	char buf[13];
	buf[0] = 'N';
	memset(&buf[1], 0x00, 7);
	int filePathLen = strlen(filePath) - 1;
	if(!writeBulkCBW("humaxXferEnd", 1, "Write CBW", dCBWTag, filePathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxXferEnd", 2, "Write bulk data", filePath + 2, filePathLen))
		return false;

	if(!readBulkCSW("humaxXferEnd", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// End of transfer so increment tag
	incCBWTag();

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Read a block of data for "M1" file transfers.
 *
 * Read a block of data for "M1" file transfers from device to host.
 * HMC Command: [O].
 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
					used throughout each transfer process.
 * \param data		Output the data buffer into which receive file data
 *					from the Humax via USB.
 * \param noBytes	Output the size of the data block to read from the
 *					Humax.
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxInterface::humaxXferNextDevToHost(int dCBWTag, char * data, int noBytes)
{
	char buf[13];
	buf[0] = 'O';
	memset(&buf[1], 0x00, 7);
	if(!writeBulkCBW("humaxXferNextDevToHost", 1, "Write CBW", dCBWTag, noBytes, 0x80, buf, 8))
		return 0;

	int cbRet = readBulkData("humaxXferNextDevToHost", 2, "Read bulk data", data, noBytes);
	if(cbRet != noBytes)
		return cbRet;

	if(!readBulkCSW("humaxXferNextDevToHost", 3, "Read CSW", buf) || buf[12] != 0)
		return 0;

	// Return success
	return cbRet;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Read a block of data for "M2" transfers.
 *
 * Read a block of data for "M2" file transfers from host to device.
 * HMC Command: [P].
 * \param dCBWTag	USB Bulk Transfer command block unique TagID value
					used throughout each transfer process.
 * \param data		Contains the data to be transferred to the Humax.
 * \param noBytes	Contains the size of the data in the data buffer
 *					to be transferred to the Humax.
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxXferNextHostToDev(int dCBWTag, char * data, int noBytes)
{
	char buf[13];
	buf[0] = 'P';
	memset(&buf[1], 0x00, 7);
	if(!writeBulkCBW("humaxXferNextHostToDev", 1, "Write CBW", dCBWTag, noBytes, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxXferNextHostToDev", 2, "Write bulk data", data, noBytes))
		return false;

	if(!readBulkCSW("humaxXferNextHostToDev", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Rename a file on the device.
 *
 * Rename a file on the device.
 * HMC Command: [R].
 * \param oldFilePath	The Humax filepath of the file or directory to
 *						be renamed.
 * \param newFilePath	The Humax filepath of the new name for the file
 *						or directory.
 * \return				Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxRenameFile(char *oldFilePath, char *newFilePath)
{
	char buf[13];
	char buf2[MAX_PATH * 2];
	buf[0] = 'R';
	memset(&buf[1], 0x00, 7);
	int oldFilePathLen = strlen(oldFilePath) - 1;
	int newFilePathLen = strlen(newFilePath) - 1;
	buf[1] = (unsigned char)((oldFilePathLen & 0xff00) >> 8);
	buf[2] = (unsigned char)(oldFilePathLen & 0xff);
	buf[3] = (unsigned char)((newFilePathLen & 0xff00) >> 8);
	buf[4] = (unsigned char)(newFilePathLen & 0xff);
	memcpy(buf2, oldFilePath + 2, oldFilePathLen);
	memcpy(buf2 + oldFilePathLen, newFilePath + 2, newFilePathLen);
	if(!writeBulkCBW("humaxRenameFile", 1, "Write CBW", incCBWTag(), oldFilePathLen + newFilePathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxRenameFile", 2, "Write bulk data", buf2, oldFilePathLen + newFilePathLen))
		return false;

	if(!readBulkCSW("humaxRenameFile", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Create a new directory on the device.
 *
 * Create a new directory on the device.
 * HMC Command: [S].
 * \param dirPath	The Humax filepath of the new directory to be created
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxCreateDir(char *dirPath)
{
	char buf2[MAX_PATH];
///	strcpy_s(buf2, dirPath);
	strcpy(buf2, dirPath);
	if(strlen(buf2) > 3 && buf2[strlen(buf2) - 1] == '/')
		buf2[strlen(buf2) - 1] = 0;
	char buf[13];
	buf[0] = 'S';
	memset(&buf[1], 0x00, 7);
	int dirPathLen = strlen(buf2) - 1;
	if(!writeBulkCBW("humaxCreateDir", 1, "Write CBW", incCBWTag(), dirPathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxCreateDir", 2, "Write bulk data", buf2 + 2, dirPathLen))
		return false;

	if(!readBulkCSW("humaxCreateDir", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Delete a file on the device
 *
 * Delete a file on the device
 * HMC Command: [T].
 * \param filePath	The Humax filepath of the file or directory to
 *					be deleted.
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxDeleteFile(char *filePath)
{
	// First attempt to delete the file. A special case exists for deleting directories. If a directory on the Humax
	// 'fails' then deleting the directory can be problematic due to bugs in the Humax file system. Quite often, if
	// you create a new directory on the Humax, then immediately rename it, it fails such that you cannot change
	// directory to it. When this occurs you have to delete the directory then recreate it again to get it working!
	// Problem is that when you try to delete the offending directory it doesn't always immediately work! So, this
	// hack (also employed the Humax Media eLinker software) first tries to delete the directory by specifying its
	// closing '/' character on the end of its name. If that fails, then it tries again without the closing '/'
	// charater on its name. Usually one of the two operations will work.
	char buf[13];
	buf[0] = 'T';
	memset(&buf[1], 0x00, 7);
	int filePathLen = strlen(filePath) - 1;
	if(!writeBulkCBW("humaxDeleteFile", 1, "Write CBW", incCBWTag(), filePathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxDeleteFile", 2, "Write bulk data", filePath + 2, filePathLen))
		return false;

	bool status = (readBulkCSW("humaxDeleteFile", 3, "Read CSW", buf) && buf[12] == 0);

	// If not a directory then exit with status (i.e. no need to try the hack mentioned above!)
	if(filePath[strlen(filePath) - 1] != '/')
		return status;

	// This is a directory delete. If status ok then return ok
	if(status)
		return true;

	// Directory delete failed so remove the trailing '/' character from the name and try again!
	char buf2[MAX_PATH];
///	strcpy_s(buf2, filePath);
	strcpy(buf2, filePath);
	buf2[strlen(buf2) - 1] = 0;
	filePathLen = strlen(buf2) - 1;
	if(!writeBulkCBW("humaxDeleteFile", 4, "Write CBW", incCBWTag(), filePathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxDeleteFile", 5, "Write bulk data", buf2 + 2, filePathLen))
		return false;

	if(!readBulkCSW("humaxDeleteFile", 6, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Formats the MP3/Image partition on the device.
 *
 * Formats the MP3/Image partition on the device and creates the initial
 * MP3' and 'Image' dirs in the root directory. Note that no known
 * interface currently exists to do the same for the Video disk partition
 * no 2 though!
 * HMC Command: [U].
 * \return	Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxFormatDiskOne()
{
	char buf[13];
	buf[0] = 'U';
	memset(&buf[1], 0x00, 7);
	if( !writeBulkCBW("humaxFormatDisk", 1, "Write CBW", incCBWTag(), 0, 0x00, buf, 8) )
		return false;

	if( !readBulkCSW("humaxFormatDisk", 2, "Read CSW", buf) || buf[12] != 0 )
		return false;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Sets the current directory on the device
 *
 * Sets the current directory on the device
 * HMC Command: [V].
 * \param dirPath	The Humax filepath of the new directory to be
 *					changed to.
 * \return			Boolean, operation succeeded, operation failed.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxSetDirectory(char *dirPath)
{
	char buf[13];
	buf[0] = 'V';
	memset(&buf[1], 0x00, 7);
	int dirPathLen = strlen(dirPath) - 1;
	if(!writeBulkCBW("humaxSetDirectory", 1, "Write CBW", incCBWTag(), dirPathLen, 0x00, buf, 8))
		return false;

	if(!writeBulkData("humaxSetDirectory", 2, "Write bulk data", dirPath + 2, dirPathLen))
		return false;

	if(!readBulkCSW("humaxSetDirectory", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Retrieves the total number of accessible disk partitions on
 *			the device.
 *
 * Retrieves the total number of accessible disk partitions on the device
 * (should always return 2) but as we already know that this software just
 * assumes that value and therefore doesn't use this function! Function
 * included for the sake of completeness anyway.
 * HMC Command: [W].
 * \return The number of partitions present (always 2 for the PVR9200T).
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxInterface::humaxGetNoPartitions()
{
	int noPartitions = 0;
	char buf[13];
	buf[0] = 'W';
	memset(&buf[1], 0x00, 7);
	if(!writeBulkCBW("humaxGetNoPartitions", 1, "Write CBW", incCBWTag(), 4, 0x80, buf, 8))
		return -1;

	if(readBulkData("humaxGetNoPartitions", 2, "Read bulk data", buf, 4) != 4)
		return -1;
	noPartitions = (buf[0] << 24) + (buf[1] << 16) + (buf[2] << 8) + buf[3];

	if(!readBulkCSW("humaxGetNoPartitions", 3, "Read CSW", buf) || buf[12] != 0)
		return -1;

	return noPartitions;
}

////////////////////////////////////////////////////////////////////////////////
	/*! \brief Sets the current disk partition on the device
	 *
	 * Sets the current disk partition on the device
	 * HMC Command: [X].
	 * \param diskNo	ASCII '1' for the Image/MP3 disk and ASCII '2' for
	 *					the Video disk.
	 * \return			Boolean, operation succeeded, operation failed.
	 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxInterface::humaxSetDiskNo(char diskNo)
{
	char buf[13];
	buf[0] = 'X';
	memset(&buf[1], 0x00, 7);
	if(!writeBulkCBW("humaxSetDiskNo", 1, "Write CBW", incCBWTag(), 4, 0x00, buf, 8))
		return false;

	// This USB interface requires numeric value [0] for Image/MP3 disk or [1] for the Video disk
	buf[0] = (char)(diskNo - '1');
	memset(&buf[1], 0x00, 3);
	if(!writeBulkData("humaxSetDiskNo", 2, "Write bulk data", buf, 4))
		return false;

	if(!readBulkCSW("humaxSetDiskNo", 3, "Read CSW", buf) || buf[12] != 0)
		return false;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
///// MY ADDITIONAL FUNCTIONS:                                             /////
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the manufacturer text from the device.
 *
 * Get the manufacturer information text from the humax device.
 * \return The manufacturer string.
 */
////////////////////////////////////////////////////////////////////////////////
QString HumaxInterface::humaxManufacturer()
{
	const int buffLen = 128;
	char humaxBuff[buffLen];
	int len = 0;

	memset(humaxBuff, 0x00, buffLen);
	len = usb_get_string_simple(mpUsbHandle,
		mpHumaxDevice->descriptor.iManufacturer, humaxBuff, buffLen);
	humaxBuff[len] = 0x00;
	return QString(humaxBuff);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the product text from the device.
 *
 * Get the product information text from the humax device.
 * \return The product string.
 */
////////////////////////////////////////////////////////////////////////////////
QString HumaxInterface::humaxProduct()
{
	const int buffLen = 128;
	char humaxBuff[buffLen];
	int len = 0;

	len = usb_get_string_simple(mpUsbHandle,
		mpHumaxDevice->descriptor.iProduct, humaxBuff, buffLen);
	humaxBuff[len] = 0x00;
	return QString(humaxBuff);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the hardware version text from the device.
 *
 * Get the hardware version information text from the humax device.
 * \return The hardware version string.
 */
////////////////////////////////////////////////////////////////////////////////
QString HumaxInterface::humaxHardwareVersion()
{
	return QString("V%1.%2").
		arg((mpHumaxDevice->descriptor.bcdDevice & 0xff00) >> 8).
		arg(mpHumaxDevice->descriptor.bcdDevice & 0xff);

}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the serial number text from the device.
 *
 * Get the serial number information text from the humax device.
 * \return The serial number string.
 */
////////////////////////////////////////////////////////////////////////////////
QString HumaxInterface::humaxSerialNumber()
{
	const int buffLen = 128;
	char humaxBuff[buffLen];
	int len = 0;

	len = usb_get_string_simple(mpUsbHandle,
		mpHumaxDevice->descriptor.iSerialNumber, humaxBuff, buffLen);
	humaxBuff[len] = 0x00;
	return QString(humaxBuff);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/*! \brief Increment the USB Bulk Transfer command block.
 *
 * Increment the USB Bulk Transfer command block TagID value and
 * return the new value.
 * \return The new USB Command Block TagID value used in USB operations.
 * \sa dCBWTag.
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxInterface::incCBWTag()
{
	dCBWTag = dCBWTag + 3;
	return dCBWTag;
}

////////////////////////////////////////////////////////////////////////////////
