////////////////////////////////////////////////////////////////////////////////
// \class HumaxItem HumaxItem.cpp
//
// \version 1.00
// \date 04-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
//
// \brief A Class to store item data for the HumaxModel class. 
//
// This class stores data for an individual item in the HumaxModel class.
//
// This file is based on part of the example classes of the Qt Toolkit.
//
// This file may be used under the terms of the GNU General Public
// License version 2.0 as published by the Free Software Foundation
// and appearing in the file LICENSE.GPL included in the packaging of
// this file.  Please review the following information to ensure GNU
// General Public Licensing requirements will be met:
// http://www.trolltech.com/products/qt/opensource.html
//
// If you are unsure which license is appropriate for your use, please
// review the following information:
// http://www.trolltech.com/products/qt/licensing.html or contact the
// sales department at sales@trolltech.com.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#include "HumaxItem.h"

#include <QStringList>

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Item Class.
 * \param data		Row data for this item.
 * \param parent	The parent of this item.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxItem::HumaxItem(const QList<QVariant> &data, HumaxItem *parent)
: mDir(FALSE)
, mRecording(FALSE)
{
	mpParentItem = parent;
	mItemData = data;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Destructor.
 *
 * Clean up any memory usage and destroy the class.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxItem::~HumaxItem()
{
	qDeleteAll(mChildItems);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Append a child item to this item.
 *
 * Add a HumaxItem to the end of the list of child items.
 * \param child The child item to be added.
 */
////////////////////////////////////////////////////////////////////////////////
void HumaxItem::appendChild(HumaxItem *item)
{
	mChildItems.append(item);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the child item at the given row.
 *
 * Get the child HumaxItem at the supplied row index.
 * \param row The index of the child item.
 * \return The child item at the index or 0 if no item found.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxItem *HumaxItem::child(int row)
{
    return mChildItems.value(row);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the number of child items.
 *
 * Returns the number of children attached to this model item.
 * \return The number of children.
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxItem::childCount() const
{
    return mChildItems.count();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the number of columns.
 *
 * Returns the number of data columns stored in the model item.
 * \return The number of data columns.
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxItem::columnCount() const
{
	return mItemData.count();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the data for a given column.
 *
 * Returns the data stored in the column requested.
 * \param column The data index of the item.
 * \return The data stored in the column.
 */
////////////////////////////////////////////////////////////////////////////////
QVariant HumaxItem::data(int column) const
{
	return mItemData.value(column);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the row of this item.
 *
 * Returns the row index of this item.
 * \return The row index.
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxItem::row() const
{
    if (mpParentItem)
        return mpParentItem->mChildItems.indexOf(const_cast<HumaxItem*>(this));

    return 0;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the parent item.
 *
 * Returns the parent model item of this item.
 * \return The parent item.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxItem *HumaxItem::parent()
{
	return mpParentItem;
}
/*
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
bool TreeItemLessThan(const TreeItem *a, const TreeItem *b)
{
	if( a->isDirectory() && !b->isDirectory() )
	{
		return true;
	}
	else if( !a->isDirectory() && b->isDirectory() )
	{
		return false;
	}
	else
	{
		return (a->data(0).toString() < b->data(0).toString());
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void HumaxItem::sortChildren()
{
	qSort(childItems.begin(), childItems.end(), TreeItemLessThan);
}
*/
////////////////////////////////////////////////////////////////////////////////
