////////////////////////////////////////////////////////////////////////////////
// \class HumaxModel HumaxModel.cpp
//
// \version 1.00
// \date 04-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
//
// \brief Model to control the data (HumaxItem's) in the Device View.
//
// Data Model to control the data (HumaxItem's) displayed in the
// Device View. See QAbstractItemModel for more details.
//
// This file is based on part of the example classes of the Qt Toolkit.
//
// This file may be used under the terms of the GNU General Public
// License version 2.0 as published by the Free Software Foundation
// and appearing in the file LICENSE.GPL included in the packaging of
// this file.  Please review the following information to ensure GNU
// General Public Licensing requirements will be met:
// http://www.trolltech.com/products/qt/opensource.html
//
// If you are unsure which license is appropriate for your use, please
// review the following information:
// http://www.trolltech.com/products/qt/licensing.html or contact the
// sales department at sales@trolltech.com.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#include "HumaxModel.h"

#include <QtGui>

#include "HumaxGlobals.h"
#include "HumaxItem.h"
#include "HumaxController.h"

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Model Class.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxModel::HumaxModel()
: QAbstractItemModel()
, mModelPath()
, mRecordingIcon()
{
	QList<QVariant> rootData;
	rootData << "Name" << "Size" << "Type"/* << "Modified"*/;
	mpRootItem = new HumaxItem(rootData);

	QFileIconProvider iconProvider;
	mRecordingIcon = iconProvider.icon(QFileIconProvider::File);

	QPixmap pixmap = mRecordingIcon.pixmap(QSize(22, 22));

	QPainter painter;
	painter.begin(&pixmap);

	painter.setPen(QColor(0,0,0));
	painter.setBrush(QBrush(QColor(255,0,0)));

	int size = qMax(pixmap.width()/2, pixmap.height()/2);
	painter.drawEllipse(0, pixmap.height() - size - 1, size, size);

	painter.end();

	mRecordingIcon.addPixmap(pixmap);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Model Class.
 * \param controller	Humax Controller class to help decode the
 *						entry list.
 * \param list			The list of entries in the current directory
 *						on the device.
 * \param parent		The parent class of the Model.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxModel::HumaxModel(HumaxController *controller, Entry *list, QObject *parent)
: QAbstractItemModel(parent)
, mModelPath()
, mRecordingIcon()
{
	QList<QVariant> rootData;
	rootData << "Name" << "Size" << "Type"/* << "Modified"*/;
	mpRootItem = new HumaxItem(rootData);
	setupModelData(controller, list, mpRootItem);

	QFileIconProvider iconProvider;
	mRecordingIcon = iconProvider.icon(QFileIconProvider::File);

	QPixmap pixmap = mRecordingIcon.pixmap(QSize(22, 22));

	QPainter painter;
	painter.begin(&pixmap);

	painter.setPen(QColor(0,0,0));
	painter.setBrush(QBrush(QColor(255,0,0)));

	int size = qMax(pixmap.width()/2, pixmap.height()/2);
	painter.drawEllipse(0, pixmap.height() - size - 1, size, size);

	painter.end();

	mRecordingIcon.addPixmap(pixmap);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Destructor.
 *
 * Clean up any memory usage and destroy the class.
 */
////////////////////////////////////////////////////////////////////////////////
HumaxModel::~HumaxModel()
{
	delete mpRootItem;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Update the model with new data.
 *
 * Populates the model with the list of entries supplied, this will
 * then be shown in the device view.
 * \param controller	Humax Controller class to help decode the
 *						entry list.
 * \param list			The list of entries in the current directory
 *						on the device.
 */
////////////////////////////////////////////////////////////////////////////////
void HumaxModel::SetEntryList(HumaxController *controller, Entry *list)
{
	QList<QVariant> rootData;
	rootData << "Name" << "Size" << "Type"/* << "Modified"*/;
	mpRootItem = new HumaxItem(rootData);
	setupModelData(controller, list, mpRootItem);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the data for a given model \a index and \a role.
 *
 * Get the data for a given model \a index and \a role for
 * display in the device view.
 * \param index	The index of the HumaxItem data to be populated.
 * \param role	The role for the type of data displayed.
 * \return The data to be displayed for the \a index and \a role.
 * \sa QAbstractItemModel::data().
 */
////////////////////////////////////////////////////////////////////////////////
QVariant HumaxModel::data(const QModelIndex &index, int role) const
{
	if (!index.isValid())
		return QVariant();

	if (role != Qt::DisplayRole && role != Qt::DecorationRole )
		return QVariant();

	HumaxItem *item = static_cast<HumaxItem*>(index.internalPointer());

	if( role == Qt::DecorationRole)
	{
		if( index.column() == 0 )
		{
			QFileIconProvider icon;
			if( item->isDirectory() )
			{
				return icon.icon(QFileIconProvider::Folder);
			}
			else
			{
				if( item->isRecording() )
				{
					return mRecordingIcon;
				}
				else
				{
					return icon.icon(QFileIconProvider::File);
				}
			}
		}
		else
		{
			return QVariant();
		}
	}
	if( index.column() == 1 )
	{
		qint64 size = item->data(index.column()).toULongLong();
		return QVariant(Humax::SizeAsString(size));
	}
	return item->data(index.column());
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the display state for a given model \a index
 * and \a role.
 *
 * Get the display state for a given model \a index and \a role
 * for display in the device view.
 * \param index	The index of the HumaxItem data to be populated.
 * \return Display flags of how the item is shown.
 * \sa QAbstractItemModel::flags().
 */
////////////////////////////////////////////////////////////////////////////////
Qt::ItemFlags HumaxModel::flags(const QModelIndex &index) const
{
	if( !index.isValid() )
		return Qt::ItemIsEnabled;

	return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Supplies the header data for a row/column section.
 *
 * Supplies the data to be displayed in the view's header row or
 * column.
 * \param section		The section index to be populated.
 * \param orientation	The header orientation being populated.
 * \param role			The role for the type of data displayed.
 * \return The data to be displayed in the header.
 * \sa QAbstractItemModel::headerData().
 */
////////////////////////////////////////////////////////////////////////////////
QVariant HumaxModel::headerData(int section, Qt::Orientation orientation,
								int role) const
{
	if( orientation == Qt::Horizontal && role == Qt::DisplayRole )
		return mpRootItem->data(section);

	if( orientation == Qt::Horizontal && role == Qt::TextAlignmentRole )
		return QVariant(Qt::AlignCenter);

	return QAbstractItemModel::headerData(section, orientation, role);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Gets the model index item for a row and column.
 *
 * Gets the model index item for the row and column position supplied
 * based on the parent index.
 * \param row		The row of the index requested.
 * \param column	The column of the index requested.
 * \param parent	The parent item to base the request.
 * \return The model index item at the position request.
 * \sa QAbstractItemModel::index().
 */
////////////////////////////////////////////////////////////////////////////////
QModelIndex HumaxModel::index(int row, int column, const QModelIndex &parent)
            const
{
	HumaxItem *parentItem;

	if (!parent.isValid())
		parentItem = mpRootItem;
	else
		parentItem = static_cast<HumaxItem*>(parent.internalPointer());

	HumaxItem *childItem = parentItem->child(row);
	if (childItem)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Gets the parent model index for the item supplied.
 *
 * Gets the parent model index for the model item supplied.
 * \param index	The index of the HumaxItem data to be populated.
 * \return The parent item or a null item if no parent available.
 * \sa QAbstractItemModel::parent().
 */
////////////////////////////////////////////////////////////////////////////////
QModelIndex HumaxModel::parent(const QModelIndex &index) const
{
	if (!index.isValid())
		return QModelIndex();

	HumaxItem *childItem = static_cast<HumaxItem*>(index.internalPointer());
	HumaxItem *parentItem = childItem->parent();

	if (parentItem == mpRootItem)
		return QModelIndex();

	return createIndex(parentItem->row(), 0, parentItem);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the number of rows in the model.
 *
 * Gets the number of rows in the model based on the parent item. If
 * the parent item is NULL then the root item is used.
 * \param parent The item to use for the row count.
 * \return The number of rows in the item.
 * \sa QAbstractItemModel::rowCount().
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxModel::rowCount(const QModelIndex &parent) const
{
	HumaxItem *parentItem;

	if (!parent.isValid())
		parentItem = mpRootItem;
	else
		parentItem = static_cast<HumaxItem*>(parent.internalPointer());

	return parentItem->childCount();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the number of columns in the model.
 *
 * Gets the number of columns in the model based on the parent item. If
 * the parent item is NULL then the root item is used.
 * \param parent The item to use for the column count.
 * \return The number of columns in the item.
 * \sa QAbstractItemModel::columnCount().
 */
////////////////////////////////////////////////////////////////////////////////
int HumaxModel::columnCount(const QModelIndex &parent) const
{
	if( parent.isValid() )
		return static_cast<HumaxItem*>(parent.internalPointer())->columnCount();
	else
		return mpRootItem->columnCount();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the filename for a model item.
 *
 * Returns the name of the item stored in the model under the
 * index given.
 * \param index	The index of the HumaxItem data to be retrieved.
 * \return The filename of the item.
 */
////////////////////////////////////////////////////////////////////////////////
QString HumaxModel::fileName(const QModelIndex &index) const
{
    if( !index.isValid() )
        return QString();

    HumaxItem *item = static_cast<HumaxItem*>(index.internalPointer());

	return item->data(0).toString();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the file size for a model item.
 *
 * Returns the size of the item stored in the model under the
 * index given.
 * \param index	The index of the HumaxItem data to be retrieved.
 * \return The size of theitem on the device.
 */
////////////////////////////////////////////////////////////////////////////////
qint64 HumaxModel::fileSize(const QModelIndex &index) const
{
	if( !index.isValid() )
		return 0;

	HumaxItem *item = static_cast<HumaxItem*>(index.internalPointer());

	return item->data(1).toLongLong();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the directory state of an item.
 *
 * Returns true if the model item index represents a directory;
 * otherwise returns false.
 * \param index	The index of the HumaxItem data to be retrieved.
 * \return Boolean, true if the model item is a directory item.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxModel::isDir(const QModelIndex & index) const
{
    if( !index.isValid() )
        return FALSE;

    HumaxItem *item = static_cast<HumaxItem*>(index.internalPointer());

	return item->isDirectory();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Get the recording state of an item.
 *
 * Returns true if the model item index represents a recording item;
 * otherwise returns false.
 * \param index	The index of the HumaxItem data to be retrieved.
 * \return Boolean, true if the model item is in a recording state.
 */
////////////////////////////////////////////////////////////////////////////////
bool HumaxModel::isRecording(const QModelIndex &index) const
{
    if( !index.isValid() )
        return FALSE;

    HumaxItem *item = static_cast<HumaxItem*>(index.internalPointer());

	return item->isRecording();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Converts the Entry list supplied into a list of HumaxItems.
 *
 *  Converts the Entry list supplied when the data changes into a list
 * of HumaxItems to be displayed in the Device View.
 * \param controller	Humax Controller class to help decode the
 *						entry list.
 * \param list			The list of entries in the current directory
 *						on the device.
 * \param parent		The top level item the list is to be added.
 */
////////////////////////////////////////////////////////////////////////////////
void HumaxModel::setupModelData(HumaxController *controller, Entry *list, HumaxItem *parent)
{
	QList<HumaxItem*> parents;
    QList<int> indentations;
    parents << parent;
    indentations << 0;

	// Loop for all files in the directory
	Entry * e = list->firstChild;
	while(e != NULL)
	{
		e->index = -1;
		bool isDir = controller->isDirectory(e);

		// Get name of file or directory and display it
		QString name = controller->getFileName(e->path);

		qint64	size = 0;
		QString	type = "Directory";

		if(!isDir)
		{
			size = e->size;
			int pos = name.lastIndexOf('.');
			if( pos >= 0 )
				type = name.mid(pos + 1) + " File";
			else
				type = "File";
		}

		// Read the column data from the rest of the line.
		QList<QVariant> columnData;
		columnData << name << size << type;

		HumaxItem *pHumaxItem = new HumaxItem(columnData, parents.last());

		if( isDir )
		{
			pHumaxItem->setDirectory(TRUE);
		}

		pHumaxItem->setRecording(e->isRecording);

		// Append a new item to the current parent's list of children.
		parents.last()->appendChild(pHumaxItem);

		// Do next entry
		e = e->sibling;
	}
}

////////////////////////////////////////////////////////////////////////////////
