////////////////////////////////////////////////////////////////////////////////
// \class AboutDialog AboutDialog.cpp
//
// \version 1.00
// \date 27-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief Media Controller Main Application Window Class.
//
// **
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
27-07-06 1.00 Initial release.

*****/
#include "MainWindow.h"

#include <QCloseEvent>
#include <QHeaderView>
#include <QInputDialog>
#include <QMessageBox>
#include <QSettings>
#include <QWhatsThis>

#include "AboutDialog.h"
#include "DeviceInfoDialog.h"
#include "HostModel.h"
#include "HumaxModel.h"
#include "PrefsDialog.h"
#include "QHostProxyModel.h"
#include "TransferDialog.h"

extern struct usb_device *mpHumaxDevice; // Handle to the USB device

#if QT_VERSION < 0x040100
#warning Recommend using Qt 4.1.0 or newer!
#endif

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Main Window Class.
 */
////////////////////////////////////////////////////////////////////////////////
MainWindow::MainWindow(QWidget *parent)
: QMainWindow(parent)
, mHumaxController()
, mLogOutput(true)
, mLogErrors(false)
, mLogDebug(false)
{
	ui.setupUi(this);

	mpHostPathModel	= new HostModel;
	mpHostPathModel->setSorting(QDir::DirsFirst|QDir::Name);

	mpHostProxyModel = new QHostProxyModel(this);

	mpHumaxModel	= new HumaxModel;

	Connections();

	mpHostProxyModel->setSourceModel(mpHostPathModel);
	ui.mpHostDriveView->setModel(mpHostProxyModel);
	ui.mpDeviceDriveView->setModel(mpHumaxModel);

	readSettings();

	QString hostPath(QDir::homePath());
	if( !ui.mpHostDrivePathEdit->text().isEmpty() )
	{
		QDir hostDir(ui.mpHostDrivePathEdit->text());
		if( hostDir.exists() )
		{
			hostPath = ui.mpHostDrivePathEdit->text();
		}
	}

	int comboCount = 0;
	for( int i = 0; i < mpHostPathModel->rowCount(QModelIndex()); ++i )
	{
		QString drive = mpHostPathModel->
			filePath(mpHostPathModel->index(i,0));

		if( drive == "A:/" || drive == "B:/" )
		{
			continue;
		}

		ui.mpHostDriveCombo->addItem(drive.left(2));

		if( drive == hostPath.left(3) )
		{
			ui.mpHostDriveCombo->setCurrentIndex(comboCount);
		}
		++comboCount;
	}

	ui.mpHostDriveView->setRootIndex(mpHostProxyModel->mapFromSource(mpHostPathModel->index(hostPath)));
	ui.mpHostDrivePathEdit->setText(hostPath);

#if QT_VERSION < 0x040100
	// Qt 4.0.1...
	ui.mpDeviceDriveCombo->setCurrentIndex(1);
#endif

	mHumaxController.Init();
	UpdateFilter();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called when the user closes the application.
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::closeEvent(QCloseEvent *event)
{
	Debug("MainWindow::closeEvent");

	writeSettings();
	event->accept();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Set up Qt connections at the start.
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::Connections()
{
	// Humax Controller...
	connect(&mHumaxController, SIGNAL(transferError(QString)),
			this, SLOT(transferError(QString)));

#if QT_VERSION < 0x040100
	// Qt 4.0.1...
	// Host...
	connect(ui.mpHostNewFolderButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionHostNewFolder_triggered()));
	connect(ui.mpHostRenameButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionHostRename_triggered()));
	connect(ui.mpHostDeleteButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionHostDelete_triggered()));
	connect(ui.mpHostRefreshButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionHostRefresh_triggered()));
	connect(ui.mpHostDriveCombo, SIGNAL(activated(int)),
		this, SLOT(on_mpHostDriveCombo_currentIndexChanged(int)));
	// Device...
	connect(ui.mpDeviceNewFolderButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionDeviceNewFolder_triggered()));
	connect(ui.mpDeviceRenameButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionDeviceRename_triggered()));
	connect(ui.mpDeviceDeleteButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionDeviceDelete_triggered()));
	connect(ui.mpDeviceRefreshButton, SIGNAL(clicked()),
		this, SLOT(on_mpActionDeviceRefresh_triggered()));
	connect(ui.mpDeviceDriveCombo, SIGNAL(activated(int)),
		this, SLOT(on_mpDeviceDriveCombo_currentIndexChanged(int)));
#endif
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Output information to the log window.
 *
 * 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::Log(const QString &text)
{
	if( !text.isEmpty() )
	{
		QString out = text;
		ui.mpOutputText->setHtml(ui.mpOutputText->toHtml()
			+ out.replace("\n","<BR>") + "<BR>");
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Output information to the log window.
 *
 * 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::Output(const QString &text)
{
	if( mLogOutput && !text.isEmpty() )
	{
		QString out = text;
		ui.mpOutputText->setHtml(ui.mpOutputText->toHtml()
			+ "<FONT color=\"#0000ff\">" + out.replace("\n","<BR>") + "</FONT><BR>");
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Output Debug information to the log window.
 *
 * 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::Debug(const QString &text)
{
	if( mLogDebug && !text.isEmpty() )
	{
		ui.mpOutputText->setHtml(ui.mpOutputText->toHtml()
			+ "<FONT color=\"#A0A0A0\">" + text + "</FONT><BR>");
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Output Warning information to the log window.
 *
 * 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::Warning(const QString &text)
{
	if( !text.isEmpty() )
	{
		if( mLogErrors )
		{
			QString out = text;
			ui.mpOutputText->setHtml(ui.mpOutputText->toHtml()
				+ "<FONT color=\"#ff0000\">" + out.replace("\n","<BR>") + "</FONT><BR>");
		}

		QMessageBox::warning(this, tr("Warning"), text,
			QMessageBox::Ok, QMessageBox::NoButton);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Output Error information to the log window.
 *
 * 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::Error(const QString &text)
{
	if( !text.isEmpty() )
	{
		if( mLogErrors )
		{
			QString out = text;
			ui.mpOutputText->setHtml(ui.mpOutputText->toHtml()
				+ "<FONT color=\"#ff0000\">" + out.replace("\n","<BR>") + "</FONT><BR>");
		}

		QMessageBox::critical(this, tr("Error"), text,
			QMessageBox::Ok, QMessageBox::NoButton);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the Main Window display availability.
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::UpdateGui()
{
	Debug("MainWindow::UpdateGui()");

	bool connected = mHumaxController.IsConnected();

	// Menu Items...
	ui.mpActionConnect->setEnabled(!connected);
	ui.mpActionDisconnect->setEnabled(connected);
	ui.mpActionInformation->setEnabled(connected);

	ui.mpDeviceDrivePathEdit->setEnabled(connected);
	ui.mpDeviceDriveCombo->setEnabled(connected);
	ui.mpDeviceDriveView->setEnabled(connected);

	ui.mpActionDeviceRefresh->setEnabled(connected);
	ui.mpDeviceRefreshButton->setEnabled(connected);
	ui.mpActionDeviceNewFolder->setEnabled(connected);
	ui.mpDeviceNewFolderButton->setEnabled(connected);

	if( ui.mpDeviceDriveCombo->currentIndex() == 0 )
	{
		ui.mpActionFormat->setEnabled(connected);
	}
	else
	{
		ui.mpActionFormat->setEnabled(FALSE);
	}

	// Host View...
	if( ui.mpHostDrivePathEdit->text().length() <= 3 )
	{
		ui.mpHostParentButton->setEnabled(FALSE);
	}
	else
	{
		ui.mpHostParentButton->setEnabled(TRUE);
	}

	bool hostRename = FALSE, hostDelete = FALSE;
	bool hostTransfer = FALSE;

	QItemSelectionModel *pHostSelModel =
			ui.mpHostDriveView->selectionModel();
	if( pHostSelModel != 0 )
	{
		int selRows = pHostSelModel->selectedIndexes().count() /
			mpHostPathModel->columnCount(mpHostPathModel->
				index(QDir::homePath()));
		if( selRows > 1 )
		{
			hostRename = FALSE; hostDelete = TRUE;
			hostTransfer = connected;
		}
		else if( selRows == 1 )
		{
			if( ui.mpHostDriveView->rootIndex() !=
				pHostSelModel->selectedIndexes()[0] )
			{
				hostRename = TRUE; hostDelete = TRUE;
				hostTransfer = connected;
			}
		}
		else
		{
			hostRename = FALSE; hostDelete = FALSE;
		}
	}

	if( hostTransfer &&
		ui.mpDeviceDrivePathEdit->text().left(9) != QString("D1:/Image") &&
		ui.mpDeviceDrivePathEdit->text().left(7) != QString("D1:/MP3"))
	{
		hostTransfer = FALSE;
	}

	ui.mpDownloadButton->setEnabled(hostTransfer);
	ui.mpActionDownload->setEnabled(hostTransfer);
	ui.mpActionHostRename->setEnabled(hostRename);
	ui.mpHostRenameButton->setEnabled(hostRename);
	ui.mpActionHostDelete->setEnabled(hostDelete);
	ui.mpHostDeleteButton->setEnabled(hostDelete);

	// Device View...
	if( !connected || ui.mpDeviceDrivePathEdit->text().length() <= 4 )
	{
		ui.mpDeviceParentButton->setEnabled(FALSE);
	}
	else
	{
		ui.mpDeviceParentButton->setEnabled(TRUE);
	}

	bool deviceRename = FALSE, deviceDelete = FALSE;
	bool deviceTransfer = FALSE;

	QItemSelectionModel *pDeviceSelModel =
			ui.mpDeviceDriveView->selectionModel();
	if( pDeviceSelModel != 0 )
	{
//		int selRows = pDeviceSelModel->selectedIndexes().count() /
//			mpHostPathModel->columnCount(mpHostPathModel->
//				index(QDir::homePath()));
		int selRows = pDeviceSelModel->selectedIndexes().count() /
			3;
		if( selRows > 1 )
		{
			deviceRename = FALSE; deviceDelete = TRUE;
			deviceTransfer = connected;
		}
		else if( selRows == 1 )
		{
			deviceRename = TRUE; deviceDelete = TRUE;
			deviceTransfer = connected;
		}
		else
		{
			deviceRename = FALSE; deviceDelete = FALSE;
		}
	}

	ui.mpActionUpload->setEnabled(deviceTransfer);
	ui.mpUploadButton->setEnabled(deviceTransfer);
	ui.mpActionDeviceRename->setEnabled(deviceRename);
	ui.mpDeviceRenameButton->setEnabled(deviceRename);
	ui.mpActionDeviceDelete->setEnabled(deviceDelete);
	ui.mpDeviceDeleteButton->setEnabled(deviceDelete);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Updates the contents of the Device view.
 */
// bool doList(usb_dev_handle * dh);
////////////////////////////////////////////////////////////////////////////////
bool MainWindow::UpdateDeviceList()
{
	Debug("MainWindow::UpdateDeviceList()");

	if( mHumaxController.IsConnected() )
	{
		// Get current directory contents
		Entry *dir = mHumaxController.getCurDeviceDir();
		int noEntries = mHumaxController.setDeviceDirectory(dir);
		if(noEntries == -1)
		{
			Error(QString("Retrieving directory contents for 'D%1' failed!").arg(dir->path));
			return false;
		}

		// Display list of files and warning msg if more than 896 entries
		mpHumaxModel->SetEntryList(&mHumaxController, dir);
		mpHumaxModel->setModelPath(dir->path);
	
		int i = 0;
		QList<int> widths;
		for( ; i < ui.mpDeviceDriveView->header()->count(); ++i )
			widths << ui.mpDeviceDriveView->columnWidth(i);

		ui.mpDeviceDriveView->setModel(mpHumaxModel);

		for( i = 0; i < ui.mpDeviceDriveView->header()->count(); ++i )
			ui.mpDeviceDriveView->header()->resizeSection(i, widths[i]);

		QString path(dir->path);
		if( path.length() > 3 && path.right(1) == "/" )
			path.resize(path.length() - 1);

		ui.mpDeviceDrivePathEdit->setText(QString("D") + path);

		if(noEntries > HUMAX_MAX_DIR_ENTRIES)
			Warning(QString("Only the first 896 entries of %1 were displayed!").arg(noEntries));
	}

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Update the filter settings associated with the host view.
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::UpdateFilter()
{
	Debug("MainWindow::UpdateFilter()");

	if( ui.mpActionFilter->isChecked() )
	{
		if( mHumaxController.IsConnected() )
		{
			if( ui.mpDeviceDriveCombo->currentIndex() == 1)
			{
				mpHostProxyModel->setFilterList(QStringList() << ".ts");
			}
			else
			{
				if( ui.mpDeviceDrivePathEdit->text().left(9) == QString("D1:/Image") )
				{
					mpHostProxyModel->setFilterList(QStringList() << ".jpg");
				}
				else if( ui.mpDeviceDrivePathEdit->text().left(7) == QString("D1:/MP3") )
				{
					mpHostProxyModel->setFilterList(QStringList() << ".mp3");
				}
				else
				{
					mpHostProxyModel->setFilterList(QStringList() << ".jpg" << ".mp3");
				}
			}
		}
		else
		{
			mpHostProxyModel->
				setFilterList(QStringList() << ".jpg" << ".mp3" << ".ts");
		}
	}
	else
	{
		mpHostProxyModel->setFilterList(QStringList());
	}

	mpHostPathModel->refresh(mpHostPathModel->
		index(ui.mpHostDrivePathEdit->text()));
	UpdateGui();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//File Menu...
////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionConnect_triggered()
{
	Debug("MainWindow::on_mpActionConnect_triggered()");

	if( mHumaxController.Connect() )
	{
		Log("Connected.");
		if( mLogOutput )
		{
			Output(mHumaxController.humaxManufacturer());
			Output(mHumaxController.humaxProduct() + "(" +
				mHumaxController.humaxHardwareVersion() + ")");
			Output(tr("Serial No :") + mHumaxController.humaxSerialNumber());
		}
		UpdateDeviceList();
	}
	else
	{
		Error("Failed to connect to the device.");
	}
	UpdateFilter();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionDisconnect_triggered()
{
	Debug("MainWindow::on_mpActionDisconnect_triggered()");

	if( !mHumaxController.Disconnect() )
	{
		Error("Failed to disconnect from the device.");
	}
	UpdateFilter();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionFormat_triggered()
{
	Debug("MainWindow::on_mpActionFormat_triggered()");

	// Prompt user and get confirmation
	int confirm = QMessageBox::warning(this, "Format",
			"YOU ARE ABOUT TO FORMAT THE 'IMAGE/MP3' DISK\n"
			"ALL CONTENTS ON THIS DISK WILL BE ERASED\n\n"
			"DO YOU WISH TO PROCEED?",
		QMessageBox::Yes|QMessageBox::Default,
		QMessageBox::No|QMessageBox::Escape);

	if( confirm == QMessageBox::Yes )
	{
		setCursor(Qt::BusyCursor);

		if( mHumaxController.Format() )
		{
			// Display confirmation
			Warning("THE 'IMAGE/MP3' DISK PARTITION HAS BEEN ERASED!");
///			Sleep(1000);
		}

		UpdateDeviceList();
		UpdateGui();
		setCursor(Qt::ArrowCursor);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionPreferences_triggered()
{
	Debug("MainWindow::on_mpActionPreferences_triggered()");

	PrefsDialog dlg(this);
	dlg.exec();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//Tools->Host Menu...
////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionHostNewFolder_triggered()
{
	Debug("MainWindow::on_mpActionHostNewFolder_triggered()");

	bool ok = FALSE;

	QString folderName = QInputDialog::getText(this, tr("New Folder"),
		tr("Set Folder Name:"), QLineEdit::Normal, tr("New Folder"), &ok);

	if( ok && !folderName.isEmpty() )
	{
		setCursor(Qt::BusyCursor);
		QDir curDir(ui.mpHostDrivePathEdit->text());
		if( curDir.exists(folderName) )
		{
			Error("A file with the name you specified already exists.");
			setCursor(Qt::ArrowCursor);
			return;
		}
		if( !curDir.mkdir(folderName) )
		{
			Error("Could not create new folder.");
			setCursor(Qt::ArrowCursor);
			return;
		}
		mpHostPathModel->refresh(mpHostPathModel->
			index(ui.mpHostDrivePathEdit->text()));
		UpdateGui();
		setCursor(Qt::ArrowCursor);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionHostRename_triggered()
{
	Debug("MainWindow::on_mpActionHostRename_triggered()");

	QItemSelectionModel *pHostSelModel =
			ui.mpHostDriveView->selectionModel();
	if( pHostSelModel != 0 )
	{
		QModelIndex index;
		foreach(index, pHostSelModel->selectedIndexes())
		{
			if( index.column() == 0 )
			{
				QModelIndex sourceIndex = mpHostProxyModel->mapToSource(index);

				QString filePath = mpHostPathModel->filePath(sourceIndex);
				QString fileName = mpHostPathModel->fileName(sourceIndex);

				bool ok = FALSE;

				if( mpHostPathModel->isDir(sourceIndex) )
				{
					QString newFileName = QInputDialog::getText(this, tr("Rename"),
						tr("New Folder Name:"), QLineEdit::Normal, fileName, &ok);

					if( ok && !newFileName.isEmpty() && newFileName != fileName )
					{
						setCursor(Qt::BusyCursor);

						QDir renameDir(filePath);
						if( !renameDir.cdUp() )
						{
							Error(QString("Failed to rename the folder \"%1\"").arg(fileName));
							continue;
						}

						if( !renameDir.rename(fileName, newFileName) )
						{
							Error(QString("Failed to rename the folder \"%1\"").arg(fileName));
						}
					}
				}
				else
				{
					QString newFileName = QInputDialog::getText(this, tr("Rename"),
						tr("New File Name:"), QLineEdit::Normal, fileName, &ok);

					if( ok && !newFileName.isEmpty() && newFileName != fileName )
					{
						setCursor(Qt::BusyCursor);

						int pos = filePath.lastIndexOf('/');

						QFile renameFile(filePath);
						if( !renameFile.rename(filePath.left(pos + 1) + newFileName) )
						{
							Error(QString("Failed to rename the file \"%1\"").arg(fileName));
						}
					}
				}
			}
		}
		mpHostPathModel->refresh(mpHostPathModel->
			index(ui.mpHostDrivePathEdit->text()));
		UpdateGui();
		setCursor(Qt::ArrowCursor);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionHostDelete_triggered()
{
	Debug("MainWindow::on_mpActionHostDelete_triggered()");

	QItemSelectionModel *pHostSelModel =
			ui.mpHostDriveView->selectionModel();
	if( pHostSelModel != 0 )
	{
		QStringList filePaths;
		QStringList fileNames;
		bool isDirectory = false;

		QModelIndex index;
		foreach(index, pHostSelModel->selectedIndexes())
		{
			if( index.column() == 0 )
			{
				QModelIndex sourceIndex = mpHostProxyModel->mapToSource(index);

				filePaths.append(mpHostPathModel->filePath(sourceIndex));
				fileNames.append(mpHostPathModel->fileName(sourceIndex));

				if( mpHostPathModel->isDir(sourceIndex) )
					isDirectory = true;
			}
		}

		QString title;
		QString message;

		if( filePaths.count() == 1 )
		{
			if( isDirectory )
			{
				title = tr("Confirm Folder Delete");
				message = QString("Are you sure you want to delete"
					" the folder '%1' and all its contents?").arg(fileNames[0]);
			}
			else
			{
				title = tr("Confirm File Delete");
				message = QString("Are you sure you want to delete"
					" \"%1\"?").arg(fileNames[0]);
			}
		}
		else
		{
			title = tr("Confirm Multiple File Delete");
			message = QString("Are you sure you want to delete"
				" these %1 items?").arg(filePaths.count());
		}

		if( filePaths.count() > 0 )
		{
			int confirm = QMessageBox::question(this, title, message,
				QMessageBox::Yes|QMessageBox::Default,
				QMessageBox::No|QMessageBox::Escape);

			if( confirm == QMessageBox::Yes )
			{
				setCursor(Qt::BusyCursor);
				for( int index = 0; index < filePaths.count(); ++index )
				{
					if( !HostDeleteFile(filePaths[index]) )
					{
						Error(QString("Failed to delete \"%1\"").arg(fileNames[0]));
						break;
					}
				}
			}
		}

		mpHostPathModel->refresh(mpHostPathModel->
			index(ui.mpHostDrivePathEdit->text()));
		UpdateGui();
		setCursor(Qt::ArrowCursor);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionHostRefresh_triggered()
{
	Debug("MainWindow::on_mpActionHostRefresh_triggered()");

	setCursor(Qt::BusyCursor);
	mpHostPathModel->refresh(mpHostPathModel->
		index(ui.mpHostDrivePathEdit->text()));
	UpdateGui();
	setCursor(Qt::ArrowCursor);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//Tools->Device Menu...
////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionDeviceNewFolder_triggered()
{
	Debug("MainWindow::on_mpActionDeviceNewFolder_triggered()");

	// Get current directory contents
	Entry * dir = mHumaxController.getCurDeviceDir();
	if( mHumaxController.setDeviceDirectory(dir) == -1 )
	{
		Error(QString("Could not retrieve directory contents for '%1'!").arg(dir->path));
		return;
	}

	bool ok = FALSE;

	QString folderName = QInputDialog::getText(this, tr("New Folder"),
		tr("Set Folder Name:"), QLineEdit::Normal, tr("New Folder"), &ok);

	if( ok && !folderName.isEmpty() )
	{
		setCursor(Qt::BusyCursor);

		QString newDir(dir->path);
		if( newDir.right(1) != "/" ) newDir += "/";
		newDir += folderName;

		char newDirStr[MAX_PATH];
		sprintf(newDirStr, "%s", newDir.toStdString().data());

		if( !mHumaxController.humaxCreateDir(newDirStr) )
		{
			Error(QString("ERROR: creating new directory '%1'!").arg(newDir));
			return;
		}

		UpdateDeviceList();
		UpdateGui();
		setCursor(Qt::ArrowCursor);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionDeviceRename_triggered()
{
	Debug("MainWindow::on_mpActionDeviceRename_triggered()");

	// Get current directory contents
	Entry * dir = mHumaxController.getCurDeviceDir();
	if( mHumaxController.setDeviceDirectory(dir) == -1 )
	{
		Error(QString("Could not retrieve directory contents for '%1'!").arg(dir->path));
		return;
	}

	QItemSelectionModel *pDeviceSelModel =
			ui.mpDeviceDriveView->selectionModel();
	if( pDeviceSelModel != 0 )
	{
		QModelIndex index;
		foreach(index, pDeviceSelModel->selectedIndexes())
		{
			if( index.column() == 0 )
			{
				QString fileName = mpHumaxModel->fileName(index);
				QString filePath(dir->path);
				if( filePath.right(1) != "/" ) filePath += "/";

				char oldFilePath[MAX_PATH];
				sprintf(oldFilePath, "%s%s", filePath.toStdString().data(),
					fileName.toStdString().data());

				bool ok = FALSE;

				if( mpHumaxModel->isDir(index) )
				{
					QString newFileName = QInputDialog::getText(this, tr("Rename"),
						tr("New Folder Name:"), QLineEdit::Normal, fileName, &ok);

					if( ok && !newFileName.isEmpty() && newFileName != fileName )
					{
						setCursor(Qt::BusyCursor);
						char newFilePath[MAX_PATH];
						sprintf(newFilePath, "%s%s", filePath.toStdString().data(),
							newFileName.toStdString().data());

						if( !mHumaxController.humaxRenameFile(oldFilePath, newFilePath) )
						{
							Error(QString("Failed to rename the folder \"%1\"").arg(fileName));
						}
					}
				}
				else
				{
					QString newFileName = QInputDialog::getText(this, tr("Rename"),
						tr("New File Name:"), QLineEdit::Normal, fileName, &ok);

					if( ok && !newFileName.isEmpty() && newFileName != fileName )
					{
						setCursor(Qt::BusyCursor);
						char newFilePath[MAX_PATH];
						sprintf(newFilePath, "%s%s", filePath.toStdString().data(),
							newFileName.toStdString().data());

						if( !mHumaxController.humaxRenameFile(oldFilePath, newFilePath) )
						{
							Error(QString("Failed to rename the file \"%1\"").arg(fileName));
						}
					}
				}
			}
		}
		UpdateDeviceList();
		UpdateGui();
		setCursor(Qt::ArrowCursor);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionDeviceDelete_triggered()
{
	Debug("MainWindow::on_mpActionDeviceDelete_triggered()");

	// Get current directory contents
	Entry * dir = mHumaxController.getCurDeviceDir();
	if( mHumaxController.setDeviceDirectory(dir) == -1 )
	{
		Error(QString("Could not retrieve directory contents for '%1'!").arg(dir->path));
		return;
	}

	QItemSelectionModel *pDeviceSelModel =
			ui.mpDeviceDriveView->selectionModel();
	if( pDeviceSelModel != 0 )
	{
		QStringList filePaths;
		QStringList fileNames;
		bool isDirectory = false;

		QModelIndex index;
		foreach(index, pDeviceSelModel->selectedIndexes())
		{
			if( index.column() == 0 )
			{
				QString fileName = mpHumaxModel->fileName(index);
				QString filePath(dir->path);
				if( filePath.right(1) != "/" ) filePath += "/";
				filePath += fileName;

				filePaths.append(filePath);
				fileNames.append(fileName);

				if( mpHumaxModel->isDir(index) )
					isDirectory = true;
			}
		}

		QString title;
		QString message;

		if( filePaths.count() == 1 )
		{
			if( isDirectory )
			{
				title = tr("Confirm Folder Delete");
				message = QString("Are you sure you want to delete"
					" the folder '%1' and all its contents?").arg(fileNames[0]);
			}
			else
			{
				title = tr("Confirm File Delete");
				message = QString("Are you sure you want to delete"
					" \"%1\"?").arg(fileNames[0]);
			}
		}
		else
		{
			title = tr("Confirm Multiple File Delete");
			message = QString("Are you sure you want to delete"
				" these %1 items?").arg(filePaths.count());
		}

		if( filePaths.count() > 0 )
		{
			int confirm = QMessageBox::question(this, title, message,
				QMessageBox::Yes|QMessageBox::Default,
				QMessageBox::No|QMessageBox::Escape);

			if( confirm == QMessageBox::Yes )
			{
				setCursor(Qt::BusyCursor);
				for( int index = 0; index < filePaths.count(); ++index )
				{
					if( !DeviceDeleteFile(filePaths[index]) )
					{
						Error(QString("Failed to delete \"%1\"").arg(fileNames[0]));
						break;
					}
				}
			}
		}

		UpdateDeviceList();
		UpdateGui();
		setCursor(Qt::ArrowCursor);
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief 
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionDeviceRefresh_triggered()
{
	Debug("MainWindow::on_mpActionDeviceRefresh_triggered()");

	UpdateDeviceList();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//Tools Menu...
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionUpload_triggered()
{
	Debug("MainWindow::on_mpActionUpload_triggered()");

	// Get current directory contents
	Entry *dir = mHumaxController.getCurDeviceDir();
	if( mHumaxController.setDeviceDirectory(dir) == -1 )
	{
		Error(QString("Could not retrieve directory contents for '%1'!").arg(dir->path));
		return;
	}

	QList<Entry *> uploadList;

	QItemSelectionModel *pDeviceSelModel =
			ui.mpDeviceDriveView->selectionModel();
	if( pDeviceSelModel != 0 )
	{
		QModelIndex index;
		foreach(index, pDeviceSelModel->selectedIndexes())
		{
			if( index.column() == 0 )
			{
				QString fileName = mpHumaxModel->fileName(index);
				qint64 fileSize = mpHumaxModel->fileSize(index);

				QString filePath(dir->path);
				if( filePath.right(1) != "/" ) filePath += "/";
				filePath += fileName;

				if( !mpHumaxModel->isDir(index) )
				{
					Entry *pNewEntry = mHumaxController.createEntry(
											filePath.toStdString().data(),
											fileName.length(), fileSize);
					uploadList.append(pNewEntry);
				}
			}
		}
	}

	if( uploadList.count() > 0 )
	{
		TransferDialog dlg(this);
		dlg.setTransferMode(TransferDialog::UPLOAD);
		dlg.SetFileCount(uploadList.count());

		connect(&mHumaxController, SIGNAL(finished()), &dlg, SLOT(accept()));
		connect(&mHumaxController, SIGNAL(transferStarted(int, QString)),
				&dlg, SLOT(transferStarted(int, QString)));
		connect(&mHumaxController, SIGNAL(uploadProgress(int, QString, double, double, double, double, double, int, int)),
				&dlg, SLOT(uploadProgress(int, QString, double, double, double, double, double, int, int)));
		connect(&mHumaxController, SIGNAL(transferComplete(int)),
				&dlg, SLOT(transferComplete(int)));
		connect(&mHumaxController, SIGNAL(transferOverwrite(QString, QString, QString, bool)),
				&dlg, SLOT(transferOverwrite(QString, QString, QString, bool)));

		connect(&dlg, SIGNAL(transferCancelled()),
				&mHumaxController, SLOT(transferCancelled()));
		connect(&dlg, SIGNAL(transferOverwriteReturn(int)),
				&mHumaxController, SLOT(transferOverwriteReturn(int)));

		mHumaxController.doTransfersDeviceToHost(uploadList,
			ui.mpHostDrivePathEdit->text());

		dlg.exec();

		disconnect(&mHumaxController,0,&dlg,0);
	}
	mpHostPathModel->refresh(mpHostPathModel->
		index(ui.mpHostDrivePathEdit->text()));
	UpdateGui();
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionDownload_triggered()
{
	Debug("MainWindow::on_mpActionDownload_triggered()");

	// Get current directory contents
	Entry *dir = mHumaxController.getCurDeviceDir();
	if( mHumaxController.setDeviceDirectory(dir) == -1 )
	{
		Error(QString("Could not retrieve directory contents for '%1'!").arg(dir->path));
		return;
	}

	QString baseDir = ui.mpHostDrivePathEdit->text();
	if( baseDir.right(1) != "/" )
		baseDir += "/";

	QStringList downloadList;
	QStringList mkDirList;

	QItemSelectionModel *pHostSelModel =
			ui.mpHostDriveView->selectionModel();
	if( pHostSelModel != 0 )
	{
		QModelIndex index;
		foreach(index, pHostSelModel->selectedIndexes())
		{
			if( index.column() == 0 )
			{
				QModelIndex sourceIndex = mpHostProxyModel->mapToSource(index);

				QString fileName = mpHostPathModel->fileName(sourceIndex);

				if( mpHostPathModel->isDir(sourceIndex) )
				{
					HostCopyFileList(baseDir, fileName,
						dir->path, downloadList, mkDirList);
				}
				else
				{
					downloadList.append(fileName);
				}
			}
		}
	}

	if( downloadList.count() > 0 )
	{
		if( mkDirList.count() > 0 )
		{
			QString index;
			foreach(index, mkDirList)
			{
				char newDirStr[MAX_PATH];
				sprintf(newDirStr, "%s", index.toStdString().data());

				if( !mHumaxController.humaxCreateDir(newDirStr) )
				{
					Error(QString("ERROR: creating new directory '%1'!").arg(index));
					return;
				}
			}
		}

		TransferDialog dlg(this);
		dlg.setTransferMode(TransferDialog::DOWNLOAD);
		dlg.SetFileCount(downloadList.count());

		connect(&mHumaxController, SIGNAL(finished()), &dlg, SLOT(accept()));
		connect(&mHumaxController, SIGNAL(transferStarted(int, QString)),
				&dlg, SLOT(transferStarted(int, QString)));
		connect(&mHumaxController, SIGNAL(downloadProgress(int, double, double, double, int, int)),
				&dlg, SLOT(downloadProgress(int, double, double, double, int, int)));
		connect(&mHumaxController, SIGNAL(transferComplete(int)),
				&dlg, SLOT(transferComplete(int)));
		connect(&mHumaxController, SIGNAL(transferOverwrite(QString, QString, QString, bool)),
				&dlg, SLOT(transferOverwrite(QString, QString, QString, bool)));

		connect(&dlg, SIGNAL(transferCancelled()),
				&mHumaxController, SLOT(transferCancelled()));
		connect(&dlg, SIGNAL(transferOverwriteReturn(int)),
				&mHumaxController, SLOT(transferOverwriteReturn(int)));

		mHumaxController.doTransfersHostToDevice(baseDir, downloadList,
			mpHumaxModel->getModelPath());

		dlg.exec();

		disconnect(&mHumaxController,0,&dlg,0);
	}

	UpdateDeviceList();
	UpdateGui();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//About Menu...
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionAbout_triggered()
{
	Debug("MainWindow::on_mpActionAbout_triggered()");

	AboutDialog dlg(this);
	dlg.exec();
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionWhatsThis_triggered()
{
	Debug("MainWindow::on_mpActionWhatsThis_triggered()");

	QWhatsThis::enterWhatsThisMode();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//Toolbar...
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionFilter_triggered(bool checked)
{
	Debug("MainWindow::on_mpActionFilter_triggered()");

	setCursor(Qt::BusyCursor);
	UpdateFilter();
	setCursor(Qt::ArrowCursor);
}
////////////////////////////////////////////////////////////////////////////////
// on_mpActionInformation_triggered()
// bool doInfo(usb_dev_handle * dh, bool fullInfo)
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpActionInformation_triggered()
{
	Debug("MainWindow::on_mpActionInformation_triggered()");

	if( mHumaxController.IsConnected() )
	{
		DeviceInfoDialog dlg(this);
		dlg.setManufacturer(mHumaxController.humaxManufacturer());
		dlg.setProductName(mHumaxController.humaxProduct());
		dlg.setSerialNo(mHumaxController.humaxSerialNumber());
		dlg.setHardwareVerson(mHumaxController.humaxHardwareVersion());

		QString humaxStr;
		humaxStr.sprintf("V%X.%02X (Full Speed, 12 Mbit/Sec)", (mpHumaxDevice->descriptor.bcdUSB & 0xff00) >> 8, mpHumaxDevice->descriptor.bcdUSB & 0xff);
		dlg.setUsbVersion(humaxStr);

		dlg.setUsbEndpoints("Bulk Xfer In/Out (512 Byte Packet Size)");

		// Get disk 1 info...
		Entry disk1;
		mHumaxController.initEntry(&disk1, "1:/", 0, 0);
		qint64 stats1[6];
		qint64 diskSpace1[2];
		if(mHumaxController.buildDeviceEntryTree(&disk1, 0, stats1) == -1)
		{
			Error("Could not build directory tree for partition 1!");
			mHumaxController.deleteEntryTree(&disk1);
			return;
		}
		if(!mHumaxController.humaxGetDiskSpace(diskSpace1))
		{
			Error("Could not get disk space info for partition 1!");
			mHumaxController.deleteEntryTree(&disk1);
			return;
		}

		dlg.setDiskOneSpace(diskSpace1[0], diskSpace1[1]);
		dlg.setDiskOneDirectories(stats1[DIRS_SIZE], stats1[DIRS_COUNT]);
		dlg.setDiskOneFiles(stats1[FILES_SIZE], stats1[FILES_COUNT]);
		dlg.setDiskOneEntries(stats1[ENTRIES_SIZE], stats1[ENTRIES_COUNT]);

		// Get disk 2 info...
		Entry disk2;
		mHumaxController.initEntry(&disk2, "2:/", 0, 0);
		qint64 stats2[6];
		qint64 diskSpace2[2];
		if(mHumaxController.buildDeviceEntryTree(&disk2, 0, stats2) == -1)
		{
			Error("Could not build directory tree for partition 2!");
			mHumaxController.deleteEntryTree(&disk1);
			mHumaxController.deleteEntryTree(&disk2);
			return;
		}
		if(!mHumaxController.humaxGetDiskSpace(diskSpace2))
		{
			Error("Could not get disk space info for partition 2!");
			mHumaxController.deleteEntryTree(&disk1);
			mHumaxController.deleteEntryTree(&disk2);
			return;
		}

		dlg.setDiskTwoSpace(diskSpace2[0], diskSpace2[1]);
		dlg.setDiskTwoDirectories(stats2[DIRS_SIZE], stats2[DIRS_COUNT]);
		dlg.setDiskTwoFiles(stats2[FILES_SIZE], stats2[FILES_COUNT]);
		dlg.setDiskTwoEntries(stats2[ENTRIES_SIZE], stats2[ENTRIES_COUNT]);

		// Display list of video files that are currently being recorded on the Humax
		Entry * e = disk2.firstChild;
		int recCount = 0;
		char name[MAX_PATH];
		QString recStatus;
		while(e != NULL)
		{
			if(e->isRecording)
			{
				++recCount;
				sprintf(name,"%s",mHumaxController.getFileName(e->path).toStdString().data());
				recStatus += name;
				if( recCount > 1 ) recStatus += ", ";
			}
			e = e->sibling;
		}
		if(recCount == 0)
			recStatus = "Idle";
		
		dlg.setRecordingStatus(recStatus);

		// Delete entry trees
		mHumaxController.deleteEntryTree(&disk1);
		mHumaxController.deleteEntryTree(&disk2);

		dlg.exec();
	}
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//Other Widget Slots:
//Host Area...
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpHostDriveCombo_currentIndexChanged(int index)
{
	Debug("MainWindow::on_mpHostDriveCombo_currentIndexChanged()");
	setCursor(Qt::BusyCursor);

	if( ui.mpHostDriveCombo->currentText() != ui.mpHostDrivePathEdit->text().left(2) )
	{
		ui.mpHostDriveView->setRootIndex(mpHostProxyModel->
			mapFromSource(mpHostPathModel->index(ui.mpHostDriveCombo->
				currentText() + "/")));
		ui.mpHostDrivePathEdit->setText(mpHostPathModel->
			filePath(mpHostProxyModel->mapToSource(ui.mpHostDriveView->
				rootIndex())));
		UpdateGui();
	}

	setCursor(Qt::ArrowCursor);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpHostParentButton_clicked()
{
	Debug("MainWindow::on_mpHostParentButton_clicked()");

	setCursor(Qt::BusyCursor);

	QModelIndex rootIndex = mpHostProxyModel->
		mapToSource(ui.mpHostDriveView->rootIndex());
	if( rootIndex.parent().isValid() )
	{
		ui.mpHostDriveView->clearSelection();
		ui.mpHostDriveView->setRootIndex(mpHostProxyModel->
			mapFromSource(rootIndex.parent()));
		if( ui.mpHostDriveView->isExpanded(mpHostProxyModel->
			mapFromSource(rootIndex)) )
		{
			ui.mpHostDriveView->setExpanded(mpHostProxyModel->
				mapFromSource(rootIndex), false);
		}
		ui.mpHostDrivePathEdit->setText(mpHostPathModel->
			filePath(rootIndex.parent()));
	}
	UpdateFilter();

	setCursor(Qt::ArrowCursor);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpHostDriveView_itemSelectionChanged()
{
	Debug("MainWindow::on_mpHostDriveView_itemSelectionChanged()");

	UpdateGui();
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpHostDriveView_doubleClicked()
{
	Debug("MainWindow::on_mpHostDriveView_doubleClicked()");
	setCursor(Qt::BusyCursor);

	QModelIndex currentIndex = mpHostProxyModel->
		mapToSource(ui.mpHostDriveView->currentIndex());
	if( mpHostPathModel->isDir(currentIndex) )
	{
		ui.mpHostDriveView->clearSelection();
		ui.mpHostDriveView->setRootIndex(mpHostProxyModel->
			mapFromSource(currentIndex));
		ui.mpHostDrivePathEdit->setText(mpHostPathModel->
			filePath(currentIndex));
	}
	UpdateGui();
	setCursor(Qt::ArrowCursor);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//Device Area...
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpDeviceDriveCombo_currentIndexChanged(int index)
{
	Debug("MainWindow::on_mpDeviceDriveCombo_currentIndexChanged()");

	mHumaxController.ChangeDisk('1' + index);

	// Get current directory contents
	Entry *dir = mHumaxController.getCurDeviceDir();

	QString drivePath = ('1' + index) + QString(":/");

	char origdir[MAX_PATH];
	strcpy(origdir, dir->path);
	strcpy(dir->path, drivePath.toStdString().data());
	if(mHumaxController.setDeviceDirectory(dir) == -1)
	{
		Error(QString("Could not change to the directory '%1'!").arg(dir->path));
		strcpy(dir->path, origdir);
		return;
	}

	UpdateDeviceList();
	UpdateFilter();
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpDeviceParentButton_clicked()
{
	Debug("MainWindow::on_mpDeviceParentButton_clicked()");

	// Get current directory contents
	Entry *dir = mHumaxController.getCurDeviceDir();

	QString path(dir->path);
	if( path.right(1) == "/" ) path.resize(path.length() - 1);

	int index = path.lastIndexOf("/");
	if( index > -1 )
	{
		path.resize(index);
		if( path.right(1) != "/" ) path += "/";

		char origdir[MAX_PATH];
		strcpy(origdir, dir->path);
		strcpy(dir->path, path.toStdString().data());
		if(mHumaxController.setDeviceDirectory(dir) == -1)
		{
			Error(QString("Could not change to the directory '%1'!").arg(dir->path));
			strcpy(dir->path, origdir);
			return;
		}

		UpdateDeviceList();
	}

	UpdateFilter();
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpDeviceDriveView_itemSelectionChanged()
{
	Debug("MainWindow::on_mpDeviceDriveView_itemSelectionChanged()");

	UpdateGui();
}

////////////////////////////////////////////////////////////////////////////////
// on_mpDeviceDriveView_doubleClicked()
// bool doChdir(usb_dev_handle * dh);
////////////////////////////////////////////////////////////////////////////////
void MainWindow::on_mpDeviceDriveView_doubleClicked()
{
	Debug("MainWindow::on_mpDeviceDriveView_doubleClicked()");
	setCursor(Qt::BusyCursor);

	if( mpHumaxModel->isDir(ui.mpDeviceDriveView->currentIndex()) )
	{
		// Get current directory contents
		Entry *dir = mHumaxController.getCurDeviceDir();

		QString dirname = mpHumaxModel->fileName(ui.mpDeviceDriveView->currentIndex());
		QString devicePath = mpHumaxModel->getModelPath();
		if( devicePath.right(1) != "/" ) devicePath += "/";
		devicePath += dirname;
		if( devicePath.right(1) != "/" ) devicePath += "/";

		char origdir[MAX_PATH];
		strcpy(origdir, dir->path);
		strcpy(dir->path, devicePath.toStdString().data());
		if(mHumaxController.setDeviceDirectory(dir) == -1)
		{
			Error(QString("Could not change to the directory '%1'!").arg(dir->path));
			strcpy(dir->path, origdir);
			return;
		}

		UpdateDeviceList();
	}

	UpdateFilter();
	setCursor(Qt::ArrowCursor);
}

////////////////////////////////////////////////////////////////////////////////
// HumaxController Slots...
void MainWindow::transferError(QString error)
{
	Error(error);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Other Functions...
////////////////////////////////////////////////////////////////////////////////
/*! \brief Read the application settings.
 *
 * Read the application settings and update the relevant display and
 * transfer variables.
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::readSettings()
{
	Debug("MainWindow::readSettings()");

	QSettings stg("AndySmith", "Humax");
	stg.beginGroup("/Humax");

	QPoint position;
	position.setX(stg.value("/geometry/x",	200).toInt());
	position.setY(stg.value("/geometry/y",	200).toInt());
	QSize size;
	size.setWidth(stg.value("/geometry/width",		700).toInt());
	size.setHeight(stg.value("/geometry/height",	450).toInt());

	QList<int> vertSizes;
	vertSizes << stg.value("/geometry/mainHeight",	350).toInt();
	vertSizes << stg.value("/geometry/logHeight",	50).toInt();

	for( int i = 0; i < ui.mpHostDriveView->header()->count(); ++i )
		ui.mpHostDriveView->header()->resizeSection(i,
			stg.value(QString("/geometry/hostColumnWidth/%1").arg(i), 100).toInt());

	for( int i = 0; i < ui.mpDeviceDriveView->header()->count(); ++i )
		ui.mpDeviceDriveView->header()->resizeSection(i,
			stg.value(QString("/geometry/deviceColumnWidth/%1").arg(i), 100).toInt());

	ui.mpHostDrivePathEdit->setText(stg.value("/host/path", "").toString());

	mHumaxController.downloadBlockSize
		= stg.value("/transfer/downloadblocksize", 160).toInt();
	mHumaxController.uploadPrimaryBlockSize
		= stg.value("/transfer/uploadprimaryblocksize", 240).toInt();
	mHumaxController.uploadRecoveryBlockSize
		= stg.value("/transfer/uploadrecoveryblocksize", 4).toInt();
	mHumaxController.ErrorRecovery(stg.value("/transfer/errorrecovery", true).toBool());
	ui.mpActionFilter->setChecked(stg.value("/transfer/filter", false).toBool());

	mHumaxController.processPriority = stg.value("/process/priority", 3).toInt();

	ui.mpOutputText->setVisible(stg.value("/logging/showwindow", true).toBool());
	mLogOutput	= stg.value("/logging/logoutput",	true).toBool();
	mLogErrors	= stg.value("/logging/logerrors",	false).toBool();
	mLogDebug	= stg.value("/logging/logdebug",	false).toBool();

	mHumaxController.TraceConsole(stg.value("/trace/console", false).toBool());
	mHumaxController.TraceLogFile(stg.value("/trace/logfile", false).toBool());

	stg.endGroup();

    move(position);
	resize(size);
	ui.mpLogSplitter->setSizes(vertSizes);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Store the application settings.
 *
 * Write the application settings from the relevant display and
 * transfer variables.
 */
////////////////////////////////////////////////////////////////////////////////
void MainWindow::writeSettings()
{
	Debug("MainWindow::writeSettings()");

	QSettings stg("AndySmith", "Humax");
	stg.beginGroup("/Humax");

	stg.setValue("/geometry/x", x());
	stg.setValue("/geometry/y", y());
	stg.setValue("/geometry/width",		width());
	stg.setValue("/geometry/height",	height());

	QList<int> vertSizes = ui.mpLogSplitter->sizes();
	stg.setValue("/geometry/mainHeight",	vertSizes[0]);
	stg.setValue("/geometry/logHeight",		vertSizes[1]);

	for( int i = 0; i < ui.mpHostDriveView->header()->count(); ++i )
		stg.setValue(QString("/geometry/hostColumnWidth/%1").arg(i),
			ui.mpHostDriveView->columnWidth(i));

	for( int i = 0; i < ui.mpDeviceDriveView->header()->count(); ++i )
		stg.setValue(QString("/geometry/deviceColumnWidth/%1").arg(i),
			ui.mpDeviceDriveView->columnWidth(i));

	stg.setValue("/host/path",	ui.mpHostDrivePathEdit->text());

	stg.setValue("/transfer/downloadblocksize",			mHumaxController.downloadBlockSize);
	stg.setValue("/transfer/uploadprimaryblocksize",	mHumaxController.uploadPrimaryBlockSize);
	stg.setValue("/transfer/uploadrecoveryblocksize",	mHumaxController.uploadRecoveryBlockSize);
	stg.setValue("/transfer/errorrecovery",				mHumaxController.ErrorRecovery());
	stg.setValue("/transfer/filter",					ui.mpActionFilter->isChecked());

	stg.setValue("/process/priority",	mHumaxController.processPriority);

	stg.setValue("/logging/showwindow",	ui.mpOutputText->isVisible());
	stg.setValue("/logging/logoutput",	mLogOutput);
	stg.setValue("/logging/logerrors",	mLogErrors);
	stg.setValue("/logging/logdebug",	mLogDebug);

	stg.setValue("/trace/console", mHumaxController.TraceConsole());
	stg.setValue("/trace/logfile", mHumaxController.TraceLogFile());

	stg.endGroup();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Delete a host File or Directory.
 *
 * 
 * \param fileName
 * \return 
 */
////////////////////////////////////////////////////////////////////////////////
bool MainWindow::HostDeleteFile(QString fileName)
{
	Debug("MainWindow::HostDeleteFile()");

	QFileInfo fileInfo(fileName);
	if( !fileInfo.exists() )
	{
		return false;
	}

	if( fileInfo.isDir() )
	{
		QDir deleteDir(fileName);

		if( fileName.right(1) != "/" )
			fileName += "/";

		bool deleted = true;
		QStringList entryList = deleteDir.entryList();
		QString index;
		foreach(index, entryList)
		{
			if( index == "." || index == ".." )
				continue;

			if( !HostDeleteFile(fileName + index) )
				deleted = false;
		}

		if( deleted )
		{
			QString dirName = deleteDir.dirName();
			if( deleteDir.cdUp() )
			{
				deleted = deleteDir.rmdir(dirName);
			}
		}
		return deleted;
	}

	return QFile::remove(fileName);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Create a list of files to be copied.
 *
 * 
 * \param basePath		
 * \param hostDirPath	
 * \param devPath		
 * \param downloadList	
 * \param mkDirList		
 * \return 
 */
////////////////////////////////////////////////////////////////////////////////
bool MainWindow::HostCopyFileList(const QString &basePath,
									const QString &hostDirPath,
									const QString &devPath,
									QStringList &downloadList,
									QStringList &mkDirList)
{
	Debug("MainWindow::HostCopyFileList()");

	QStringList filterList = mpHostProxyModel->getFilterList();

	QDir hostDir(basePath + hostDirPath);
	if( hostDir.exists() )
	{
		mkDirList.append(devPath + hostDirPath);

		QString dirPath = hostDirPath;

		if( dirPath.right(1) != "/" )
			dirPath += "/";

		QStringList entryList = hostDir.entryList();
		QString index;
		foreach(index, entryList)
		{
			if( index == "." || index == ".." )
				continue;

			QFileInfo fileInfo(basePath + dirPath + index);
			if( fileInfo.isDir() )
			{
				HostCopyFileList(basePath, dirPath + index, devPath,
					downloadList, mkDirList);
			}
			else
			{
				if( filterList.count() == 0 )
				{
					downloadList.append(dirPath + index);
				}
				else
				{
					QString filterIndex;
					foreach(filterIndex, filterList)
					{
						QString fileType =
							index.right(filterIndex.length()).toLower();
						if( fileType == filterIndex )
						{
							downloadList.append(dirPath + index);
							break;
						}
					}
				}
			}
		}
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
bool MainWindow::DeviceDeleteFile(QString fileName)
{
	Debug("MainWindow::DeviceDeleteFile()");

	int pos = fileName.lastIndexOf('/');
	if( pos == -1 )
	{
		return false;
	}

	Entry dir;
	mHumaxController.initEntry(&dir,
		fileName.left(pos + 1).toStdString().data(), 0, 0);

	if( mHumaxController.setDeviceDirectory(&dir) == -1 )
	{
		mHumaxController.deleteEntryTree(&dir);
		return false;
	}

	Entry *file = dir.firstChild;
	bool found = false;
	while( !found && file != NULL )
	{
		QString name(file->path);

		if( name.right(1) == "/" )
			name.resize(name.length() - 1);

		found = (fileName == name);

		if( !found )
			file = file->sibling;
	}
	if( !found )
	{
		mHumaxController.deleteEntryTree(&dir);
		return false;
	}

	if( mHumaxController.isDirectory(file) )
	{
		if( !mHumaxController.humaxDeleteFile(file->path) )
		{
			mHumaxController.deleteEntryTree(&dir);
			return false;
		}
	}
	else
	{
		if( !mHumaxController.humaxDeleteFile(file->path) )
		{
			mHumaxController.deleteEntryTree(&dir);
			return false;
		}
	}

	mHumaxController.deleteEntryTree(&dir);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
