////////////////////////////////////////////////////////////////////////////////
// \class PrefsDialog PrefsDialog.cpp
//
// \version 1.00
// \date 14-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief Media Controller Preferences Dialog Class.
//
// Preferences dialog to display information about the current
// Transfer and Logging settings.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
14-07-06 1.00 Initial release.

*****/
#include "PrefsDialog.h"

#include "MainWindow.h"

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Preferences Dialog Class.
 */
////////////////////////////////////////////////////////////////////////////////
PrefsDialog::PrefsDialog(QWidget *parent)
: QDialog(parent)
, mDataChanged(false)
{
	ui.setupUi(this);

	if( parent )
	{
		MainWindow *pMain = (MainWindow *)parent;
		if( pMain != 0 )
		{
			// Transfer Tab...
			int downloadBlockSize = pMain->DownloadBlockSize();
			double downloadMaxRate = (12000000.0 / (8192.0 
				+ (downloadBlockSize * 2048.0))) * (downloadBlockSize * 256.0);
			ui.mpDownloadBlockSize->setValue(downloadBlockSize);

			QString text;
			text.sprintf("(%.2f KB)", (downloadBlockSize / 4.0));
			ui.mpBlockRateEdit->setText(text);

			text.sprintf("%.2f KB/Sec", (downloadMaxRate / 1024.0));
			ui.mpDownloadRateEdit->setText(text);

			int uploadPrimaryBlockSize = pMain->UploadPrimaryBlockSize();
			double uploadMaxRate = (12000000.0 / (8192.0
				+ (uploadPrimaryBlockSize * 2048.0))) * (uploadPrimaryBlockSize * 256.0);
			ui.mpUploadPrimaryBlockSize->setValue(uploadPrimaryBlockSize);

			text.sprintf("(%.2f KB)", (uploadPrimaryBlockSize / 4.0));
			ui.mpPrimaryRateEdit->setText(text);

			text.sprintf("%.2f KB/Sec", (uploadMaxRate / 1024.0));
			ui.mpUploadRateEdit->setText(text);

			int uploadRecoveryBlockSize = pMain->UploadRecoveryBlockSize();
			ui.mpUploadRecoveryBlockSize->setValue(uploadRecoveryBlockSize);

			ui.mpErrorRecovery->setCurrentIndex(!pMain->ErrorRecovery());

			text.sprintf("(%.2f KB)", (uploadRecoveryBlockSize / 4.0));
			ui.mpRecoveryRateEdit->setText(text);

			ui.mpProcessPriority->setCurrentIndex(pMain->ProcessPriority());

			// Logging Tab...
			ui.mpShowLogWindowToggle->
				setCheckState((pMain->IsShowingLogWindow()?
					Qt::Checked:Qt::Unchecked));

			ui.mpShowOutputToggle->
				setCheckState((pMain->LogOutput()?Qt::Checked:Qt::Unchecked));
			ui.mpShowErrorsToggle->
				setCheckState((pMain->LogErrors()?Qt::Checked:Qt::Unchecked));
			ui.mpShowDebugToggle->
				setCheckState((pMain->LogDebug()?Qt::Checked:Qt::Unchecked));

			ui.mpTraceConsoleToggle->setCheckState((pMain->
				TraceConsole()?Qt::Checked:Qt::Unchecked));
			ui.mpTraceLogFileToggle->setCheckState((pMain->
				TraceLogFile()?Qt::Checked:Qt::Unchecked));

			mDataChanged = false;
			ui.applyButton->setEnabled(false);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::accept()
{
	if( mDataChanged )
	{
		UpdateSettings();
	}
	QDialog::accept();
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_applyButton_clicked()
{
	if( mDataChanged )
	{
		UpdateSettings();
		mDataChanged = false;
	}
	ui.applyButton->setEnabled(false);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpDownloadBlockSize_valueChanged(int i)
{
	double downloadMaxRate = (12000000.0 / (8192.0 
		+ (i * 2048.0))) * (i * 256.0);

	QString text;
	text.sprintf("(%.2f KB)", (i / 4.0));
	ui.mpBlockRateEdit->setText(text);

	text.sprintf("%.2f KB/Sec", (downloadMaxRate / 1024.0));
	ui.mpDownloadRateEdit->setText(text);

	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpUploadPrimaryBlockSize_valueChanged(int i)
{
	double uploadMaxRate = (12000000.0 / (8192.0
		+ (i * 2048.0))) * (i * 256.0);

	QString text;
	text.sprintf("(%.2f KB)", (i / 4.0));
	ui.mpPrimaryRateEdit->setText(text);

	text.sprintf("%.2f KB/Sec", (uploadMaxRate / 1024.0));
	ui.mpUploadRateEdit->setText(text);

	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpUploadRecoveryBlockSize_valueChanged(int i)
{
	QString text;
	text.sprintf("(%.2f KB)", (i / 4.0));
	ui.mpRecoveryRateEdit->setText(text);

	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpErrorRecovery_currentIndexChanged(int index)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpProcessPriority_currentIndexChanged(int index)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpDefaults_clicked()
{
	ui.mpDownloadBlockSize->setValue(160);
	ui.mpUploadPrimaryBlockSize->setValue(240);
	ui.mpUploadRecoveryBlockSize->setValue(4);

	ui.mpErrorRecovery->setCurrentIndex(0);

	ui.mpProcessPriority->setCurrentIndex(3);

	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpShowLogWindowToggle_stateChanged(int state)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);

	if( ui.mpShowLogWindowToggle->checkState() == Qt::Unchecked )
	{
		ui.mpShowOutputToggle->setEnabled(false);
		ui.mpShowErrorsToggle->setEnabled(false);
		ui.mpShowDebugToggle->setEnabled(false);
	}
	else
	{
		ui.mpShowOutputToggle->setEnabled(true);
		ui.mpShowErrorsToggle->setEnabled(true);
		ui.mpShowDebugToggle->setEnabled(true);
	}
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpShowOutputToggle_stateChanged(int state)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpShowErrorsToggle_stateChanged(int state)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpShowDebugToggle_stateChanged(int state)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpTraceConsoleToggle_stateChanged(int state)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::on_mpTraceLogFileToggle_stateChanged(int state)
{
	mDataChanged = true;
	ui.applyButton->setEnabled(true);
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
void PrefsDialog::UpdateSettings()
{
	MainWindow *pMain = (MainWindow *)parentWidget();
	if( pMain != 0 )
	{
		// Transfer Tab...
		pMain->SetTransferBlockSizes(
			ui.mpDownloadBlockSize->value(),
			ui.mpUploadPrimaryBlockSize->value(),
			ui.mpUploadRecoveryBlockSize->value());
		
		pMain->ErrorRecovery(!ui.mpErrorRecovery->currentIndex());

		pMain->ProcessPriority(ui.mpProcessPriority->currentIndex());

		// Logging Tab...
		if( ui.mpShowLogWindowToggle->checkState() == Qt::Unchecked )
		{
			pMain->ShowLogWindow(false);
		}
		else
		{
			pMain->ShowLogWindow(true);
		}
		pMain->SetLoggingFlags(
			(ui.mpShowOutputToggle->checkState() == Qt::Unchecked ? false : true),
			(ui.mpShowErrorsToggle->checkState() == Qt::Unchecked ? false : true),
			(ui.mpShowDebugToggle->checkState() == Qt::Unchecked ? false : true));

		pMain->SetTraceFlags(
			(ui.mpTraceConsoleToggle->checkState() == Qt::Unchecked ? false : true),
			(ui.mpTraceLogFileToggle->checkState() == Qt::Unchecked ? false : true));
	}
}

////////////////////////////////////////////////////////////////////////////////
