////////////////////////////////////////////////////////////////////////////////
// \class QHostProxyModel QHostProxyModel.h
//
// \version 1.00
// \date 11-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief Class to enable filtering of the Host Directory View.
//
// A Proxy Model class to enable the filtering of the host directory
// model/view implementation. Used to filter file types while still leaving
// the directories visible. Usual filters are '.jpg', '.mp3', and '.ts'.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
11-07-06 1.00 Initial release.

*****/
#include "QHostProxyModel.h"

#include <QDirModel>

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Host Filter Proxy Model Class.
 */
////////////////////////////////////////////////////////////////////////////////
QHostProxyModel::QHostProxyModel(QObject *parent)
: QSortFilterProxyModel(parent)
, mFilterList()
{
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Decides whether a row should be filtered based on the list.
 *
 * Filters the given row based on the list supplied to the proxy.
 * Automatically called from the model/view when updating the display.
 * \param source_row	The row of the model that the filter is to
 *						be applied.
 * \param source_parent	The parent model to get the filter data.
 * \return Boolean, TRUE displays the row, FALSE filters it out.
 * \sa setFilterList() and mFilterList.
 */
////////////////////////////////////////////////////////////////////////////////
bool QHostProxyModel::filterAcceptsRow(int source_row,
										const QModelIndex &source_parent) const
{
	QDirModel *pSourceModel = (QDirModel *)sourceModel();
	QModelIndex index = pSourceModel->index(source_row, 0, source_parent);

	if( pSourceModel->isDir(index) )
	{
		return TRUE;
	}

	if( mFilterList.count() == 0 )
	{
		return TRUE;
	}


	QString filterIndex;
	foreach( filterIndex, mFilterList )
	{
		QString fileType = pSourceModel->
			data(index).toString().right(filterIndex.length()).toLower();
		if( fileType == filterIndex )
		{
			return TRUE;
		}
	}

	return FALSE;
}

////////////////////////////////////////////////////////////////////////////////
