////////////////////////////////////////////////////////////////////////////////
// \class TransferDialog TransferDialog.cpp
//
// \version 1.00
// \date 05-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief Media Controller Data Transfer Dialog Class.
//
// Data Transfer dialog to display information about the current
// upload/download transfer to/from the humax pvr9200t device.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
05-07-06 1.00 Initial release.

*****/
#include "TransferDialog.h"

#include "MainWindow.h"
#include "FileReplaceDialog.h"

const QString TransferDialog::mcMB(" MB");
const QString TransferDialog::mcKBPS(" KB/s");

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 *
 * Construct and initialise the data in the Transfer Dialog Class.
 */
////////////////////////////////////////////////////////////////////////////////
TransferDialog::TransferDialog(QWidget *parent)
: QDialog(parent, Qt::WindowTitleHint)
, mMode(DOWNLOAD)
, mFileCount(0)
, mCurrentFile("")
{
	ui.setupUi(this);

	ui.mpUploadFrame->hide();
	resize(QSize(570, 190).expandedTo(minimumSizeHint()));
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Dialog transfer mode set accessor.
 *
 * Set up the dialogs display mode for either upload or download.
 * Default is download.
 * \param mode The display mode to be shown.
 * \sa TransferMode.
 */
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::setTransferMode(TransferMode mode)
{
	mMode = mode;
	if( mode == DOWNLOAD )
	{
		ui.mpDownloadFrame->show();
		ui.mpUploadFrame->hide();

		resize(QSize(570, 190).expandedTo(minimumSizeHint()));
	}
	else
	{
		ui.mpDownloadFrame->hide();
		ui.mpUploadFrame->show();

		resize(QSize(650, 200).expandedTo(minimumSizeHint()));
	}
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called by the controller when a file transfer is started.
 *
 * Called by the controller transfer thread to indicate when it has
 * started downloading a file.
 * \param id		The index of the file download.
 * \param fileName	The filename of the download.
 * \sa transferComplete() and mFileCount.
 */
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::transferStarted(int id, QString fileName)
{
	mCurrentFile = fileName;

	ui.mpFileProgressLabel->setText("File Progress (" + fileName + "):");
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Displays upload progess on the dialog.
 *
 * Updates the upload progress information in the
 * dialog and the progress bar.
 * \param percent		The percentage the transfer is complete.
 * \param source		The type of transfer, EG transfer, chase, recovery.
 * \param good			The number of good bytes transferred.
 * \param recovered		The number of bytes recovered.
 * \param bad			The number of bad bytes transferred.
 * \param total			The total number of bytes transferred.
 * \param rate			The transfer rate of the data.
 * \param elapsedSecs	The number of seconds that have elapsed since the
 *						start of the transfer.
 * \param remainSecs	The number of seconds (approx) remaining to the
 *						end of the transfer.
 */
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::uploadProgress(int percent, QString source,
									double good, double recovered,
									double bad, double total, double rate,
									int elapsedSecs, int remainSecs)
{
	ui.mpFileProgress->setValue(percent);

	ui.mpFileProgressLabel->setText(source +
		tr(" Progress (") + mCurrentFile + tr("):"));

	ui.mpGoodBytesEdit->setText(QString::number(good, 'f', 3) + mcMB);

	if( recovered < 0.0 )
	{
		ui.mpRecoveredBytesLabel->hide();
		ui.mpRecoveredBytesEdit->hide();
	}
	else
	{
		ui.mpRecoveredBytesEdit->setText(QString::number(recovered, 'f', 3) + mcMB);

		ui.mpRecoveredBytesLabel->show();
		ui.mpRecoveredBytesEdit->show();
	}

	ui.mpBadBytesEdit->setText(QString::number(bad, 'f', 3) + mcMB);

	ui.mpTotalBytesEdit->setText(QString::number(total, 'f', 3) + mcMB);

	if( rate < 0.0 )
	{
		ui.mpURateLabel->hide();
		ui.mpURateEdit->hide();
	}
	else
	{
		ui.mpURateEdit->setText(QString::number(rate, 'f', 2) + mcKBPS);

		ui.mpURateLabel->show();
		ui.mpURateEdit->show();
	}

	QString text;
	text.sprintf("%d:%02d:%02d",
		elapsedSecs / 3600, elapsedSecs / 60 % 60, elapsedSecs % 60);
	ui.mpUElapsedTimeEdit->setText(text);

	// Could be -1
	text.sprintf("%d:%02d:%02d",
		remainSecs / 3600, remainSecs / 60 % 60, remainSecs % 60);
	ui.mpURemainingTimeEdit->setText(text);

	update();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Displays download progess on the dialog.
 *
 * Updates the download progress information in the
 * dialog and the progress bar.
 * \param percent		The percentage the transfer is complete.
 * \param transferred	The number of bytes transferred.
 * \param remaining		The number of bytes yet to be transferred.
 * \param rate			The transfer rate of the data.
 * \param elapsedSecs	The number of seconds that have elapsed since the
 *						start of the transfer.
 * \param remainSecs	The number of seconds (approx) remaining to the
 *						end of the transfer.
 */
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::downloadProgress(int percent, double transferred,
									  double remaining, double rate,
									  int elapsedSecs, int remainSecs)
{
	ui.mpFileProgress->setValue(percent);

	ui.mpBytesTransferredEdit->setText(QString::number(transferred, 'f', 3) + mcMB);

	ui.mpBytesRemainingEdit->setText(QString::number(remaining, 'f', 3) + mcMB);

	ui.mpDRateEdit->setText(QString::number(rate, 'f', 2) + mcKBPS);

	QString text;
	text.sprintf("%d:%02d:%02d",
		elapsedSecs / 3600, elapsedSecs / 60 % 60, elapsedSecs % 60);
	ui.mpDElapsedTimeEdit->setText(text);

	text.sprintf("%d:%02d:%02d",
		remainSecs / 3600, remainSecs / 60 % 60, remainSecs % 60);
	ui.mpDRemainingTimeEdit->setText(text);

	update();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Indicates the transfer of a file is complete.
 *
 * At the end of each file transfer this slot function is called to
 * notify the transfer dialog of the completion in order to update the
 * progress bars.
 * \param id	The index of the file that has finished transferring.
 */
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::transferComplete(int id)
{
	int percent = int((double(id + 1) / double(mFileCount))
				* double(100.0));
	ui.mpTotalProgress->setValue(percent);

	MainWindow *pParent = reinterpret_cast<MainWindow *>(parentWidget());
	if( pParent )
	{
		QString logString;
		if( mMode == DOWNLOAD )
		{
			logString += "Downloaded " + mCurrentFile + ". "
					+ ui.mpBytesTransferredEdit->text() + " transferred in "
					+ ui.mpDElapsedTimeEdit->text() + " at "
					+ ui.mpDRateEdit->text();
		}
		else
		{
			logString += "Uploaded " + mCurrentFile + ". "
						+ ui.mpTotalBytesEdit->text() + " transferred in "
						+ ui.mpUElapsedTimeEdit->text() + " at "
						+ ui.mpURateEdit->text();
		}

		pParent->Log(logString);
	}
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::transferOverwrite(QString name, QString targetSize,
									   QString sourceSize, bool single)
{
	FileReplaceDialog dlg(this);
	dlg.SetSingleFileReplace(single);
	dlg.SetFilename(name);
	dlg.SetTargetInfo(targetSize);
	dlg.SetSourceInfo(sourceSize);

	int retVal = dlg.exec();
	emit transferOverwriteReturn(retVal);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called if the user presses the cancel button.
 *
 * This function is called when the user presses the cancel button in
 * order to abort the transfer. The function will notify the transfer
 * process thread to cancel and then wait for it to finish.
 */
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::on_cancelButton_clicked()
{
	ui.cancelButton->setEnabled(false);
	emit transferCancelled();
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Called when the Close [x] buttons is pressed.
 *
 * This event is raise when the windows close [x] button is pressed.
 * Note that the close button should be removed from the window.
 * \param event	The close event information.
 */
////////////////////////////////////////////////////////////////////////////////
void TransferDialog::closeEvent(QCloseEvent *event)
{
	emit transferCancelled();
}

////////////////////////////////////////////////////////////////////////////////
