////////////////////////////////////////////////////////////////////////////////
// \class UsbInterface UsbInterface.cpp
//
// \version 1.00
// \date 04-07-2006
// \author Andrew Smith <andrew.smith@freenet.co.uk>
//
// \par Copyright:
// See COPYING file that comes with this distribution.
//
// \brief USB Interface Class.
//
// Low Level USB interface class used to communicate with libusb.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
///// CHANGE HISTORY                                                       /////
////////////////////////////////////////////////////////////////////////////////
/*****

Date:     Ver: Description:
-------- ---- ------------------------------------------------------------------
04-07-06 1.00 Initial release.

*****/
#include "UsbInterface.h"

#include <string.h>

/*!
 * Handle to the USB device.
 */
struct usb_device *mpHumaxDevice;

////////////////////////////////////////////////////////////////////////////////
///// OTHER DEFINES                                                        /////
////////////////////////////////////////////////////////////////////////////////

/*!
 * File name of the USB data trace log file.
 */
#define USB_LOG_FILE			"humax_usb.txt"

////////////////////////////////////////////////////////////////////////////////
///// TRACE FUNCTION                                                       /////
////////////////////////////////////////////////////////////////////////////////

/*!
 * Output USB data trace if switched on.
 * \sa usbTrace()
 */
#define USB_TRACE(status, func, id, msg, data, dataLen) if(usbDataTraceConsole || usbDataTraceLogFile) \
			usbTrace(usbDataTraceConsole, usbDataTraceLogFile, status, func, id, msg, data, dataLen);

////////////////////////////////////////////////////////////////////////////////
/*! \brief Constructor.
 * Construct and initialise the data in the USB Interface Class.
 */
UsbInterface::UsbInterface()
: mpUsbHandle(NULL)
, uploadErrorRecovery(true)	// Default: error-detection Enabled.
, usbDataTraceConsole(false)	// Default: Console trace Disabled.
, usbDataTraceLogFile(false)	// Default: Log file trace Disabled.
{
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Destructor.
 * Clean up any memory usage and destroy the class.
 */
UsbInterface::~UsbInterface()
{
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Read the device status block.
 * Reads a Bulk Transfer "CSW" status block from the device via USB.
 * \param functionName	String name of calling function.
 * \param checkpointID	Checkpoint ID within the calling function.
 * \param msg			General display message.
 * \param buf			Data buffer into which to read the USB data from
 *						the device.
 * \return true - operation succeeded; false - operation failed.
 */
bool UsbInterface::readBulkCSW(char *functionName, int checkpointID,
								char *msg, char *buf)
{
	// Read USB
	memset(buf, 0, 13);
	int cbRet = usb_bulk_read(mpUsbHandle, 0x81, buf, 13, 1000);
	if( cbRet != 13 )
	{
		USB_TRACE(false, functionName, checkpointID, msg, buf, 13);
        return false;
	}
	USB_TRACE(true, functionName, checkpointID, msg, buf, 13);

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Reads a device data block.
 * Reads a Bulk Transfer data block from the device via USB and returns
 * the number of bytes read.
 * \param functionName	String name of calling function.
 * \param checkpointID	Checkpoint ID within calling function.
 * \param msg			General display message.
 * \param buf			Output data buffer into which to read the USB
 *						data from the device.
 * \param cb			Expected length of received USB data.
 * \return The actual number of bytes received.
 */
int UsbInterface::readBulkData(char *functionName, int checkpointID,
								char *msg, char *buf, int cb)
{
	// Read USB
	memset(buf, 0, cb);
	int cbRet = usb_bulk_read(mpUsbHandle, 0x81, buf, cb, 1000);
	if( cbRet != cb )
	{
		USB_TRACE(false, functionName, checkpointID, msg, buf, cb);
		return cbRet;
	}
	USB_TRACE(true, functionName, checkpointID, msg, buf, cb);

	// Return no bytes read
	return cbRet;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Write a device command block.
 * Writes a Bulk Transfer "CBW" command block to the device via USB.
 * \param functionName				Name of calling function.
 * \param checkpointID				Checkpoint ID withing calling function.
 * \param msg						General display message.
 * \param dCBWTag					Unique USB transaction TagID value,
 *									different for each USB operation.
 * \param dCBWDataTransferLength	Expected data size transfer length.
 * \param bmCBWFlags				USB directional flags (0x00 = Host to
 *									Humax, 0x80 = Humax to Host).
 * \param CBWCB						USB CBW (Command Block) data buffer
 *									(excluding the CBW header, just the
 *									data part).
 * \param bCBWCBLength				Length of Command Block data (excluding
 *									the CBW header).
 * \return true - operation succeeded; false - operation failed.
 */
bool UsbInterface::writeBulkCBW(char *functionName, int checkpointID,
								char *msg, int dCBWTag,
								int dCBWDataTransferLength, int bmCBWFlags,
								char *CBWCB, int bCBWCBLength)
{
	// Setup CBW buffer
	char cbw[31];
	memcpy(&cbw[0], "USBC", 4);
	cbw[4] = (unsigned char)(dCBWTag & 0xff);
	cbw[5] = (unsigned char)((dCBWTag >> 8) & 0xff);
	cbw[6] = (unsigned char)((dCBWTag >> 16) & 0xff);
	cbw[7] = (unsigned char)((dCBWTag >> 24) & 0xff);
	cbw[8] = (unsigned char)(dCBWDataTransferLength & 0xff);
	cbw[9] = (unsigned char)((dCBWDataTransferLength >> 8) & 0xff);
	cbw[10] = (unsigned char)((dCBWDataTransferLength >> 16) & 0xff);
	cbw[11] = (unsigned char)((dCBWDataTransferLength >> 24) & 0xff);
	cbw[12] = (unsigned char)(bmCBWFlags & 0xff);
	cbw[13] = 0x01;  // Logical Unit Number (LUN), always 1
	cbw[14] = (unsigned char)(bCBWCBLength & 0xff);
	memcpy(&cbw[15], CBWCB, bCBWCBLength);

	// Write USB
	if(usb_bulk_write(mpUsbHandle, 0x01, &cbw[0], bCBWCBLength + 15, 1000) < 0)
	{
		USB_TRACE(false, functionName, checkpointID, msg, cbw, bCBWCBLength + 15);
		return false;
	}
	USB_TRACE(true, functionName, checkpointID, msg, cbw, bCBWCBLength + 15);

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Write a device data block.
 * Writes a Bulk Transfer data block to the device via USB.
 * \param functionName	String name of calling function.
 * \param checkpointID	Checkpoint ID for location within the calling 
 *						function.
 * \param msg			General display message.
 * \param buf			Data buffer containing data to be sent over USB
 *						to the device.
 * \param len			Length of data to send
 * \return true - operation succeeded; false - operation failed.
 */
bool UsbInterface::writeBulkData(char *functionName, int checkpointID,
								char *msg, char *buf, int len)
{
	// Write USB
	if(usb_bulk_write(mpUsbHandle, 0x01, buf, len, 1000) < 0)
	{
		USB_TRACE(false, functionName, checkpointID, msg, buf, len);
		return false;
	}
	USB_TRACE(true, functionName, checkpointID, msg, buf, len);

	// Return success
	return true;
}

////////////////////////////////////////////////////////////////////////////////
///// USB TRACING FUNCTIONS                                                /////
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/*! \brief Display USB trace.
 * Display a USB transaction data trace
 * \param usbDataTraceConsole	True if output to console required.
 * \param usbDataTraceLogFile	True if output to log file required.
 * \param status				True if operation succeeded, false if failed.
 * \param func					String name of function.
 * \param id					Checkpoint ID within function.
 * \param msg					Informational message string to be displayed.
 * \param data					USB data to be displayed.
 * \param dataLen				Length of USB data to be displayed.
 * \return None (void).
 */
void UsbInterface::usbTrace(bool usbDataTraceConsole, bool usbDataTraceLogFile,
							bool status, char *func, int id, char *msg,
							char *data, int dataLen)
{
	// Open log file if requested
	char buf[128];
	FILE * logFile = NULL;
	if(usbDataTraceLogFile)
		logFile = fopen(USB_LOG_FILE, "at");

	// Display trace header
	sprintf(buf, "[USB %s Function=%s() CheckpointID=%02d DataLen=%d]\n   %s\n", (status ? "__PASS__" : "**FAIL**"), func, id, dataLen, msg);
	writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);

	// Loop to display beginning and end of data (first pass displays up to the first 256 bytes of data and the
	// second pass displays the last 256 bytes of data)
	int p = 0;
	for(int a = 0; a < (dataLen > 256 ? 2 : 1); a++)
	{
		// Loop for up to 16 lines (256 bytes) of data
		for(int b = 0; b < 16 && p < dataLen; b++)
		{
			// Loop for 16 bytes of data
			sprintf(buf, "      %04x:", p);
			writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
			for(int c = 0; c < 16 && p < dataLen; c++)
			{
				// Display hex bytes
				if(p + c < dataLen)
				{
					sprintf(buf, " %02x", (unsigned char)data[p + c]);
					writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
				}
				else
				{
					sprintf(buf, "   ");
					writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
				}
			}

			// Loop for 16 characters
			sprintf(buf, "  ");
			writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
			for(int c = 0; c < 16 && p < dataLen; c++)
			{
				// Display character data
				if(p + c < dataLen)
				{
					sprintf(buf, "%c", (data[p + c] >= 32 && data[p + c] <= 126 ? data[p + c] : '.'));
					writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
				}
				else
				{
					sprintf(buf, " ");
					writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
				}
			}
			sprintf(buf, "\n");
			writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
			p = p + 16;
		}

		// If still more data available then display "....." separator ready to display last 256 bytes of data
		// on the second pass through the main loop
		if(a == 0 && dataLen > 256)
		{
			p = ((dataLen + 15) / 16) * 16 - 256;
			if(p < 256)
				p = 256;
			else
			{
				sprintf(buf, "            .....\n");
				writeTraceInfo(logFile, usbDataTraceConsole, usbDataTraceLogFile, buf);
			}
		}
	}

	// Close log file if necessary
	if(logFile != NULL)
		fclose(logFile);
}

////////////////////////////////////////////////////////////////////////////////
/*! \brief Output USB trace.
 * Display USB transaction trace info to console and/or log file
 * \param logFile				File handle to log file.
 * \param usbDataTraceConsole	True if output to console requested.
 * \param usbDataTraceLogFile	True if output to log file requested.
 * \param buf					Buffer containing data to be
 * 								displayed / written.
 * \return None (void).
 */
void UsbInterface::writeTraceInfo(FILE *logFile, bool usbDataTraceConsole,
									bool usbDataTraceLogFile, char *buf)
{
	// Display to console if requested
	if(usbDataTraceConsole)
		printf("%s", buf);

	// Write to log file if requested
	if(usbDataTraceLogFile && logFile != NULL)
	{
		int len = strlen(buf);
		if(buf[len - 1] == 0x0d)
			buf[len] = 0x0a;
		fwrite(buf, 1, len + (buf[len - 1] == 0x0d ? 1 : 0), logFile);
	}
}

////////////////////////////////////////////////////////////////////////////////
